//    ATTN:
//     _____              _ _     _    _            _______ _     _
//    |  __ \            ( ) |   | |  | |          |__   __| |   (_)
//    | |  | | ___  _ __ |/| |_  | |  | |___  ___     | |  | |__  _ ___
//    | |  | |/ _ \| '_ \  | __| | |  | / __|/ _ \    | |  | '_ \| / __|
//    | |__| | (_) | | | | | |_  | |__| \__ \  __/    | |  | | | | \__ \
//    |_____/ \___/|_| |_|  \__|  \____/|___/\___|    |_|  |_| |_|_|___/
//
//    This module is not finalized. The functionality contained herein
//    is not finalized. Everything in this file will change in future
//    releases. If you use this module or code, you are guaranteeing
//    the code you write will break in future releases.
//
//    To reiterate: DO NOT USE THIS MODULE.
//                  DO NOT USE THIS CODE.
//

/* global console */
/* global require */
/* global define */

/**
 * This Module registers legacy EntityView functions and variables on the global Javascript scope.
 * Any methods accessed through here should log a message to the console.
 * EntityView Deprecated Module
 * @module EntityView/Deprecated
 */
define(
    [
        "module",
        "PortalTools",
        "DebugUtil",
        "global"
    ],
    function(
        module,
        PortalTools,
        DebugUtil,
        globals
    ) {
        "use strict";
//alert("Deprecated A");

        var DEBUG = new DebugUtil(module.id);

        DEBUG.log("Setting up global variables.");

        var global = window;
        var view,
            navigation;

        /**
         * Container only for: onSubmit, submitInProgress, projectEditorSubmitMode
         * Developers may use (but not redefine) the members of EntityView.
         * @type {Object}
         * @readonly
         * @global
         * @deprecated since v6.2
         */
        if(!global.EntityView) {
            global.EntityView = {};
        }

        ///**
        // * Events may define "done" and "then" handlers, while "fail"
        // * handlers are not currently used.
        // * @type {PortalTools.EventType}
        // * @readonly
        // * @global
        // * @deprecated since v6.2
        // */
        //global.EntityView.onSubmit = null;

        /**
         * @type {null|boolean}
         * @readonly
         * @global
         * @deprecated since v6.2
         */
        global.EntityView.submitInProgress = null;

        /**
         * Read-only
         * This will be null if the entity view is not displayed in the
         * Project Editor.
         *
         * Valid values:
         *   null,
         *   'view',
         *   'view-save',
         *   'wizard-close'
         *   'wizard-close-nosave',
         *   'wizard-close-save',
         *   'wizard-continue,
         *   'wizard-exit',
         *   'wizard-save',
         *   'wizard-save-redirect',
         *   'wizard-save-showframe',
         *   'wizard-save-bustframe'
         * @type {null|string}
         * @readonly
         * @global
         * @deprecated since v6.2
         */
        global.EntityView.projectEditorSubmitMode = null;

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Navigation
        //

        /**
         * @global
         * @deprecated since v6.2
         */
        global.submitForm = globals.getFunction(function(strSubmitMode){
            navigation.navigate({mode:strSubmitMode});
        }, "submitForm", "navigationModuleInstance.navigate({mode:strSubmitMode})");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.submitInProgress = globals.getFunction(function(){
            return view.isNavigating();
        }, "submitInProgress", "viewModuleInstance.isNavigating()");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickBack = globals.getFunction(function(){
            navigation.navigate({
                mode:"back",
                validate: false,
                submissionFunction: function() {
                    global.history.go(-1);
                }
            });
        }, "onClickBack", "navigationModuleInstance.navigate, or document.history.go(-1)");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickSave = globals.getFunction(function() {
            navigation.navigate({
                mode:"wizard-save",
                validate: false
            });
        }, "onClickSave", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickExit = globals.getFunction(function(){
            navigation.navigate({
                modeFunction: function() {
                    if (view.isReadOnly()) {
                        return 'wizard-close-nosave';
                    } else if (view.isDirty()) {
                        return 'loadExitDialog';
                    } else {
                        return 'wizard-close-nosave';
                    }
                },
                validate: false
            });
        }, "onClickExit", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickContinue = globals.getFunction(function(){
            navigation.navigate({
                mode:"wizard-continue",
                validate: true
            });
        }, "onClickContinue", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickFinish = globals.getFunction(function(){
            navigation.navigate({
                mode:"wizard-close",
                validate: true
            });
        }, "onClickFinish", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickOK = globals.getFunction(function(){
            navigation.navigate({
                mode:"view",
                validate: true
            });
        }, "onClickOK", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickReadonlyOk = globals.getFunction(function(){
            navigation.navigate({
                mode:"view", // TODO: not necessary
                validate: false,
                submissionFunction: function() {
                    global.location.assign(globals.get("returnUrl"));
                }
            });
        }, "onClickReadonlyOk", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickCancel = globals.getFunction(function(){
            navigation.navigate({
                mode:"cancel",
                validate: false,
                submissionFunction: function() {
                    global.location.assign(globals.get("returnUrl"));
                }
            });
        }, "onClickCancel", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.onClickApply = globals.getFunction(function(){
            navigation.navigate({
                mode:"view-save",
                validate: true
            });
        }, "onClickApply", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.saveAndRedirect = globals.getFunction(function(action, destUrl){
            var strSubmitMode,
                strRedirectUrl,
                bustFrame = false;

            switch(action) {
                case "jumpToView":
                    strSubmitMode = "wizard-save-redirect";
                    strRedirectUrl = "ProjectEditor?" +
                    ( globals.get("mode")              ? "Mode="               + globals.get("mode")              : "" ) +
                    ( globals.get("project")           ? "&Project="           + globals.get("project")           : "" ) +
                    ( globals.get("viewOID")           ? "&ViewOID="           + globals.get("viewOID")           : "" ) +
                    ( globals.get("projectEditorView") ? "&ProjectEditorView=" + globals.get("projectEditorView") : "" ) +
                    ( destUrl                          ? "&DestURL="           + destUrl                          : "" ) +
                    ( globals.get("wizardPageOID")     ? "&WizardPageOID="     + globals.get("wizardPageOID")     : "" ) +
                    ( globals.get("sectionOID")        ? "&SectionOID="        + globals.get("sectionOID")        : "" );
                    break;
                //case "showHideErrors":
                //    if (window.top !== window.self) {
                //        strSubmitMode = "wizard-save-bustframe";
                //        strRedirectUrl = window.self.location;
                //    } else {
                //        strSubmitMode = "wizard-save-showframe";
                //        strRedirectUrl = globals.get("editorWithValidationUrl");
                //    }
                //    break;
                //MB 7/6/07: Added to support ValidateProject.redirectOpener(theUrl)
                case "bustFrameRedirect":
                    //strSubmitMode = "wizard-save-bustframe";
                    strSubmitMode =  "wizard-save-redirect";
                    strRedirectUrl = destUrl ? destUrl : global.self.location;
                    bustFrame = true;
                    break;
                case "goToUrl":
                    strSubmitMode = "wizard-save-redirect";
                    strRedirectUrl = destUrl;
                    break;
                default:
                    throw new Error("Deprecated code only allowed for: jumpToView, showHideErrors, bustFrameRedirect, goToUrl. Use navigation. instead.");
            }
			
			if (Mode == 'view'){  //Mode is written out as a client side var by the prerender on this page
				strSubmitMode = strSubmitMode.replace(/^wizard-/, "view-");
			}
	
            navigation.navigate({
                mode : strSubmitMode,
                redirectUrl : strRedirectUrl,
                validate : false,
                bustFrame : bustFrame
            });
        }, "saveAndRedirect", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.exitWarn = globals.getFunction(function(){
            //if (confirm("If you have not saved changes to the current form, they will be lost.\nClick Cancel to return to your form and click Save before exiting.\nClick OK to close the form without saving changes.")) {
            //return submitForm('wizard-close-nosave', '');
            return navigation.navigate({
                mode: "loadExitDialog",
                validate: false
            });
        }, "exitWarn", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.loadExitDialog = globals.getFunction(function() {
            navigation.navigate({
                mode: "loadExitDialog",
                validate: false
            });
        }, "loadExitDialog", "navigationModuleInstance.navigate");

        /**
         * @global
         * @deprecated since v6.2
         */
            // Feedback stuff
        global.resetFeedback = globals.getFunction(function(){
            navigation.navigationBusyIndicator.stop();
        }, "resetFeedback", "navigationModuleInstance.navigationBusyIndicator.stop()");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.busyImageHandler = globals.getFunction(function(){
            navigation.navigationBusyIndicator.start();
        }, "busyImageHandler", "navigationModuleInstance.navigationBusyIndicator.start()");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.feedBackHandler = globals.getFunction(function(){
            navigation.navigationBusyIndicator.start();
        }, "feedBackHandler", "navigationModuleInstance.navigationBusyIndicator.start()");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.clearBusyImageHandler = globals.getFunction(function(){
            navigation.navigationBusyIndicator.stop();
        }, "clearBusyImageHandler", "navigationModuleInstance.navigationBusyIndicator.stop()");

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        // utility functions
        //

        /**
         * Create a cookie with the specified name and value.
         * The cookie expires at the end of dtExpires.
         * name - name of the cookie
         * value - value of the cookie
         * [expires] - expiration date of the cookie (defaults to end of current session) // todo default is a lie
         * [path] - path for which the cookie is valid (defaults to path of calling document)
         * [domain] - domain for which the cookie is valid (defaults to domain of calling document)
         * [secure] - Boolean value indicating if the cookie transmission requires a secure transmission
         *   * an argument defaults when it is assigned null as a placeholder
         *   * a null placeholder is not required for trailing omitted arguments<FONT color=#0000cc>
         * @global
         * @deprecated since v6.2
         */
        global.setCookie = globals.getFunction(function(name, value, expires, path, domain, secure) {
            var now = new Date();
            //var milliseconds = expires.getTime() - now.getTime();
            var days = (expires) ? (expires.getTime() - now.getTime() / (1000*60*60*24)) : null;
            return PortalTools.createCookie(name, value, days, path, domain, secure);
        }, "setCookie", "PortalTools.createCookie(name, value, days, path, domain, secure)");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.getCookie =
            global.getViewUrl =
                globals.getFunction(function(cookieName) {
                    return PortalTools.readCookie(cookieName);
                }, "getCookie", "PortalTools.readCookie(cookieName)");

        /**
         * @global
         * @deprecated since v6.2
         */
        global.printView = globals.getFunction(function() {
            var commandBars = $("#CommandBarTop, #CommandBarBottom");
            commandBars.css("display","none");
            window.print();
            commandBars.css("display","block");
        }, "printView");

        ///**
        // * @global
        // * @deprecated since v6.2
        // */
        //global.clearRadioButtons = globals.getFunction(function(formID, fieldName) {
        //    view.getProperty(fieldName).clear();
        //}, "clearRadioButtons", 'viewInstance.getProperty("fullyQualifiedName").clear()');

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Obsolete functions
        //

        /**
         * No longer functioning since 6.2
         * Will be removed in 7.0
         * @global
         * @deprecated since v6.2
         */
        global.doSafariCDCachingWorkaround = globals.obsoleteFunction("doSafariCDCachingWorkaround");

        /**
         * No longer functioning since 6.2
         * Will be removed in 7.0
         * @global
         * @deprecated since v6.2
         */
        global.isSafari = globals.obsoleteFunction("isSafari");

        /**
         * No longer functioning since 6.2
         * Will be removed in 7.0
         * @global
         * @deprecated since v6.2
         */
        global.wipeSubmittedVarForOpera = globals.obsoleteFunction("wipeSubmittedVarForOpera");

        /**
         * No longer functioning since 6.2
         * Will be removed in 7.0
         * @global
         * @deprecated since v6.2
         */
        global.form_onSubmitHandler = globals.obsoleteFunction("form_onSubmitHandler");

        /**
         * No longer functioning since 6.2
         * Will be removed in 7.0
         * @global
         * @deprecated since v6.2
         */
        global.setupLoadExitDialog = globals.obsoleteFunction("setupLoadExitDialog");

//alert("Deprecated B");

        return {
            registerView : function(viewObject) {
                view = viewObject;
            },
            registerNavigation : function(navigationObject) {
                navigation = navigationObject;
            }
        };

    });
