
/*
* Support for drag and drop file upload for document component or document set view control
*/

//main document uploader object
//manages file uploads for all controls on the page
function documentUploader()
{
	this.fileProgressTable = document.getElementById('filesProgress');
	this.fileProgressTableBody;
	this.filesBeingUploaded = [];
	this.totalFileCount = 0;
	this.refreshAfterFileUpload = false;
	this.temp_webrUnencodedUrl = _webrUnencodedUrl;
	this.dragDropDocumentControls = {};
	this.rowsToRemove = [];
	this.activeDropZone;
	
	//Get rid of any files that don't have a URL defined for uploading
	this.clearUnwantedFiles = function() {
		for (var i = this.filesBeingUploaded.length - 1; i >= 0; i--) {
			if (this.filesBeingUploaded[i].url == null || this.filesBeingUploaded[i].url == "")
				this.filesBeingUploaded.splice(i, 1);
		}
	};
}


if (!window.documentUploaderLoaded) {

	$("<table id='filesProgress' class='fileUploadProgressTable' style='width:640px;'></table>").appendTo(document.body);
	
	//create a document uploader object
	var mainDocumentUploader = new documentUploader();
	//on page load, hide some of the divs we don't want to see and set up event handlers
	$(document).ready(function () {
		createFileTable(mainDocumentUploader.fileProgressTable);
	});
}

window.documentUploaderLoaded = true;

//on page load, hide some of the divs we don't want to see and set up event handlers
$(document).ready(function () {
	$('#documentFormHolder').hide();
	$('#filesProgress').hide();
	$('.dragAndDropHint').hide();
	
	//if the current browser supports file API then we set stuff up
	if (window.File && window.FileReader) {
		$(document).on("viewControlUpdated", function(ev, controlInfo) {
		
			//update file inputs and their associated labels
			if (typeof WebrSetupAllFileInputs == 'function') WebrSetupAllFileInputs();

			//re-wire drag and drop event handlers for the new DOM element that replaced the previous one
			var control = findDragAndDropControlByName(controlInfo.name);

			//make sure this is an update to a control we care about			
			if (control) {
	 			updateDropZoneEvents(control);
	
				//first go through and update the existing files list if any have been added
				if (controlInfo.metadataJson) {
					var files = JSON.parse(controlInfo.metadataJson);
					var length = files.length;
					for (var i = 0; i < length; i++) {
						if (files[i].urlIsScript)
							files[i].url = eval(files[i].url);
					}
					if (control) {
						control.existingFiles = files;
					}
				}
		
				//go through the file list from the event and remove any files that have since been deleted
				//this will happen from a client side delete that doesn't go back to the server to render existing files		
				if (control.existingFiles) {
					for (var i = control.existingFiles.length - 1; i >= 0; i--) {
						var found = false;
						var length = 0;
						if (controlInfo.value) length = controlInfo.value.length;
						for (var j = 0; j < length; j++) {
							if (controlInfo.value[j] == control.existingFiles[i].poref) {
								found = true;
							}
						}
						if (!found) {
							control.existingFiles.splice(i, 1);
						}
					}
				}
				UploadNextFile();
			}
	    });    
	}
});


//Enables drag and drop file upload the the given control. 
//The control object has the following attributes
//{
//	name: fully qualified name of ALD or null for document component, 
//	ald: ALD OID or null for document component, 
//	id: id of the div where files will be dropped, 
//	url: URL to the document form  
//	maxFiles: the number of files this control will allow to be dropped.  Typically 1 for a single document or not defined for a set.
//
//
//	existingFiles: array of existing files for the control - used to look for duplicates and do revisions
//	each one looks like this
//	{
//		name: name of the document entity
//		url: a method call including parameters where eval() on it will return a URL for the document form
//	}
//{
function AddDragDropDocumentControls(control)
{
	if (window.File && window.FileReader) {
		var alreadyExists = false;
		if (mainDocumentUploader.dragDropDocumentControls[control.id] != null && typeof mainDocumentUploader.dragDropDocumentControls[control.id] != "undefined")
			alreadyExists = true;

		mainDocumentUploader.dragDropDocumentControls[control.id] = (control);
		if (control.existingFiles) {
			var length = control.existingFiles.length;
			for (var i = 0; i < length; i++) {
				if (control.existingFiles[i].urlIsScript) {
					control.existingFiles[i].url = eval(control.existingFiles[i].url);
				}
			}
		}

		//if this is the first time this control has been added, we need to subscribe to events
		//otherwise, the call to subscribeToDragDropEvents will happen in the controlUpdated event handler
		if (!alreadyExists)
			subscribeToDragDropEvents(control);		
	}
}

//set up drag and drop event listeners for the given control
function subscribeToDragDropEvents(control)
{
	var dropZone = document.getElementById(control.id);
	$(dropZone).addClass('dropZone');	
	
	//drag and drop event handlers
	dropZone.addEventListener('dragenter', handleDragEnter, false);
	dropZone.addEventListener('dragover', handleDragOver, false);
	dropZone.addEventListener('dragleave', handleDragLeave, false);
	dropZone.addEventListener('drop',  handleDrop , false);
	dropZone.addEventListener('dragend',  handleDragEnd , false);

	$('.dragAndDropHint').hide();
    
    $(dropZone).hover(
	    function() {
	    	var voffset = 20;
			
			//for views and smartforms, leave the hint where it belongs
	    	if (control.ald)
	    		voffset = 0;
	    		
	    	var hints = $(this).find('.dragAndDropHint');
	    	if (hints.length <= 0) {
	    		hints = $(this).parent().find('.dragAndDropHint');
	    	}
	    	var hint = hints.first();
	    	var pos = $(this).position();
			if (!control.ald) {
				hint.css('position', 'absolute');
				hint.css('top', (pos.top - 20)); 
				hint.css('left', pos.left + $(this).width() - $(hint).width() - 15); 
			}
	    	hints.show();
	    },
	    function() {
	    	$('.hintpointer').remove();
	    	$('.dragAndDropHint').hide();
	    }
	);
}

//get the number of rows in the tbody section of a table
function getRowCount(tableid)
{
	return document.getElementById(tableid).getElementsByTagName("tbody")[0].getElementsByTagName("tr").length;
}

//Updates the event handlers for a given control
//controlInfo only contains the ALD name, so we need to find the div ID based on who has registered for drag/droip events
function updateDropZoneEvents(control)
{
	if (control) {
		subscribeToDragDropEvents(control);
	}
}

//returns a drag and drop control by name.  We store them by OID but the name is in there too...
function findDragAndDropControlByName(name)
{
	var keys = [];
	for (var key in mainDocumentUploader.dragDropDocumentControls) {
  		if (mainDocumentUploader.dragDropDocumentControls.hasOwnProperty(key)) {
    		keys.push(key);
  		}
	}
	
	for (var i = 0; i < keys.length; i++)
	{
		if (mainDocumentUploader.dragDropDocumentControls[keys[i]].name == name)
		{
			return mainDocumentUploader.dragDropDocumentControls[keys[i]];
		}
	}
	return null;
}



//creates a table where we will display current uploads in progress
function createFileTable(table)
{
	var existingHeader = document.getElementById("fileDropHeader");
	if (!existingHeader) {			
		var header = document.createElement('thead');
		mainDocumentUploader.fileProgressTableBody = document.createElement('tbody');
		table.appendChild(header);
		table.appendChild(mainDocumentUploader.fileProgressTableBody);
		
		header.setAttribute("id", "fileDropHeader");		
		header.setAttribute("class", "fileUploadProgressTableHeader");		
		var hrow = header.insertRow(0);
		var hcell2 = hrow.insertCell(0);
		hcell2.innerHTML = "<strong style='color:#FFFFFF'>" + progressHeaderTxt + "</strong>";
		var hcell3 = hrow.insertCell(1);
		hcell3.innerHTML = "<strong style='color:#FFFFFF'></strong>";
		var hcell1 = hrow.insertCell(2);
		hcell1.innerHTML = "<strong style='color:#FFFFFF'>" + fileNameHeaderTxt + "</strong>";		
	}
	$(table).hide();
}

//used to manage drag flicker on older IE.  We cound the events to determine if we are still dragging across the same element
var dragCounter = 0;

//handle drag leave events to hide the visual indicators
function handleDragLeave(event) {
	dragCounter--;
	event.stopPropagation();
	event.preventDefault();
	hideDropNotification(this);
	return false;
}

//handle drag over events to show the visual indicators
function handleDragOver(event) {
	event.stopPropagation();
	event.preventDefault();
	return false;
}

//handle drag end events to hide the visual indicators
function handleDragEnd(event)
{
	dragCounter = 0;	
	event.stopPropagation();
	event.preventDefault();
	hideDropNotification(this);
	return false;
}

//handle drag enter events to show the visual indicators
function handleDragEnter(event) {
	if (mainDocumentUploader.activeDropZone != this)
	{
		$(".activeDropZone").removeClass("activeDropZone");
	}
	dragCounter++;
	event.stopPropagation();
	event.preventDefault();	
	mainDocumentUploader.activeDropZone = this;
	showDropNotification(this);
	return false;
}

//handle the drop event where we start processing dropped files
function handleDrop(event) {
	dragCounter = 0;
	event.stopPropagation();
	event.preventDefault();
	hideDropNotification(this);
	var control = mainDocumentUploader.dragDropDocumentControls[mainDocumentUploader.activeDropZone.id]
	handleDragDropFile(event, control);
	return false;
}

//add dropped files to our table, then upload them one by one
function handleDragDropFile(event, control) {

	var tempUrl = control.url;
	
	//if this is a single file control that has a value in it, but it didn't have a value when the page loaded
	//then the doc upload URL will not know about the existing file.  So we grab it here and append it to the 
	//URL if we need to.  It's also possible that the original was deleted, then a new document uploaded, so 
	//the versionedResource OID will have changed.  In either case we update it with the current value
	if (control.name && control.ald) {
		var currentFileValue = document.forms[0].elements[control.name].value;
		if (currentFileValue && currentFileValue != "set" && currentFileValue != ":00000000000000000000000000000000") {
			tempUrl = removeFromString(tempUrl, "VersionedResource", "&");
			tempUrl += "&VersionedResource=" + currentFileValue;
		}
	}
	
	//Deal with inline cdt views
	if($('#inlineCdtViewMask').is(":visible")){
		$(mainDocumentUploader.fileProgressTable).css("z-index", "99");
	} else {
		$(mainDocumentUploader.fileProgressTable).css("z-index", "");
	}
	
	//files are added to the table asynchronously in order to allow for dialogs when the user must confirm how to handle duplicates
	var filesAdded = AddFilesToTable(event.dataTransfer.files, tempUrl, control.ald, control.maxFiles, control.existingFiles, control.norefresh, control.name);
	
	//once the files are added we clear out the ones the user cancelled and then upload the rest one at a time.
	filesAdded.done(function() {
		mainDocumentUploader.clearUnwantedFiles();
		if (mainDocumentUploader.fileProgressTableBody.getElementsByTagName("tr").length > 0) {	
			$(mainDocumentUploader.fileProgressTable).show();
			UploadNextFile();
		}
		else {
			mainDocumentUploader.filesBeingUploaded.length = 0;	
		}
	});

	return false;
}

//show some visual indication when a control is dragged over
function showDropNotification(el, div)
{
	$(el).addClass("activeDropZone");
}

//hide visual indicator when drag event leaves or ends
function hideDropNotification(el)
{
	if (dragCounter == 0)
	{
		$(".activeDropZone").removeClass("activeDropZone");
	}
}

//adds the dropped files to our file upload table
function AddFilesToTable(filesToAdd, url, ald, maxFiles, existingFiles, norefresh, controlName)
{
	var deferred = $.Deferred();
	var dupsFound = false;

	//maxFiles is optional.  If not supplied, we just add them all
	if (typeof maxFiles == "undefined") {
		maxFiles = filesToAdd.length;
	}

	//Look for existing files with the same filename. 
	for (var j = 0; j < filesToAdd.length && j < maxFiles; j++) {
		var currentFile = new fileUpload(filesToAdd[j], url, ald, norefresh, controlName);
		var duplicateCount = 0;
		var revisionUrl;
		if (typeof existingFiles != "undefined"){
			for (var i = 0; i < existingFiles.length; i++){
				if (existingFiles[i].fileName == filesToAdd[j].name){
					dupsFound = true;
					currentFile.duplicateOptions.push(existingFiles[i]);
				}
			}
		}
	}
	
	//if any duplicates have been found, we will show a dialog to let the user decide what to do
	if (dupsFound) {
		$('#duplicateFiles').empty();
	    var dialogArea = $('<div/>', {id: "duplicateFiles"});
		var html = "<h3>Some of these files already exist. What do you want to do?</h3>";
		html += "<table id='duplicateFilesTable' border='0' cellspacing='0' cellpadding='3' width='100%'>";
		for (var i = 0; i < mainDocumentUploader.filesBeingUploaded.length; i++)
		{
			var radioName = mainDocumentUploader.filesBeingUploaded[i].file.name.replace(/ /g,'');
			html += "<tr class='rowHighlight'><td class='List'>" + mainDocumentUploader.filesBeingUploaded[i].file.name + "</td>";			

			if (mainDocumentUploader.filesBeingUploaded[i].duplicateOptions.length == 1) {				
				html += "<td class='List'><input type='radio' name='" + radioName + "action' id='" + radioName + "action2'value='revision' checked>";
				html += "<label for='" + radioName + "action2'>Revision</label>";
				html += "</td>";
				html += "<td class='List'><input type='radio' name='" + radioName + "action' id='" + radioName + "action1' value='new'>";
				html += "<label for='" + radioName + "action1'>New</label>";
				html += "</td>";
				html += "<td class='List'><input type='radio' name='" + radioName + "action' id='" + radioName + "action3'value='cancel'>";
				html += "<label for='" + radioName + "action3'>Cancel Upload</label>";
				html += "</td>";
			}			
			else if (mainDocumentUploader.filesBeingUploaded[i].duplicateOptions.length == 0) {				
				html += "<td class='List'/>"
				html += "<td class='List'><input type='radio' name='" + radioName + "action' id='" + radioName + "action1' value='new' checked>";
				html += "<label for='" + radioName + "action1'>New</label>";
				html += "</td>";
				html += "<td class='List'><input type='radio' name='" + radioName + "action' id='" + radioName + "action3'value='cancel'>";
				html += "<label for='" + radioName + "action3'>Cancel Upload</label>";
				html += "</td>";
			}			
			else if (mainDocumentUploader.filesBeingUploaded[i].duplicateOptions.length > 1) {
				mainDocumentUploader.filesBeingUploaded[i].url = "";
				html += "<td align='right' colspan='3' class='List'>Multiple duplicates found. Unable to upload.</td>";
			}
			html += "</tr>";
		}
		html += "</table>";
		
		dialogArea.html(html);
		dialogArea.dialog({
            dialogClass: "dialog-no-header",
	        resizable:   false,
	        width: 600,
	        modal:       true,
	        draggable:   false,
			buttons: [
			{
	  			text: "OK",
	  			click: function() {
					for (var j = 0; j < mainDocumentUploader.filesBeingUploaded.length && j < maxFiles; j++) {
						var action = $("input[name='" + mainDocumentUploader.filesBeingUploaded[j].file.name.replace(/ /g,'') + "action']:checked").val();
						if (action == "cancel")
							mainDocumentUploader.filesBeingUploaded[j].url = "";
						else if (action == "revision" && mainDocumentUploader.filesBeingUploaded[j].duplicateOptions.length == 1)
							mainDocumentUploader.filesBeingUploaded[j].url = mainDocumentUploader.filesBeingUploaded[j].duplicateOptions[0].url;
						
						if ( mainDocumentUploader.filesBeingUploaded[j].url != null && mainDocumentUploader.filesBeingUploaded[j].url != "") {
							mainDocumentUploader.filesBeingUploaded[j].addFileToTable();	
						}
					}
					dialogArea.dialog("close");
					deferred.resolve();
	  			}
			},
			{
	  			text: "Cancel",
	  			click: function() {
					mainDocumentUploader.filesBeingUploaded.length = 0;	
					dialogArea.dialog("close");
					deferred.resolve();
	  			}
			}]
		});
	}
	else{
		for (var j = 0; j < mainDocumentUploader.filesBeingUploaded.length && j < maxFiles; j++) {
			if ( mainDocumentUploader.filesBeingUploaded[j].url != null && mainDocumentUploader.filesBeingUploaded[j].url != "")
				mainDocumentUploader.filesBeingUploaded[j].addFileToTable();	
		}
		deferred.resolve();
	}
	return deferred;
}


//upload the next file in the list
function UploadNextFile()
{
	//if we are done uploading, then delete completed files and hide the table
	if (mainDocumentUploader.filesBeingUploaded.length == 0)
	{
		deleteCompletedFileRows();
		return;
	}
	mainDocumentUploader.filesBeingUploaded[0].uploadFile();
}

//deletes completed rows
//if the page is going to update via ajax, then we hide the table
//otherwise, refresh the page
function deleteCompletedFileRows()
{	
	setTimeout(function(){
		for (var i = 0; i < mainDocumentUploader.rowsToRemove.length; i++)
		{
			$(mainDocumentUploader.rowsToRemove).remove();
		}
		$(mainDocumentUploader.fileProgressTable).hide();
		if (mainDocumentUploader.refreshAfterFileUpload)
			location.reload();
	}, 2000);               	           
}

//Creates a fileUpload object for each file that got dropped
function fileUpload(fileObject, url, ald, norefresh, controlName)
{
	mainDocumentUploader.totalFileCount++;

	//need a unique form ID for each file that we are going to upload - an index is fine
	this.progressBarId = "progressbar" + mainDocumentUploader.totalFileCount;
	this.div_progress = "div_progressbar" + mainDocumentUploader.totalFileCount;
	this.file = fileObject;
	this.uploaded = false;
	this.id = mainDocumentUploader.totalFileCount;
	this.ald = ald;
	mainDocumentUploader.filesBeingUploaded.push(this);
	this.oids = [];
	this.duplicateOptions = [];
	this.url = url;
	this.norefresh = norefresh;
	this.controlName = controlName;
	this.cancelEventHandler = null;
		
	//Creates a row in the file table for this file
	this.addFileToTable = function()
	{
		var row = mainDocumentUploader.fileProgressTableBody.insertRow(getRowCount('filesProgress'));
		
		var size = (Math.round(this.file.size / 1024) == 0 ? (this.file.size > 0 ? 1 : 0) : (Math.round(this.file.size / 1024)));
		this.cell2 = row.insertCell(0);
		this.cell3 = row.insertCell(1);
		this.cell1 = row.insertCell(2);
		$(this.cell1).addClass("List"); 
		$(this.cell2).addClass("List"); 
		$(this.cell3).addClass("List"); 
		this.cell1.innerHTML = this.file.name + " (" + size + " KB)";
		this.row = row;
	}
		
	//uploads the file
	this.uploadFile = function() {
		if (this.uploaded)
			return;

		var fileBeingUploaded = this;
		var uiDialog = $('#documentFormHolder');		
		this.url += "&SuppressPortalTools=1&ajaxUpload=1";
			
		var isSimpleDeletePending = false;	
				
		//this if check indicates that this is a control on a view rather than a documents component
		if (fileBeingUploaded.controlName != null && fileBeingUploaded.controlName != "") {
			var currentFileValue = document.forms[0].elements[fileBeingUploaded.controlName].value;
			var simpleDocDeletionFieldName = "_simpleDocumentDelete_" + fileBeingUploaded.controlName;
			var simpleDocDeletionField = document.forms[0].elements[simpleDocDeletionFieldName];
			isSimpleDeletePending = ((simpleDocDeletionField && simpleDocDeletionField.value == "true") || currentFileValue == ":00000000000000000000000000000000");
			if (isSimpleDeletePending)
			{	
				//get rid of these so the server will know we are uploading a new doc that can get a new OID
				//then the new doc won't get deleted when the update postback happens
				if (simpleDocDeletionField) simpleDocDeletionField.value = "false";
				this.url = removeFromString(this.url, "VersionedResource", "&");
			}
		}
				
		//use the control specific URL to load the form into a hidden div
		uiDialog.load(this.url, function() {				
			var size = (Math.round(fileBeingUploaded.file.size / 1024) == 0 ? (fileBeingUploaded.file.size > 0 ? 1 : 0) : (Math.round(fileBeingUploaded.file.size / 1024)));
			fileBeingUploaded.cell1.innerHTML = fileBeingUploaded.file.name + " (" + size + " KB)";	
			fileBeingUploaded.cell2.innerHTML = "<progress class='fileUploadProgressBar' id=" + fileBeingUploaded.progressBarId + " max=100></progress>" + "<div id=" + fileBeingUploaded.div_progress + ">0%</div>";
			
			fileBeingUploaded.cell3.innerHTML = "<a href='#' class='cancelFileUploadLink' id=cancel_progress" + fileBeingUploaded.index + ">" + cancelButtonTxt + "</a>"
			
			var theForm = uiDialog.find('form');

			//remove the file input from the retrieved form since it is read-only
			var originalFileInput = theForm.find("input[name='Document.targetURL']");
			originalFileInput.remove();
			var button = theForm.find("input[name='Button']");
			if (button)
				button.attr("value", "OK");
						
			var url = $(theForm).attr("action");		
				
			//this check is to determine if the user deleted a document and then immediately dragged a new one
			if (isSimpleDeletePending) //indicates a delete happened
			{	
				//force a new document to get created when the ajax form is posted
				var _webrUpdateOID = theForm.find("input[name='_webrUpdateOID']");
				_webrUpdateOID.attr("value", "");			
				url = removeFromString(url, "VersionedResource", "&");
			}

			var data = new FormData(theForm.get(0));			
		
			//add our own file input to replace the one we deleted
			data.append('Document.targetURL', fileBeingUploaded.file);
			
			var xhr = new XMLHttpRequest();
			xhr.upload.progressbar = fileBeingUploaded.progressBarId;
			xhr.upload.xhreq = xhr;
			xhr.msgCell = fileBeingUploaded.cell3;
			xhr.actCell = fileBeingUploaded.cell3;
			xhr.fileBeingUploaded = fileBeingUploaded;
			xhr.upload.addEventListener("progress", function (e) {
				uploadProgress(e, this, fileBeingUploaded)
			}, false);
			xhr.onreadystatechange = function () {
				stateChanged(this, this.fileBeingUploaded);
			};
			xhr.open('POST', url, true);
			xhr.send(data);
		
			fileBeingUploaded.uploaded = true;
		});				
	}
}

//onreadystatechange event handler for our ajax file upload request 
function stateChanged(xReq,fileBeingUploaded)
{   
	var DONE_READYSTATE = 4;
	var CUSTOM_ERROR_CODE = 566; //this is defined on the server when an exception is thrown trying to create the document entity

	if(xReq.readyState == DONE_READYSTATE )
	{
	
		var uiDialog = $('#documentFormHolder');
		uiDialog.remove('form');        
		uiDialog.html("");

		var fileIndex = mainDocumentUploader.filesBeingUploaded.indexOf(fileBeingUploaded);
		if (fileIndex >= 0)
			mainDocumentUploader.filesBeingUploaded.splice(fileIndex, 1);

        if(xReq.status == CUSTOM_ERROR_CODE){ 
        	//an exception was thrown on the server
            xReq.msgCell.innerHTML="<div class='Error'>Error uploading document.</div>";
    		UploadNextFile();
        }
        else
        {        
        	var responseText = xReq.getResponseHeader("x-document-poref");      
            // upon cancelling the file upload, the statusText is empty instead OID of the document that is uploaded successfully.
            if(responseText != null && responseText != "") 
            {
            	xReq.msgCell.innerHTML="<div class='fileUploadSuccess'>" + fileUploadTxt + " " + successfulTxt + "!</div>";    
				mainDocumentUploader.rowsToRemove.push(fileBeingUploaded.row);
            }
            else
            {
            	xReq.msgCell.innerHTML="<div class='fileUploadCancelled'>" + fileUploadTxt + " " + cancelledTxt + "!</div>";
            }

			//view with ALD will be updated via ajax            
            if (fileBeingUploaded.ald)
            {
				fileBeingUploaded.oids.push(responseText);  //save the OID of the new file so the ajax update works properly    			
     			callBack(fileBeingUploaded);
     		}
     		else //document component needs to be refreshed
     		{
	            if(responseText != null && responseText != "") 
     				mainDocumentUploader.refreshAfterFileUpload = true;
				UploadNextFile();
     		}
       	}                 	
   	}
}

//updates the progress bar for a given file upload request
function uploadProgress(evt, i, fileBeingUploaded) {
	var bar = document.getElementById(i.progressbar);
	var divs = $("#div_" + i.progressbar);
	if (fileBeingUploaded.cancelEventHandler == null) {
		fileBeingUploaded.cancelEventHandler = function(e) {
			e.preventDefault();
			i.xhreq.abort();
		}
		$("#cancel_progress" + fileBeingUploaded.index).click(function (e) 
		{ 
			fileBeingUploaded.cancelEventHandler(e); 
		});
	}
	if (evt.lengthComputable) {
		var percentComplete = (evt.loaded / evt.total) * 100;
		bar.value = parseInt(percentComplete);
		$(divs).text(bar.value + "%");
	}
}

//callback that updates the views document control after a file is uploaded
function callBack(fileBeingUploaded)
{ 
	if (mainDocumentUploader.temp_webrUnencodedUrl == "") {
		_webrUnencodedUrl = location.href;
	}
	else {
		_webrUnencodedUrl = mainDocumentUploader.temp_webrUnencodedUrl;
	}
	if(fileBeingUploaded.norefresh) {
		doPostback = false;
		$(mainDocumentUploader.fileProgressTable).hide();
		UploadNextFile();
		
	}	
	// chooserCallback(bPostback, iFormIdx, arrOIDs, sValueFieldName, arrDisplayStrings,
    //                     sDisplayFieldName, bForceDirty, sAldOID, fAldUpdatedCallback)
	
    var dispArr = [];
    dispArr.push(fileBeingUploaded.file.name);
   	
    var docForceDirty = true;  //always dirty the form when a new document is uploaded
    chooserCallback(doPostback, 0, fileBeingUploaded.oids, valueField, dispArr, displayField, docForceDirty, fileBeingUploaded.ald);
}

//remove a substring, then keep removing up til and including the delimeter
//the delimiter is assumed to be a single character
function removeFromString(str, removeStr, endDelim) {
	var index = str.indexOf(removeStr);
	if (index >= 0) {
		var endIndex = str.indexOf(endDelim, index + 1);
		if (endIndex >= 0) {
			str = str.replace(str.slice(index, endIndex + 1) , "");
		}
		else {
			str = str.replace(str.slice(index) , "");						
		}
	}
	return str;
}