//    ATTN:
//     _____              _ _     _    _            _______ _     _
//    |  __ \            ( ) |   | |  | |          |__   __| |   (_)
//    | |  | | ___  _ __ |/| |_  | |  | |___  ___     | |  | |__  _ ___
//    | |  | |/ _ \| '_ \  | __| | |  | / __|/ _ \    | |  | '_ \| / __|
//    | |__| | (_) | | | | | |_  | |__| \__ \  __/    | |  | | | | \__ \
//    |_____/ \___/|_| |_|  \__|  \____/|___/\___|    |_|  |_| |_|_|___/
//
//    This module is not finalized. The functionality contained herein
//    is not finalized. Everything in this file will change in future
//    releases. If you use this module or code, you are guaranteeing
//    the code you write will break in future releases.
//
//    To reiterate: DO NOT USE THIS MODULE.
//                  DO NOT USE THIS CODE.
//

/* global console */
/* global CKEDITOR */
/* global require */
/* global define */

/**
 * EntityView Navigation Module
 * @module EntityView/Navigation
 */
define(
    [
        "module",
        "jquery",
        "underscore",
        "domReady",
        "PortalTools",
        "DebugUtil",
        "dialog",
        "global",
        "BusyIndicator",
        "./Deprecated"
    ],
    function(
        module,
        $,
        _,
        domReady,
        PortalTools,
        DebugUtil,
        dialog,
        globals,
        busyIndicator,
        Deprecated
    ) {
        "use strict";

        var global = window;

        var DEBUG = new DebugUtil(module.id);

        /**
         * @type {PortalTools.EventType}
         */
        //var onSubmit_PortalTools = new PortalTools.EventType();
        var onSubmit_PortalTools = window.EntityView.onSubmit;

        /**
         * Object containing language from LocalizedStringResource
         * @const
         */
        var _webrFormFeedbackStrings = globals.get("_webrFormFeedbackStrings");

        /**
         * Contains all instances of the Navigation object
         * @type {Array.<Navigation>}
         */
        var navigationInstances = [];

        ////////////////////////////////////////////////////////////////////////////////
        // Navigation options.
        //

        /**
         * TODO: these settings should not belong in this file. They should be passed in as configuration when declaring a new Navigation object.
         * Generates the Navigation Options based on the viewLocation.
         * @param parameters
         * @returns {{readOnly: (boolean), viewLocation: (!string), navigationSelector: (string), navigationContainer: (string), navigationDefinitions: (Object)}}
         */
        var getNavigationOptions = function(parameters) {

            var options = {
                readOnly     : parameters.readOnly,
                viewLocation : parameters.viewLocation
            };

            switch(options.viewLocation) {
                case 'activity':
                    options.navigationSelector = "#CLICKPORTAL_FAKESELECTOR";
                    options.navigationContainer = "#CLICKPORTAL_FAKESELECTOR";
                    options.navigationDefinitions = {};
                    //    options.navigationSelector = ".navigation";
                    //    options.navigationContainer = "#navigation";
                    //    options.navigationDefinitions = {
                    //        okBtn : {
                    //            mode: "ok",
                    //            validate: true
                    //        },
                    //        cancelBtn : {
                    //            mode: "cancel",
                    //            validate: false,
                    //            submissionFunction: function() {
                    //                global.close();
                    //            }
                    //        }
                    //    };
                    break;
                case 'projectEditor':
                    options.navigationSelector = ".navigation";
                    options.navigationContainer = "#CommandBarTop, #CommandBarBottom";
                    options.navigationDefinitions = {
                        apply_btnName : {
                            mode: "view-save",
                            validate: true
                        },
                        back_btnName : {
                            mode: "back",
                            validate: false,
                            submissionFunction: function() {
                                global.history.go(-1);
                            }
                        },
                        cancel_btnName : {
                            mode: "cancel",
                            validate: false,
                            submissionFunction: function() {
                                global.location.assign(globals.get("returnUrl"));
                            }
                        },
                        continue_btnName : {
                            mode: "wizard-continue",
                            validate: true
                        },
                        lnkExitProjectEditorName : {
                            mode: "wizard-exit",
                            //modeFunction: function() {
                            //    if (options.readOnly) {
                            //        return 'wizard-close-nosave';
                            //    } else if (parameters.isFormDirty && parameters.isFormDirty.call({})) {
                            //        return 'loadExitDialog';
                            //    } else {
                            //        return 'wizard-close-nosave';
                            //    }
                            //},
                            validate: false
                        },
                        finish_btnName : {
                            mode:"wizard-close",
                            validate: true
                        },
                        lnkHideShowErrorsProjectEditorName : {
                            mode: (function() {
                                var prefix;
                                
                                // this currently takes presidence since the parameters.viewMode only takes in to acount where the form is made, not where it is displayed
                                var globalMode = globals.get("Mode");
                                
                                var tempMode = globalMode ? globalMode : parameters.viewMode;
								
                                switch(tempMode) {
                                    case "smartform": prefix = "wizard"; break;
                                    case "view":      prefix = "view";   break;
                                    default:
                                        throw new Error("Unknown view mode");
                                }

                                //Toggles errors.
                                if (window.top !== window.self) {
                                    return prefix + "-save-bustframe";
                                }
                                return prefix + "-save-showframe";
                            })(),
                            redirectUrl: (function() {
                                if (window.top !== window.self) {
                                    return global.self.location;
                                }
                                return globals.get("editorWithValidationUrl");
                            })(),
                            validate: false
                        },
                        ok_btnName : {
                            mode:"view",
                            validate: true
                        },
                        readonlyOk_btnName : {
                            mode:"view", // TODO: not necessary?
                            validate: false,
                            submissionFunction: function() {
                                global.location.assign(globals.get("returnUrl"));
                            }
                        },
                        lnkSaveProjectEditorName : {
                            modeFunction: function() {
                                return "wizard-save";
                                //switch(View.getviewLocation()) {
                                //    case "":
                                //}
                            },
                            validate: false
                        }
                    };
                    break;
                case 'customDataType':
                    options.navigationSelector = ".navigation";
                    options.navigationContainer = "#navigation";
                    options.navigationDefinitions = {
                        ok_btnName : {
                            mode:"OK",
                            validate: true
                        },
                        okAddMore_btnName : {
                            mode:"OK AND ADD ANOTHER",
                            validate: true
                        },
                        cancel_btnName : {
                            mode:"cancel",
                            validate: false,
                            submissionFunction: function() {
                                global.close();
                            }
                        }
                    };
                    break;
                case 'selfRegistration': // Self-Registration component
                	options.navigationSelector = ".navigation";
                	options.navigationContainer = ".navigation";
                	options.navigationDefinitions = {};
                	break;
                case 'projectTabProperties': // in RMConsole
                case 'partyTabProperties': // in RMConsole
                case 'entityAdminPropertiesForm': // in admin pages
                case 'personManagementContactform':
                case 'personManagementOrgform':
                case 'documentForm':
                case 'ajaxView':
                    options.navigationSelector = "#CLICKPORTAL_FAKESELECTOR";
                    options.navigationContainer = "#CLICKPORTAL_FAKESELECTOR";
                    options.navigationDefinitions = {};
                    //options.navigationSelector = ".navigation";
                    //options.navigationContainer = "#NavigationTop, #NavigationBottom";
                    //options.navigationDefinitions = {
                    //    apply_btnName : {
                    //        mode:"view-save",
                    //        validate: true
                    //    }
                    //};
                    break;
                default:
                    throw new Error('Unknown Form Type: ' + parameters.viewLocation);
            }

            if(options.beforeUnloadHandler) {
                options.beforeUnloadHandler = parameters.onbeforeunloadHandler;
            }

            if(options.viewLocation === "projectEditor" || options.viewLocation === "selfRegistration") {
                options.navigationBusyIndicator = (function() {
                    var indicator = busyIndicator.newIndicator(options.navigationContainer);
                    return {
                        start : indicator.start,
                        stop  : indicator.stop
                    };
                })();
            }

            return options;
        };

        /**
         * JavaScript object to handle EntityView navigation
         * @param formData
         * @returns {{viewId: (*|!string|View.viewId), navigate: Function, isNavigating: Function, preventNavigation: Function, getSubmissionMode: Function, registerView: Function, navigationBusyIndicator: *}}
         * @constructor
         * @class
         * @alias module:EntityView/Navigation
         */
        function Navigation(formData) {
            var view;

            /**
             * @type {boolean}
             */
            var isCurrentlyNavigating = false;

            /**
             * @type {boolean}
             */
            var isSpecialCaseRedirectInProgress = false;

            /**
             * Holds information about the current navigation event.
             * @type {null|Event}
             */
            var navigationEvent = null;

            /**
             * if true, no navigation is permitted (through this Module)
             * @type {boolean}
             */
            var isNavigationPrevented = false;

            /**
             * if true, no validation is run
             * @type {boolean}
             */
            var isValidationPrevented = false;

            /**
             * Holds information about the current submission mode
             * @type {string}
             */
            var submissionMode;

            /**
             * View ID this navigation module instance is associated to.
             * @readonly
             * @type {*|!string|View.viewId}
             */
            var viewId = formData.viewId;

            /**
             * The view associated to this instance of the navigation module is readonly or not.
             * @readonly
             * @type {*|boolean|View.readOnly}
             */
            var readOnly = formData.readOnly;

            /**
             * Holds information about the view's location
             * @readonly
             * @type {*|!string|View.viewLocation}
             */
            var viewLocation = formData.viewLocation;

            /**
             * Holds information about the view's mode
             * @readonly
             * @type {string}
             */
            var mode = formData.mode;

            ////////////////////////////////////////////////////////////////////////////////
            // Functions with defaults
            // this is here at the top so that getNavigationOptions can use isDirty
            //

            var addError,
                clearError,
                displayErrors,
                validateForm,
                isDirty,
                viewOnSubmit;

            /**
             * Adds error for display to the view.
             * @param str
             * @returns {*}
             */
            addError = function(str) {
                if(view && view.addError) {
                    return view.addError.apply({}, arguments);
                } else {
                    console.error(str);
                }
            };

            /**
             * Clears errors from the view
             * @returns {*}
             */
            clearError = function() {
                if(view && view.clearError) {
                    return view.clearError.apply({}, arguments);
                }
            };
            
            
            /*
             * Displays errors in the view
             */
            displayErrors = function(){
            	if(view && view.displayErrors){
            		return view.displayErrors.apply({}, arguments);
            	} 
            }

            /**
             * Runs validation on the view
             * @returns {Promise}
             */
            validateForm = function() {
                if(view && view.validate) {
                    return view.validate.apply({}, arguments);
                }
                console.warn("No validation is setup.");
                return $.Deferred().resolve().promise();
            };

            /**
             * Determines if the view has been dirtied.
             * @returns {boolean}
             */
            isDirty = function() {
                if(view && view.isDirty) {
                    return view.isDirty.apply({}, arguments);
                }
                return false;
            };

            /**
             * Runs the onSubmitCallbacks function on View.
             */
            viewOnSubmit = function() {
                if(view && view.onSubmit) {
                    view.onSubmit.apply({});
                }
            };



            ////////////////////////////////////////////////////////////////////////////////
            // Setup
            //

            var submissionModeFormObject, // jQuery Object
                redirectUrlDomObject, // jQuery Object
                options,
                navigationBusyIndicator,
                navigationDefinitions,
                allNavigationItems,
                loadExitDialog;

            /**
             * @type {{readOnly: (boolean), viewLocation: (!string), navigationSelector: (string), navigationContainer: (string), navigationDefinitions: (Object)}}
             */
            options = getNavigationOptions({
                viewLocation : viewLocation,
                viewMode     : mode,
                readOnly     : readOnly,
                isFormDirty  : isDirty
            });

            if(!options.navigationSelector) {
                throw new Error("Missing options.navigationSelector");
            }
            if(!options.navigationContainer) {
                throw new Error("Missing options.navigationContainer");
            }

            /**
             * Navigation definitions. Basically the settings for the various ways to navigate the form.
             * @type {{okBtn: {mode: string, validate: boolean}, cancelBtn: {mode: string, validate: boolean, submissionFunction: Function}}|*}
             */
            navigationDefinitions = options.navigationDefinitions;

            /**
             * The popup dialog when trying to exit a dirtied view.
             */
            loadExitDialog = options.loadExitDialogFunction
                ? options.loadExitDialogFunction
                : function() {

                dialog.open({
                    type : "confirm",
                    text : "Changes were made to the form. What would you like to do?",
                    buttons : [
                        {
                            text: "Save Changes & Exit",
                            callbackFunction : function() {
                                this.close();
                                navigate({
                                    mode : "wizard-close-save",
                                    validate: false
                                });
                            }
                        },
                        {
                            text : "Ignore Changes & Exit",
                            callbackFunction : function() {
                                this.close();
                                navigate({
                                    mode : "wizard-close-nosave",
                                    validate: false
                                });
                            }
                        },
                        {
                            text : "Cancel",
                            callbackFunction : function() {
                                this.close();
                            }
                        }
                    ]
                });
            };

            if(options.beforeUnloadHandler) {
                global.onbeforeunload = options.beforeUnloadHandler;
            }

            /**
             * Placeholder for the navigation busy indicator
             * The indicator object is created when the view associated with this navigation object.
             */
            navigationBusyIndicator = options.navigationBusyIndicator
                ? options.navigationBusyIndicator
                : { // default
                start : function() {},
                stop  : function() {}
            };

//            Deprecated.setNavigationBusyIndicator(navigationBusyIndicator);



            ////////////////////////////////////////////////////////////////////////////////
            // Functions
            //

            var cancelNavigation,
                constructRedirectUrl,
                constructSubmissionMode,
                constructSubmissionFunction,
                getSubmissionMode,
                isNavigating,
                navigate,
                navigationEventHandler,
                registerView,
                setNavigationState,
                preventClientValidation,
                preventNavigation,
                resetView,
                setRedirectUrl,
                setSubmissionMode,
                setTargetTop,
                submitEvent,
                submitHandler,
                formPostErrorHandler,
                updateCKEditorInstances;

            /**
             * Cancels navigation.
             *   * stops Event propagation
             *   * resets navigation state
             *   * stops busy indicator
             */
            cancelNavigation = function() {
                if(navigationEvent) {
                    navigationEvent.stopImmediatePropagation();
                }
                if(submitEvent) {
                    submitEvent.stopImmediatePropagation();
                }

                resetView();
            };

            /**
             * Resets view so that it can be navigateable and usable again.
             */
            resetView = function() {
                navigationEvent = null;
                submitEvent = null;

                setRedirectUrl(null);
                setSubmissionMode(null);
                setNavigationState(false);

                navigationBusyIndicator.stop();
            };

            /**
             * Builds the redirect url from String or a Function
             * @param typeObject
             * @returns {null|string}
             */
            constructRedirectUrl = function(typeObject){
                if(typeObject.redirectUrlFunction) {
                    return typeObject.redirectUrlFunction.call({});
                } else if(typeObject.redirectUrl) {
                    return typeObject.redirectUrl;
                }
                return null;
            };

            /**
             * Builds the submission mode from String or a Function
             * @param typeObject
             * @returns {*}
             */
            constructSubmissionMode = function(typeObject){
                if(typeObject.modeFunction) {
                    return typeObject.modeFunction.call({});
                }
                return typeObject.mode;
            };

            /**
             * Builds the submission function.
             * If navigation is prevented, it will cancel navigation. right before navigating
             * @param typeObject
             * @returns {Function}
             */
            constructSubmissionFunction = function(typeObject) {
                if(isNavigationPrevented) {
                    return function() {
                        DEBUG.warn("Navigation prevented.");
                        viewOnSubmit();
                        cancelNavigation();
                    };
                }

                if(typeObject.submissionFunction) {
                    return function() {
                        viewOnSubmit();
                        typeObject.submissionFunction.apply({});
                    };
                }

                if(view) {
                    var formObject = view.getFormObject();
                    return function() {
                        viewOnSubmit();
                        formObject.submit();
                    };
                }

                return function() {
                    DEBUG.info("Submission Function is empty. No form was registered with the Navigation Module.");
                };
            };


            /**
             * This performs all the actions necesary during the View navigation process.
             * @param {Object} typeObject
             */
            navigate = function(typeObject) {
                try {
                    DEBUG.log(function() {
                        if(typeObject.modeFunction) {
                            return "Navigating to " + typeObject.modeFunction.call({});
                        }
                        return "Navigating to " + typeObject.mode;
                    });

                    // Check to see if form was already submitted.
                    // We don't want to submit again if it was.
                    if (isNavigating()) {
                        dialog.alert(_webrFormFeedbackStrings.alreadySubmitted);
                        //throw new Error(_webrFormFeedbackStrings.alreadySubmitted);
                        return;
                    }
                    if(isSpecialCaseRedirectInProgress) {
                        DEBUG.log(1, "Special case redirect is already in progress");
                        return;
                    }

                    var submissionMode = constructSubmissionMode(typeObject);

                    if(submissionMode === "loadExitDialog") {
                        loadExitDialog();
                        return;
                    }

                    clearError();

                    navigationBusyIndicator.start();

                    var submissionFunction = constructSubmissionFunction(typeObject);

                    /// todo move in to the construct submission function

                    isSpecialCaseRedirectInProgress = submissionMode === "specialCaseRedirect";
                    setNavigationState(true);

                    setSubmissionMode(submissionMode);
                    //setTargetTop(submissionMode === "wizard-save-bustframe" || submissionMode === "view-save-bustframe");
                    setTargetTop(typeObject.bustFrame || submissionMode === "wizard-save-bustframe" || submissionMode === "view-save-bustframe");
                    setRedirectUrl(constructRedirectUrl(typeObject));
                    
                    if(view) {
                        view.updateFormMetadata();
                        var formObj = view.getFormMetadata();
                    }
                    
                    if(!isValidationPrevented && typeObject.validate) {
                        validateForm() // returns promise
                            .done(function () {
                                DEBUG.log("All properties passed validation");
                                submissionFunction.call({});
                            })
                            .fail(function () {
                                DEBUG.log("Some properties failed validation");
                                if(view) {
                                    view.displayErrors();
                                }
                                cancelNavigation();
                            });
                    } else {
                        DEBUG.log("No Validation");
                        submissionFunction.call({});
                    }


                } catch(error) {
                    cancelNavigation();
                    //DEBUG.error(error.message);
                    //TODO: there must be a better message to put here. This is the type of thing a user can't fix.
                    //TODO: this could possibly be a dialog since this is a really bad error.
                    if(_webrFormFeedbackStrings) {
                        addError(_webrFormFeedbackStrings.submissionError);
                    }
                    throw error;
                }

            };

            /**
             * Whenever an item in allNavigationItems causes this eventHandler to run, we determine which element
             * this is and runs the correct navigation type.
             * @param {Event} event
             */
            navigationEventHandler = function(event) {
                navigationEvent = event;
                event.preventDefault();
                event.stopPropagation();

                var targetName = event.currentTarget.getAttribute("name");
                var mappedTargetObject = navigationDefinitions[targetName];

                if(mappedTargetObject === undefined) {
                    throw new Error("Could not find defined navigation for name: " + targetName);
                }
                navigate(mappedTargetObject);
            };

            /**
             * Registers the view with this navigation instance (so we can run methods on the view)
             * @param theView
             */
            registerView = function(theView) {
                if(!theView) {
                    throw new Error("view is required");
                }

                view = theView;

                var formObject = view.getFormObject();

                redirectUrlDomObject = formObject.get(0).redirectUrl;

                //TODO: ideally we should change the activity page to use the submissionMode object instead of button
                // problem is some code might be relying on the fact that the submission mode is set on the button
                submissionModeFormObject = viewLocation === "activity"
                    ? $(formObject.get(0).button)
                    : $(formObject.get(0).submissionMode);

				//Avoid attaching multiple handlers
				//This check is mostly needed only for inline cdt views
				var formObjectHandlers = $._data(formObject[0], "events");
				var submitHandlerExists = (formObjectHandlers && formObjectHandlers["submit"] &&
					_(formObjectHandlers["submit"]).find(function(ele){
						//Function.prototype.name is ES6 standard and IE11 doesn't support it
						var handlerName = ("name" in Function.prototype)
							? ele.handler.name
							: ele.handler.toString().match(/^\s*function\s+([^\(\s]*)\s*/)[1];
						return handlerName  === "submitHandler";
					}) !== undefined
				);
				if(!submitHandlerExists){
					formObject.submit(submitHandler);
				}
            };
            
            /**
             * This function executes on the submit event of the view.
             * This executes all the onSubmit callback functions
             * @param event
             */
            function submitHandler(event) { //For an issue in IE11, this function has to be declared this way

                // SC00000995 - start
                // DJB 2006-03-02
                // A fairly standard browser behavior is that when there is an HTML form
                // with *ONE* single line text box control, hitting ENTER while in that text
                // box will submit the form.  If that happens, it doesn't run through our
                // normal submitForm() client side script and the submit mode isn't set.
                // This can cause problems with the server side form handler.  So here, we
                // don't let the form submit if the submission mode isn't set.  This is called
                // by the form's onSubmit event.
                // SC00000995 - end
                //
                // Also if the state says it's not currently in submission, then something went wrong.
                if (viewLocation === "projectEditor" && !submissionMode) {
                    throw new Error("Problem encountered in navigate:"
                    + "\nIs projectEditor and submissionMode is:  " + submissionMode);
                }
                if (!isNavigating()) {
                    throw new Error("Problem encountered in navigate:"
                    + "\nisNavigating: " + isNavigating());
                }

                try {
                    // this has to go here because it passes in the event to client event handlers.
                    onSubmit_PortalTools.prepare().resolve(event);
                } catch(error) {
                    //DEBUG.error("Error occurred when running View.onSubmit.prepare().resolve(event): " + error.message);

                    addError(_webrFormFeedbackStrings.submissionError + "<br />");
                    cancelNavigation(event);
                    throw error;
                }

                // Grids is using prevent default to stop the submission process.
                if(event.isDefaultPrevented()) {
                    DEBUG.log("Event DefaultPrevented. (2)");
                    cancelNavigation(event);
                }


                if(submissionMode === "wizard-exit") {
                    cancelNavigation(event);

                    var mode;

                    if (readOnly) {
                        mode = 'wizard-close-nosave';
                    } else if (isDirty && isDirty.call({})) {
                        mode = 'loadExitDialog';
                    } else {
                        mode = 'wizard-close-nosave';
                    }

                    navigate({mode: mode, validate:false});
                    return false;
                }
            };
            
            /**
             * This function handles the form post fail condition
             */
            formPostErrorHandler = function(str){
            	addError(str);
            	displayErrors();
            	cancelNavigation();
            }

			/**
			 * Helper function updating ckeditor instances if any.
			 * Useful for ajax form posts where ckeditors are not automatically updated
			 */
			updateCKEditorInstances = function(){
				if (typeof CKEDITOR != "undefined"){
					$.each(CKEDITOR.instances, function(){
						this.updateElement();
					});
				}
			}
			
			////////////////////////////////////////////////////////////////////////////////
            // Getters and Setters
            //

            /**
             * @returns {string} the submission mode object
             */
            getSubmissionMode = function() {
                return submissionMode;
            };

            /**
             * sets submissionMode
             * @param submissionModeString
             */
            setSubmissionMode = function(submissionModeString) {
                submissionMode = submissionModeString;
                if(submissionModeFormObject) {
                    submissionModeFormObject.val(submissionModeString);
                }
                globals.get("EntityView").projectEditorSubmitMode = submissionModeString;
            };

            /**
             * @returns {boolean} true if currently in the navigation process
             */
            isNavigating = function() {
                return isCurrentlyNavigating;
            };

            /**
             * @param {boolean} boolState - Sets isCurrentlyNavigating
             */
                //We are using a function here because we have to handle the legacy values.
            setNavigationState = function(boolState) {
                isCurrentlyNavigating = boolState;

                //This is the legacy value we need to support
                globals.get("EntityView").submitInProgress = boolState;
            };

            /**
             * Prevents Client-Side validation (validation through JavaScript with this instance
             * of the Navigation object.
             * @param bool
             */
            preventClientValidation = function(bool) {
                isValidationPrevented = bool !== false;
            };

            /**
             * Prevents navigation with this instance of the Navigation object.
             * Right before form submission, if true, the process will fail.
             * @param bool
             */
            preventNavigation = function(bool) {
                isNavigationPrevented = bool !== false;
            };

            /**
             * Sets the value on redirectUrlDomObject
             * @param {null|string} strRedirectUrl
             */
            setRedirectUrl = function(strRedirectUrl) {
                if(redirectUrlDomObject) {
                    redirectUrlDomObject.value = strRedirectUrl;
                }
            };

            /**
             * Sets target top on view form object.
             * @param {boolean} isSetTarget
             */
            setTargetTop = function(isSetTarget) {
                if(view && isSetTarget && window.top !== window.self) {
                    view.getFormObject().prop("target", "_top");
                }
            };


            ////////////////////////////////////////////////////////////////////////////////
            // Initialize
            //

            domReady(function() {
                /**
                 * Holds all the items on the page that are used for navigation (links, buttons, etc)
                 * @type {jQuery}
                 */
                allNavigationItems = $(options.navigationSelector, options.navigationContainer);

                allNavigationItems.click(navigationEventHandler);

                // Opera at one point required this.
                // TODO: test to see if Opera still needs this.
                $(document.body).unload(function() {
                    DEBUG.info("Wiping navigation state");
                    setNavigationState(false);
                });
            });

            // This event 'pagehide' needed to be added here due to the BFCache (back-forward cache) basically
            // hides the page in it's current state. So, when the user hits the back button, they would still see
            // the navigationBusyIndicator.
            //
            // "When a user navigates to a cached page, inline scripts and the onload handler do not
            //  run (steps 2 and 3), since in most cases, the effects of these scripts have been preserved."
            // - https://developer.mozilla.org/en-US/docs/Using_Firefox_1.5_caching
            $(window).on("pagehide" , resetView);


            ////////////////////////////////////////////////////////////////////////////////
            // Public Functions / Variables
            //

            // return function
            var navigation = {
                viewId                  : viewId,
                navigate                : navigate,
                isNavigating            : isNavigating,
                preventClientValidation : preventClientValidation,
                preventNavigation       : preventNavigation,
                getSubmissionMode       : getSubmissionMode,
                registerView            : registerView,
                navigationBusyIndicator : navigationBusyIndicator,
                cancelNavigation        : cancelNavigation,
                submitHandler			: submitHandler,
                formPostErrorHandler	: formPostErrorHandler,
                updateCKEditorInstances : updateCKEditorInstances
            };

            Deprecated.registerNavigation(navigation);

            ////////////////////////////////////////////////////////////////////////////////
            // register and return
            //

            navigationInstances.push(navigation);
            return navigation;
        }


        /**
         * //FIXME
         * @param viewId
         * @returns {Navigation} Navigation instance with specified viewId
         */
            // no input gets first instance
        Navigation.get = function(viewId) {
            if(navigationInstances.length === 0) {
                return undefined;
            }

            if(viewId) {
                return _.findWhere(navigationInstances, {viewId: viewId});
            }

            return navigationInstances[0];
        };

        return Navigation;

    });