//    ATTN:
//     _____              _ _     _    _            _______ _     _
//    |  __ \            ( ) |   | |  | |          |__   __| |   (_)
//    | |  | | ___  _ __ |/| |_  | |  | |___  ___     | |  | |__  _ ___
//    | |  | |/ _ \| '_ \  | __| | |  | / __|/ _ \    | |  | '_ \| / __|
//    | |__| | (_) | | | | | |_  | |__| \__ \  __/    | |  | | | | \__ \
//    |_____/ \___/|_| |_|  \__|  \____/|___/\___|    |_|  |_| |_|_|___/
//  
//    This module is not finalized. The functionality contained herein
//    is not finalized. Everything in this file will change in future
//    releases. If you use this module or code, you are guaranteeing
//    the code you write will break in future releases.
//  
//    To reiterate: DO NOT USE THIS MODULE.
//                  DO NOT USE THIS CODE.
//

/* global console */
/* global CKEDITOR */
/* global require */
/* global define */
/* global window */
/* global JSON */

/**
 * EntityView View Module
 * @module EntityView/View
 * @version 1.0
 */
define(
    [
        "module",
        "jquery",
        "underscore",
        "PortalTools",
        "DebugUtil",
        "dialog",
        "global",
        "domReady",
        "./Deprecated",
        //"moment", // moved to when a date is used -> AKA we are lazy loading this
        "jQuery/Plugins/jquery.textchange/jquery.textchange"
    ],
    function(
        module,
        $,
        _,
        PortalTools,
        DebugUtil,
        dialog,
        globals,
        domReady,
        deprecated,
        //moment,
        textchange // should be undefined
    ) {
        "use strict";
//alert("View A");

        var DEBUG = new DebugUtil(module.id);

        /**
         * Object.create Polyfill
         */
        if (typeof Object.create !== 'function') {
            (function () {
                var F = function () {};
                Object.create = function (o) {
                    if (arguments.length > 1) {
                        throw new Error('Second argument not supported');
                    }
                    if (typeof o !== 'object') {
                        throw new TypeError('Argument must be an object');
                    }
                    F.prototype = o;
                    return new F();
                };
            })();
        }

        var root = window;

        /**
         * Null OID string: used to compare form values against.
         */
        var nullOIDString = ":00000000000000000000000000000000";

        /**
         * Contains all initialized instances of the view object.
         * @type {Array}
         * @memberof module:EntityView/View
         */
        var viewInstances = [];

        /**
         * default view options
         * @type {{ruleNamespace: string, debounceWait: number}}
         * @memberof module:EntityView/View
         */
        var defaultOptions = {
            ruleNamespace : ".rule.conditionalDisplay",
            debounceWait  : 300
        };


        ////////////////////////////////////////////////////////////////////////////////
        // Features: cachedMemoize
        //

        //TODO convert to requirejs config options
        var cacheFunctions = true;

        //TODO convert to an underscore mixin
        // cache and memoize an expensive function by storing its results.
        var cachedMemoize = function(func, cacheTimeFunction, hasher) {
            if(!cacheFunctions) {
                return func;
            }
            var internalCachedTime = null;
            var memo = {};
            /* jshint expr:true */
            cacheTimeFunction || (cacheTimeFunction = _.now);
            hasher || (hasher = _.identity);
            return function() {
                var currentCacheTime = cacheTimeFunction.apply(this, arguments);
                var key = hasher.apply(this, arguments);

                if(currentCacheTime > internalCachedTime) {
                    internalCachedTime = currentCacheTime;
                    memo = {};
                }

                //if( !_.has(memo, key) ) {
                //	memo[key] = func.apply(this, arguments);
                //}
                //return memo[key];

                return _.has(memo, key) ? memo[key] : (memo[key] = func.apply(this, arguments));
            };
        };


        ////////////////////////////////////////////////////////////////////////////////
        // Features: cachedMemoize
        //

        // TODO convert to either an underscore mixin or a jquery extension
        // This a copy of jQuery 1.11.1's $.when function.
        // I've changed this to run through each promise regardless of failure.
        // The master function will fail at the end when any of the promises fails.
        //
        // This first creates an array of promises (_.map...)
        //     These promises are promises to validate each property
        // We then return a master promise (when.apply...)
        //     This master promise promises to return on completion of all
        //     promises in the array. This only completes successfully
        //     if all promises in the array completes successfully.
        var whenAllFinish = function( subordinate /* , ..., subordinateN */ ) {
            var i = 0,
                failed = false,//added
                resolveValues = [].slice.call( arguments ),//modified
                length = resolveValues.length,

            // the count of uncompleted subordinates
                remaining = length !== 1 || ( subordinate && jQuery.isFunction( subordinate.promise ) ) ? length : 0,

            // the master Deferred. If resolveValues consist of only a single Deferred, just use that.
                deferred = remaining === 1 ? subordinate : jQuery.Deferred(),

            // Update function for both resolve and progress values
                updateFunc = function( i, contexts, values ) {
                    return function( value ) {
                        contexts[ i ] = this;
                        values[ i ] = arguments.length > 1 ? [].slice.call( arguments ) : value;
                        if ( values === progressValues ) {
                            deferred.notifyWith( contexts, values );

                        } else if ( !(--remaining) ) {
                            if(failed) { //added
                                deferred.rejectWith(contexts, values); //added
                            } else { //added
                                deferred.resolveWith( contexts, values );
                            } //added
                        }
                    };
                },

                progressValues, progressContexts, resolveContexts,

                failFunc = function() { //added
                    failed = true; //added
                    --remaining; //added
                }; //added

            // add listeners to Deferred subordinates; treat others as resolved
            if ( length > 1 ) {
                progressValues = new Array( length );
                progressContexts = new Array( length );
                resolveContexts = new Array( length );
                for ( ; i < length; i++ ) {
                    if ( resolveValues[ i ] && jQuery.isFunction( resolveValues[ i ].promise ) ) {
                        var temp = resolveValues[ i ].state();
                        resolveValues[ i ].promise()
                            .done( updateFunc( i, resolveContexts, resolveValues ) )
                            .fail( failFunc ) //modified
                            //.fail( function() { failFunc(); } ) //modified
                            .progress( updateFunc( i, progressContexts, progressValues ) );
                    } else {
                        --remaining;
                    }
                }
            }

            // if we're not waiting on anything, resolve the master
            if ( !remaining ) {
                if(failed) { //added
                    deferred.rejectWith(resolveContexts, resolveValues); //added
                } else { //added
                    deferred.resolveWith( resolveContexts, resolveValues );
                } //added
            }

            return deferred.promise();
        };


        ////////////////////////////////////////////////////////////////////////////////
        // Features: onReady
        //

        /**
         * Contains all jQuery Defer objects for View Objects
         * @type {Array.<string,$.Deferred>}
         * @memberof module:EntityView/View
         */
        var viewReadyDefers = [];

        /**
         * Contains all callbacks for @see viewReadyDefers
         * @type {Array.<function>}
         * @memberof module:EntityView/View
         */
        var viewReadyGlobalCallbacks = [];

        var getReadyDefer,
            globalReadyFunction;

        /**
         * Gets the jQuery defer object for the specified view ID.
         * @param viewId
         * @returns $.Deferred
         */
            //TODO[LOW] allow passing html elements so we can check which form it is within. this way we don't need to know the viewId
            //TODO[LOW] allow passing view object
        getReadyDefer = function(viewId) {
            if(!viewReadyDefers[viewId]) {
                viewReadyDefers[viewId] = $.Deferred();

                // add current callback functions to this defer
                _.each(viewReadyGlobalCallbacks, function(callbackFunction) {
                    viewReadyDefers[viewId].done(callbackFunction);
                });
            }
            return viewReadyDefers[viewId];
        };

        /**
         * Adds a onReady callback function to all View objects.
         * @param {function} callbackFunction
         * @memberof module:EntityView/View
         */
        globalReadyFunction = function(callbackFunction) {
            if(!(callbackFunction instanceof Function)) {
                throw new Error("First argument must be a function.");
            }

            // check to see if this is global callbacks
            if(arguments.length === 1) {
                viewReadyGlobalCallbacks.push(callbackFunction);

                // add callback to existing viewready deferred objects
                _.each(viewReadyDefers, function(defer) {
                    defer.done(arguments[0]);
                });

                return;
            }

            var defer;
            for(var i = 1; i < arguments.length; i++) {
                // these are for specific views
                defer = getReadyDefer(arguments[i]);
                defer.done(callbackFunction);
                //return defer.promise();
            }

        };

        ////////////////////////////////////////////////////////////////////////////////
        // Utility methods
        //

        /**
         * Regular expression to strip away all markup from the CKEditor.
         * @type {RegExp}
         * @memberof module:EntityView/View
         */
        var richTextRegex = new RegExp("(</?\\w+(((\\s|\\n)+\\w+((\\s|\\n)*=(\\s|\\n)*(?:\".*?\"|'.*?'|[^'\">\\s]+))?)+(\\s|\\n)*|(\\s|\\n)*)/?>)|&nbsp;", "gi");

        /**
         * Executes all callbacks when DOM is ready.
         * Errors do not cause the callbacks to be aborted.
         * //TODO finish documentation
         * @memberof module:EntityView/View
         */
        //TODO[LOW] consider moving this to it's own module
        var faultTolerantDomReady = (function() {

            var isBrowser = typeof window !== "undefined" && window.document,
                doc = isBrowser ? document : null,
                hasCallbacksRun = false,
                readyCalls = [];

            function runCallbacks(callbacks, doc) {
                var i;
                var errors = [];
                for (i = 0; i < callbacks.length; i += 1) {
                    try {
                        callbacks[i](doc);
                    } catch(e) {
                        errors.push(e);
                    }
                }

                hasCallbacksRun = true;

                if(errors.length > 0) {
                    console.error("faultTolerantDomReady errors", errors);
                    throw new Error("faultTolerantDomReady encountered " + errors.length + " error(s).");
                }
            }

            domReady(function() {
                var callbacks = readyCalls;
                readyCalls = [];
                runCallbacks(callbacks, doc);
            });

            return function(callback, context) {
                //var isPageLoaded = !isBrowser;

                var callbackFunction = _.bind(callback, (context ? context : {}));
                if (hasCallbacksRun) {
                    callbackFunction(doc);
                } else {
                    readyCalls.push(callbackFunction);
                }
                return domReady;
            };

        })();

        /**
         * jQuery plugin utility to apply CSS animations.
         * TODO[LOW] do we really want to register a global jQuery plugin here?
         * @param {string} animationClassName
         * @returns {jQuery} this
         * @memberof module:EntityView/View
         */
        $.fn.applyCSSAnimation = function(animationClassName) {
            // the animation end event doesn' always run. not sure why. :(
            var obj = this;
            return obj
                .one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function(){
                    obj.removeClass(animationClassName);
                })
                .addClass(animationClassName);
        };

        /**
         * Enumeration of display types
         * @enum {{ENABLED: {name: string, isEditable: boolean, isVisible: boolean, cssClassName: string}, DISABLED: {name: string, isEditable: boolean, isVisible: boolean, cssClassName: string}, READONLY: {name: string, isEditable: boolean, isVisible: boolean, cssClassName: string}, HIDDEN: {name: string, isEditable: boolean, isVisible: boolean, cssClassName: string}, getDisplayType: Function}}
         * @private
         * @memberof module:EntityView/View
         */
        var _displayType = {
            ENABLED: {
                name: "Editable",
                isEditable: true,
                isVisible: true,
                cssClassName: "conditionalDisplay-Enabled"
            },
            DISABLED: {
                name: "Disabled",
                isEditable: false,
                isVisible: true,
                cssClassName: "conditionalDisplay-Disabled"
            },
            READONLY: {
                name: "Read-Only",
                isEditable: false,
                isVisible: true,
                cssClassName: "conditionalDisplay-ReadOnly"
            },
            HIDDEN: {
                name: "Hidden",
                isEditable: false,
                isVisible: false,
                cssClassName: "conditionalDisplay-Hidden"
            },
            getDisplayType: function (displayTypeName) {
                if (!displayTypeName) {
                    return undefined;
                }
                var displayType = _displayType[displayTypeName.toUpperCase()];
                if (displayType === undefined) {
                    throw new Error("Display Type not recognized: " + displayTypeName);
                }
                return displayType;
            }
        };

        /**
         * Enumeration of ALD form post persistence types.
         * Settings for this are located in the ALD properties dialog in the view editor.
         * @type {{CLEAR: {name: string}, IGNORE: {name: string}, SAVE: {name: string}, getPersistenceType: Function}}
         * @private
         * @memberof module:EntityView/View
         */
        var _persistenceType = {
            CLEAR: {
                name: "clear"
            },
            IGNORE: {
                name: "ignore"
            },
            SAVE: {
                name: "save"
            },
            getPersistenceType: function (persistenceTypeName) {
                if (!persistenceTypeName) {
                    return undefined;
                }
                var persistenceType = _persistenceType[persistenceTypeName.toUpperCase()];
                if (persistenceType === undefined) {
                    throw new Error("Display Type not recognized: " + persistenceTypeName);
                }
                return persistenceType;
            }
        };

        /**
         * Creates an escaped regex string which is used later to help remove the currency symbol, group separator,
         * and whitespace from currency string.
         * @param {string} str
         * @returns {?string}
         * @private
         * @memberof module:EntityView/View
         */
        var currencyEscapeStringRegex = function(str) {
            return str.replace(/([.?*+^$[\]\\(){}|-])/g, "\\$1");
        };

        /**
         * @name currencyOptions
         * @param {string} CurrencyGroupSeparator
         * @param {string} CurrencySymbol
         * @param {string} NegativeSign
         * @param {string=} CurrencyNegativePattern
         * @param {string} CurrencyDecimalSeparator
         * @param {number} CurrencyDecimalDigits
         * @memberof module:EntityView/View
         */

        /**
         * Converts a currency string to a float value
         * @param {string} currencyString
         * @param {currencyOptions} currencyOptions
         * @returns {?float}
         * @private
         * @memberof module:EntityView/View
         */
        var convertCurrencyStringToFloat = function(currencyString, currencyOptions) {
            if(!currencyString) {
                return null;
            }

            var currencyGroupSeparator   = currencyOptions.CurrencyGroupSeparator,
                currencySymbol           = currencyOptions.CurrencySymbol,
                negativeSign             = currencyOptions.NegativeSign,
            //currencyNegativePattern  = currencyOptions.CurrencyNegativePattern,
                currencyDecimalSeparator = currencyOptions.CurrencyDecimalSeparator,
                currencyDecimalDigits    = currencyOptions.CurrencyDecimalDigits;

            // Remove currency symbol, group separator, and whitespace
            var regexString = "[\\s" + currencyEscapeStringRegex(currencyGroupSeparator)
                + "]|" + currencyEscapeStringRegex(currencySymbol);
            var numString = currencyString.replace(new RegExp(regexString,"g"),'');

            //
            // Determine negativity and remove negative symbols
            //
            var isNegative;

            // if first char is ( and last char is )
            if(numString.charAt(0) === '(' && numString.charAt(numString.length-1) === ')') {
                isNegative = true;
                numString = numString.substr(1, numString.length-2);
            }
            // if first character is currencyOptions.NegativeSign
            else if(numString.charAt(0) === negativeSign) {
                isNegative = true;
                numString = numString.substr(1);
            }
            // if last character is currencyOptions.NegativeSign
            else if(numString.charAt(numString.length-1) === negativeSign) {
                isNegative = true;
                numString = numString.substr(0, numString.length-1);
            }

            //
            // Reconstruct number using decimal point .
            //
            numString = numString
                .split(currencyDecimalSeparator)
                .join('.');

            var num = currencyDecimalDigits === 0 ? parseInt(numString) : parseFloat(numString);

            //
            // re-establish negativity
            //
            if(isNegative) {
                num = -num;
            }

            return num;
        };

        /**
         * Gets Moment. This uses the
         * @param {string} value
         * @param {string=} format
         * @param {string=} dateFormat
         * @returns {Moment}
         * @private
         * @requires module:moment
         * @see {@link http://momentjs.com/|moment.js}
         * @memberof module:EntityView/View
         */
        // require(["moment"]); is added to Property object if it's type is "Date"
        var getMoment = function(value, format, dateFormat) {
            //TODO[MEDIUM] This is dodgy, but works because this function only gets called on user input.
            // If between page load and moment module load, the user inputs, they will relieve an error because
            // it has not finished loading.
            var Moment = require("moment");
            var moment = Moment(value, dateFormat || Moment.ISO_8601);

            switch (format) {
                case 'shortTime':
                case 'mediumTime':
                case 'longTime':
                    // Date does not matter, only time, so set to unix epoch
                    moment.set({'year': 1970, 'dayOfYear': 1});
                    break;
            }
            return moment;
        };

        /**
         * Compares two moments
         * @param {Moment} momentA
         * @param {Moment} momentB
         * @returns {boolean}
         * @private
         * @requires module:moment
         * @see {@link http://momentjs.com/|moment.js}
         * @memberof module:EntityView/View
         */
        var isSameMoment = function(momentA, momentB) {
            var Moment = require("moment");

            if(Moment.isMoment(momentA) && Moment.isMoment(momentB)) {
                return momentA.isSame(momentB);
            }
            // should cover all other cases. null, undefined, etc.
            return _.isEqual(momentA, momentB);
        };

        /**
         * Converts from "containerName_container" to "containerName"
         * @param {string} containerID
         * @returns {string}
         * @private
         * @memberof module:EntityView/View
         */
        var convertContainerIdToFQN = _.memoize(function(containerId) {
            return containerId.substring(1, containerId.length-10);
        });

        /**
         * Determine if value is empty based on the controlType of and ALD.
         * @param {*} currentValue
         * @param {string} controlTypeName
         * @returns {boolean}
         * @memberof module:EntityView/View
         */
        var isValueEmptyForControlType = function(currentValue, controlTypeName) {
            if(currentValue === null || currentValue === undefined) {
                return true;
            }

            switch (controlTypeName) {
                case "dataentrygrid":
                case "dataEntry":
                case "inlinecdtview":
                case "simpledocumenttable":
                case "checkboxSelection":
                case "chooserSelection":
                    return !currentValue || currentValue.length === 0;
                case "richtext":
                    // Need to strip away all markup from the CKEditor.
                    currentValue = currentValue.replace(richTextRegex, "");
                    return currentValue === "";
                case "calendar":
                    return currentValue === "";
                case "singleline":
                    if (controlTypeName === "Currency") {
                        return currentValue === "";
                    }
                /* falls-through */
                default:
                    return currentValue === null || currentValue === undefined;
            }
        };

        /**
         * //TODO: this is kind of horrible.
         * Parses the form value to a JavaScript value based on ALD information.
         * @param {*} valueToCompare
         * @param {Object} valueMetadata
         * @returns {*}
         * @memberof module:EntityView/View
         */
        var parseFormValue = function(valueToCompare, valueMetadata) {
            var value = valueToCompare,
            	lcValue = typeof value == "string" ? value.toLowerCase() : null;

            var controlType        = valueMetadata.controlType,
                hidden             = valueMetadata.hidden,
                type               = valueMetadata.type,
                format             = valueMetadata.format,
                dateFormat         = valueMetadata.dateFormat,
                currencyFormat     = valueMetadata.currencyFormat,
                fullyQualifiedName = valueMetadata.fullyQualifiedName,
                containingObject   = valueMetadata.containingObject;

            switch (controlType) {
                case "checkbox":
                    if (hidden) {
                        if (value !== null && value !== "") {
                            value = (lcValue === "yes" || lcValue === "true" || lcValue === "on");
                        }
                    } else {
                        value = (lcValue === "yes" || lcValue === "true" || lcValue === "on");
                    }
                    break;
                case "radio":
                case "select":
                    // returns true, false, null
                    if (value && type === "Boolean") {
                        value = (lcValue === "yes" || lcValue === "true" || lcValue === "on");
                    }
                    break;
                case "documentContentFileUploadView":
                    // if a file is set to upload, return it's value

                    // otherwise, grab the view link href
                    if (!value) {
                        var viewLink = $(
                            'a[id="' + PortalTools.escapeJQuerySelector(fullyQualifiedName) + '_viewLink"]',
                            containingObject
                        );
                        if (viewLink.length && viewLink.parent().css("display") !== "none") {
                            value = viewLink.attr("href");
                        } else {
                            value = null;
                        }
                    }
                    break;
                case "simpledocument":
                    // if a file is set to upload, return it's value

                    // otherwise, grab the poref
                    if (!value) {
                        value = $(
                            'input.inputControl[name="' + fullyQualifiedName + '"]',
                            containingObject
                        ).val();
                    }
                    if (value === "" || value === nullOIDString) {
                        value = null;
                    }

                    // otherwise, grab the view link href
                    //if(!value) {
                    //    value = $(
                    //        'a[id$="_display"]',
                    //        this.containingObject
                    //    );
                    //    if(value.length) {
                    //        value = value.attr("href");
                    //    } else {
                    //        value = null;
                    //    }
                    //}
                    break;
                case "richtext":
                case "textarea":
                    // the persisted values have \r\n but for some reason form values only has \n
                    value = value.replace(/\r?\n/g, '\r\n');
                    if (value === "") {
                        value = null;
                    }
                    break;
                case "singleline":
                    value = value.replace(/\r?\n/g, '\r\n');
                    if (value === "" && type !== "Currency") {
                        value = null;
                    } else {
                        switch (type) {
                            case "Currency":
                                value = convertCurrencyStringToFloat(value, currencyFormat);
                                break;
                            case "Integer":
                                value = parseInt(value, 10);
                                break;
                            case "Float":
                            case "Double":
                                value = parseFloat(value);
                                break;
                        }
                    }
                    break;
                case "simpledocumenttable":
                case "chooserSelection":
                case "dataEntry":
                case "inlinecdtview":
                    if (type === "Set" && value instanceof Array) {
                        // old way of denoting no value: value === nullOIDString
                        value = _.compact(value);
                        //value = _.without(value, "", nullOIDString, null);
                    }
                    else if (type === "Entity" && value === nullOIDString) {
                        value = null;
                    }
                    break;
                case "singleEntityDataEntry":
                case "singleEntitySelection":
                case "singleDocumentUploadEditView":
                    if (value === "" || value === nullOIDString) {
                        value = null;
                    }
                    break;
                case "calendar":
                    value = !value ? null : getMoment(value, format, dateFormat);
                    break;
                case "checkboxSelection":
                case "documentTableUploadEditview":
                case "dataentrygrid":
                    if (value instanceof Array) {
                        value = _.compact(value);
                    }
                    break;
                case "dataSnapShot":
                    DEBUG.warn(controlType + " control type is not supported.");
                    break;
                default:
                    throw new Error("getParsedFormValue from control type '" + controlType + "' is not implemented.");
            }

            return value;
        };




        /**
         * Sets up various project editor stuff.
         * TODO[LOW] the following really belongs on the Project Editor page, not here.
         * @param {View} view
         * @param {Navigation} navigation
         * @memberof module:EntityView/View
         */
        var projectEditorFunctionality = function(view, navigation) {
            if(view.viewLocation !== "projectEditor") {
                return;
            }

            var initializeBustOutOfFrameset,
                initializeSpecialCaseRedirect,
                initializeTabbingIndex;


            // PRS 18,473 18473
            // Project.formHandler() sets a client cookie "bustOutOfFrameset" during the form post for hide errors
            // If this cookie exists, reset it first and then get out of the frameset. The cookie + reset is
            // done so that a back browser action does not fall into a recursive loop of busting out of the frame
            // and never be able to traverse the browser history.
            // TODO[LOW] Problem - Hitting Browser back takes to the frameset page twice, if the show and then hide errors was selected.
            /**
             * //FIXME
             */
            initializeBustOutOfFrameset = function() {
                if (PortalTools.readCookie('bustOutOfFrameset') === 'true') {
                    DEBUG.warn("BustOutOfFrameset had to execute.");
                    //scopeAll - variable set in generateHTML script on Project/ProjectEditor
                    //secureCookie - variable set in generateHTML script on Project/ProjectEditor
                    PortalTools.createCookie(
                        'bustOutOfFrameset',
                        'false',
                        null,
                        globals.get("scopeAll"),
                        null,
                        globals.get("secureCookie")
                    );
                    try {
                        globals.get("bustOutOfFrameset")();
                    } catch(error) {
                        DEBUG.error("Bust out of frameset failed");
                    }
                }
            };

            //DJB PRS 17499 17,499
            //If we're on the first page of the SmartForm and it's a creation type URL, we
            //need to look for the special cookie that tells us if we need to *replace* the
            //current URL with an edit mode URL.  Once this is done, we will never come back
            //to the creation version of the URL, so the existence of the cookie is irrelevant.
            //Note that having two SmartForms going at the same time in two browsers in the
            //same process could be a problem.  This cookie is wiped out whenever a new project
            //is started from a project creator component.
            /**
             * //FIXME
             * @param navigation
             */
            initializeSpecialCaseRedirect = function(navigation) {
                if (document.location.href.toLowerCase().indexOf('caller=projectcreator') === -1){
                    return;
                }
                // if the cookie 'InitialCreationWork' exists, then run this code.
                var initialCreationWork = PortalTools.readCookie('InitialCreationWork');
                if(initialCreationWork) {
                    DEBUG.info("specialCaseRedirect is in progress");
                    navigation.navigate({
                        mode : "specialCaseRedirect",
                        submissionFunction: function() {
                            // After using replace() the current page will not be saved in session History,
                            // meaning the user won't be able to use the back button to navigate to it.
                            window.location.replace(decodeURIComponent(initialCreationWork));
                        }
                    });
                }
            };

            // remove tabbing ability from bottom bar except for the continue button
            // this is supposedly ok because the user can still access the top menu bar.
            // Also, from there allow shift+tab to take the user back to the last element.
            /**
             * //FIXME
             * @param navigationContainerSelector
             */
            initializeTabbingIndex = function(navigationContainerSelector) {
                $("a, :input:not(:hidden, .tabbableButton)", navigationContainerSelector)
                    .each(function() {
                        this.tabIndex = -1;
                    });
            };

            view.ready(function() {
                initializeBustOutOfFrameset();
                initializeSpecialCaseRedirect(navigation);
                initializeTabbingIndex("#CommandBarBottom");
            });

        };


        /**
         * MessageCenter is used to keep track of viewControlUpdated events triggered on the document DOM element.
         * These events are executed by various ALDs. Normally we can see form events on the ALD container, but
         * some ALD's don't bubbly up events to the container and only register their changes to the document DOM
         * element.
         * @param {View} view
         * @constructor
         * @memberof module:EntityView/View
         */
        function MessageCenter(view) {
            _.bindAll(this,
                'addBodyTrigger',
                'viewControlUpdatedHandler',
				'updateViewMetadataHandler'
            );

            this.view = view;

            this.messageObject = $({});

            this.propertiesWatched = []; // Array of strings

            $(root.document).on("viewControlUpdated", this.viewControlUpdatedHandler);
            $(root.document).on("updateViewMetadata", this.updateViewMetadataHandler);
        }

        /**
         * Adds a ViewAsset for the MessageCenter to watch.
         * @param {ViewAsset} viewAsset
         */
        MessageCenter.prototype.addBodyTrigger = function (viewAsset) {
            //DEBUG.info("Watching: " + property.fullyQualifiedName);
            this.propertiesWatched.push(viewAsset.fullyQualifiedName);
        };

        /**
         * TODO[LOW] get all params in here
         * @name viewControlUpdatedControlInfoObject
         * @param {string} name
         */

        /**
         * Event Handler for registered viewControlUpdated events
         * TODO[LOW] This function is currently an unfortunate necessity.
         * Ideally we shouldn't have this. We should change how the various custom properties (CDTs,
         * etc) trigger the containing objects (instead of using viewControlUpdatedHandler, it triggers the
         * property container.
         * @param {Event} event
         * @param {viewControlUpdatedControlInfoObject} controlInfo
         */
        MessageCenter.prototype.viewControlUpdatedHandler = function (event, controlInfo) {
            // This is so we ignore the property updates we don't care about
            if (_.contains(this.propertiesWatched, controlInfo.name)) {
                DEBUG.custom({type: "event", data: "MESGCENTER EVENT (change): " + controlInfo.name});
                event.name = controlInfo.name;
                var property = this.view.getProperty(controlInfo.name);
                property.formObjects = property.findFormDomObjects();
                Property.prototype.changeHandler.apply(property, arguments);
            }
        };
		
		/*
		* Handles requests to update the external properties of the view through the new metadata
		* and execute those rules depending on these attributes
		* @param {Event} event
		* @param {MetadataJsonObject} metadata
		* @param {string} attrName - Fully qualified name of the parent ald from which this external property is dereferenced
		* @param {string} itemOID - OID of the set item that is deleted. Null for single entity references
		*/       
        MessageCenter.prototype.updateViewMetadataHandler = function(event, metadata, attrName, itemOID) {
        	var view = this.view;
        	var externalProperties = {};
        	var rulesToExecute = [];
        	
        	_(view.externalProperties).each(function(property){
        		//'attrName' is the FQN of the CDT containing the dereferenced attributes
        		//So the FQN of the external attribute should contain 'attrName' as a part of it
        		if(property.fullyQualifiedName.indexOf(attrName) != -1){
        			_(view.rules).each(function(rule){
	        			if(_.contains(rule.triggerProperties, property)){
	        				rulesToExecute.push(rule);
	        				if(externalProperties[property.fullyQualifiedName] === undefined){
	        					externalProperties[property.fullyQualifiedName] = [];
	        				}
	        				externalProperties[property.fullyQualifiedName].push(rule);
	        				return false;
	        			}
	        		});
        		}
        	});
        	
        	//Bail out when there are no dependencies to care about
        	if(externalProperties.length == 0){
        	 	return;  
        	}
        	
        	if(metadata){
	        	//Update the external properties with the new metadata
	        	_(externalProperties).each(function(assocRules, externalProperty){
	        		_(metadata.externalAttributes).each(function(attr){
	        			if(attr.fullyQualifiedName == externalProperty){
	        				var currProperty = view.getProperty(externalProperty);
	        				var newProperty = new ExternalProperty(attr, view);
	        				view.externalProperties[_.indexOf(view.externalProperties, currProperty)] = newProperty;
	        				view.properties[_.indexOf(view.properties, currProperty)] = newProperty;
	        				_(assocRules).each(function(triggerRule){
	        					triggerRule.triggerProperties[_.indexOf(triggerRule.triggerProperties, currProperty)] = newProperty;
	        				});
	        			}
	        		});
	        	});
	        } else {
	        	//We are here because of deleting entity reference.
	        	//Hence, null out any external property values dereferenced of it.
	        	_(externalProperties).each(function(assocRules, externalProperty){
	        		var prop = view.getProperty(externalProperty);
	        		if(!Array.isArray(prop.persistedValue) || !itemOID){
		        		prop.persistedValue = {
		        			raw : null,
		        			string : ""
		        		};
		        	} else {
		        		prop.persistedValue = _.filter(prop.persistedValue, function(x){
		        			return x.setItemPoref != itemOID;
		        		});
		        	}
	        	});
	        }
        	
        	//Execute the rules we care about
        	_(rulesToExecute).each(function(x) {
            	x.attachUpdateDisplayTrigger();
                x.executeLogic();
            });
		}

		/**
         * @name regions
         * @param {number} layoutOrder -
         * @param {string} parentRegionId -
         * @param {boolean} readOnly -
         * @param {string} fullyQualifiedName -
         * @param {string} currentDisplayType -
         * @param {string} ruleName -
         * @param {string} rulePoref -
         * @param {Object} displayTypes -
         * @param {string} displayTypes.ruleTrue -
         * @param {string} displayTypes.ruleFalse -
         */

        /**
         * @name externalAttributes
         * @param {string} fullyQualifiedName -
         * @param {string} qualifiedName -
         * @param {string} targetTypeName -
         * @param {string} type -
         * @param {Object} persistedValue -
         * @param {*} persistedValue.raw -
         * @param {string} persistedValue.string -
         */

        /**
         * @name internalAttributes
         * @param {!string} fullyQualifiedName -
         * @param {string} parentRegionId -
         * @param {?string} qualifiedName -
         * @param {?string} targetTypeName -
         * @param {string} poref -
         * @param {!string} type -
         * @param {!string} controlType -
         * @param {?number} layoutOrder -
         * @param {!boolean} hidden -
         * @param {!boolean} readOnly -
         * @param {!boolean} required -
         * @param {string} currentDisplayType -
         * @param {?Object} persistedValue -
         * @param {*} persistedValue.raw -
         * @param {?string} persistedValue.string -
         * @param {?string=} format -
         * @param {?Object=} currencyFormat -
         * @param {!number} currencyFormat.CurrencyDecimalDigits -
         * @param {!string} currencyFormat.CurrencyDecimalSeparator -
         * @param {!string} currencyFormat.CurrencyGroupSeparator -
         * @param {!number[]} currencyFormat.CurrencyGroupSizes -
         * @param {!string} currencyFormat.CurrencySymbol -
         * @param {!string} currencyFormat.NegativeSign -
         * @param {!number} currencyFormat.CurrencyNegativePattern -
         * @param {!number} currencyFormat.CurrencyPositivePattern -
         * @param {?string=} ruleName -
         * @param {?string=} rulePoref -
         * @param {?boolean=} validateWhenNonEditable -
         * @param {?boolean=} persistanceWhenNonEditable -
         * @param {?Object=} displayTypes -
         * @param {!string} displayTypes.ruleTrue -
         * @param {!string} displayTypes.ruleFalse -
         */

        /**
         * @name rules
         * @param {string} name -
         * @param {string} poref -
         * @param {boolean} currentRuleResult -
         * @param {Object} logic -
         * @param {Object} triggerNames -  //TODO change to string of the fullyQualifiedName
         * @param {string} triggerNames.targetType -
         * @param {string} triggerNames.qualifiedName -
         * @param {string} triggerNames.fullyQualifiedName -
         */

        /**
         * @name formData
         * @param {!string} formData.viewId -
         * @param {string} formData.poref -
         * @param {string} formData.name -
         * @param {!string} formData.viewLocation -
         * @param {string} formData.targetType -
         * @param {string} formData.targetBaseType -
         * @param {boolean} formData.readOnly -
         * @param {string} formData.mode - //TODO not necessary here?
         * @param {boolean} formData.isNew -
         * @param {boolean} formData.isSecureRendering -  //TODO this should be a sitewide setting, not a view setting
         * @param {string} formData.eTypePOREF -
         * @param {Array.<regions>} formData.regions -
         * @param {Array.<externalAttributes>} formData.externalAttributes -
         * @param {Array.<internalAttributes>} formData.internalAttributes -
         * @param {Array.<rules>} formData.rules -
         */

        /**
         * JavaScript object representation of an EntityView
         * @param {!formData} formData -
         * @param {Navigation=} navigation -
         * @returns {View}
         * @constructor
         * @class
         * @alias module:EntityView/View
         */
        function View(formData, navigation) {
            // This won't work. Automatically generated views (CDTs with no defined views) will
            // return a null value for the viewID. :(
            //if(!formData.viewId) {
            //    throw new Error("viewId is required.");
            //}

            var view = this;

            _.bindAll(this,
                'addError',
                'clearError',
                'displayErrors',
                'findDirtyObjects',
                'findErrorField',
                'resetErrorField',
                'findFormMetadataField',
                'findFormDomObject',
                'focus',
                'getFormMetadata',
                'getFormObject',
                'getProperties',
                'getProperty',
                'getPropertyByInputName',
                'getViewLocation',
                'getRegion',
                'getRegions',
                'getRule',
                'getRules',
                'getViewAssets',
                'getViewId',
                'initializeSafariCDCachingWorkaround',
                'initializeViewAsset',
                'isDirty',
                'isReadOnly',
                'on',
                'onSubmit',
                'ready',
                'trigger',
                'validate',
                'updateFormMetadata',
                'updateMetadata'
            );

            this.errorArray     = [];
            this.viewReadyDefer = getReadyDefer(formData.viewId); //$.Deferred();
            this.messageCenter  = new MessageCenter(this);

            // Initialize cache values
            this.cache = {
                hasFormChanged : false
            };
            this.cache.region =
                this.cache.property =
                    this.cache.rule = _.now();

            // TODO[LOW] figure out which variables are necessary and which are optional
            this.viewId            = formData.viewId;
            this.poref             = formData.poref;
            this.name              = formData.name;
            this.eTypePOREF        = formData.eTypePOREF;
            this.targetType        = formData.targetType;
            this.targetBaseType    = formData.targetBaseType;
            this.readOnly          = formData.readOnly;
            this.projectEditorMode = formData.mode;
            this.isNew             = formData.isNew;

            // View Types -> search for locations of wom.putContext("EntityViewLocation", <value>, true);
            this.viewLocation = formData.viewLocation;

            // TODO[LOW] do we really want the secure rendering bit here?
            // seems like secure rendering information belongs elsewhere since it is not a view setting
            // check to see if we are using secure rendering anywhere.

            // this isn't a view setting, but a site setting
            this.isSecureRendering = formData.isSecureRendering;

            // This has no bearing on rendering. If isSecureRendering -> true then we render as textarea
            // This is currently for informational purposes only.
            this.secureRenderingExclusionForNotesAsStr = formData.secureRenderingExclusionForNotesAsStr;

            DEBUG.custom({
                indent: 1,
                context: this,
                data: function() {
                    var prefix = "\n";
                    return "Parameters:" +
                        prefix + "viewId            : " + this.viewId +
                        prefix + "name              : " + this.name +
                        prefix + "targetType        : " + this.targetType +
                        prefix + "targetBaseType    : " + this.targetBaseType +
                        prefix + "readOnly          : " + this.readOnly +
                        prefix + "projectEditorMode : " + this.projectEditorMode +
                        prefix + "isNew             : " + this.isNew +
                        prefix + "eTypePOREF        : " + this.eTypePOREF +
                        prefix + "isSecureRendering : " + this.isSecureRendering +
                        prefix + "viewLocation      : " + this.viewLocation +
                        prefix + "secureRenderingExclusionForNotesAsStr : " + this.secureRenderingExclusionForNotesAsStr;
                }
            });


            ////////////////////////////////////////////////////////////////////////////////
            // Initialize
            //
            faultTolerantDomReady(function() {
                try {
                    DEBUG.log("Setting up View DOM objects.");

                    // TODO[LOW] grab a container object?
                    // for forms, it could be the form object, but in read-only, there is nothing suitable.
                    // Not suitable due to the container being a span and the span gets broken out of due to bad HTML.

                    this.formObject = this.findFormDomObject();

                    if (this.formObject.is("form")) {
                        this.errorField = this.findErrorField();
                        this.formMetadataField = this.findFormMetadataField();
                        this._webrCDHash        = $('input[name="_webrCDHash"]',        this.formObject);
                        this._webrCDCurrentHash = $('input[name="_webrCDCurrentHash"]', this.formObject);

                        // TODO[LOW] is this valid in non-projectEditor context?
                        this.initializeSafariCDCachingWorkaround();

                        this.focus();
                    }

                    if (navigation) {
                        navigation.registerView(this);
                    }

                    this.formObject.on("change", function () {
                        view.cache.hasFormChanged = true;
                    });

                    //TODO[LOW] this doesn't really belong here
                    projectEditorFunctionality(this, navigation);

                    this.dirtyObjects = this.findDirtyObjects();

                    deprecated.registerView(this);
                }
                catch(error) {
                    this.viewReadyDefer.rejectWith({}, this);
                    throw new Error("Error initializing view: " + this.viewId);
                }

            }, this);

            // Sort the view assets by layout order. Not necessary, but nice when debugging
            formData.internalAttributes = _.sortBy(formData.internalAttributes, function (attribute) {
                return attribute.layoutOrder;
            });

            // We check if this is an old type of display which renders the html incorrectly for phrasing and
            // flow content. (rendering as div vs span and visa versa.

            // Theoretically, the #_webr_EntityView element will always be available (even before dom is ready) since
            // this script occurs at the end of the view that it is grabbing. However, this is not nice since this
            // forces a tight coupling.
            var _webr_EntityView = $("#_webr_EntityView");
            var isNewViewType = _webr_EntityView.length === 0 ? true : _webr_EntityView.prop("tagName").toLowerCase() === "div";
            if(!isNewViewType) {
                DEBUG.warn("EntityView is old view type.");
            }

            // NOTE: This belongs between the two faultTolerantDomReady so that the domReady created in these functions
            // are executed in correct order.
            if(isNewViewType && this.viewReadyDefer.state() !== "rejected") {
				var childMetadata = metadataBucket.get(this.poref);
                if(childMetadata) {
                    formData.regions = _.union( formData.regions, _.flatten(_.map(childMetadata, function(metadata) {
                        return metadata.regions;
                    })));
                    formData.internalAttributes = _.union( formData.internalAttributes, _.flatten(_.map(childMetadata, function(metadata) {
                        return metadata.internalAttributes;
                    })));
                    formData.externalAttributes = _.union( formData.externalAttributes, _.flatten(_.map(childMetadata, function(metadata) {
                        return metadata.externalAttributes;
                    })));
                	formData.rules = _.union( formData.rules, _.flatten(_.map(childMetadata, function(metadata) {
                        return metadata.rules;
                    })));
                }

                // These must be initialized in this order as each one relies on the previous.
                this.updateMetadata(formData);
                this.rules = this.initializeViewAsset(Rule, formData.rules, view);
            }

            faultTolerantDomReady(function() {
                if(this.viewReadyDefer.state() === "rejected") {
                    return;
                }
                try {
                    //FIXME change to use metadata, then move this to the above faulttolerantdomready
                    // though, if one of the properties in the metadata fail, it would not capture this...
                    // perhaps in it's catch block add it to dirtyObjectArray
                    //this.dirtyObjects = this.findDirtyObjects();
                    //this.dirtyObjectArray = this.findDirtyObjectArray();

                    this.viewReadyDefer.resolveWith(this);
                }
                catch(error) {
                    this.viewReadyDefer.rejectWith({}, this);
                    throw new Error("Error initializing view: " + this.viewId);
                }

            }, this);

            DEBUG.log(function() {
                var prefix = "\n";
                return "ViewAssets Generated:" +
                    prefix + "regions    : " + (view.regions    ? view.regions.length    : 0) +
                    prefix + "properties : " + (view.properties ? view.properties.length : 0) +
                    prefix + "rules      : " + (view.rules      ? view.rules.length      : 0);
            });

            ////////////////////////////////////////////////////////////////////////////////
            // register and return
            //

            viewInstances.push(this);
            return this;
        }
			
		View.prototype.updateMetadata = function(metadata) {
			// These must be initialized in this order as each one relies on the previous.
			this.regions = this.initializeViewAsset(Region, metadata.regions, this);
			this.internalProperties = this.initializeViewAsset(InternalProperty, metadata.internalAttributes, this);
			this.externalProperties = this.initializeViewAsset(ExternalProperty, metadata.externalAttributes, this);
			this.properties = _.union(this.internalProperties, this.externalProperties);
			//this.rules = this.initializeViewAsset(Rule, metadata.rules, this);
		}	
				
        View.prototype.findDirtyObjects = function() {
            var changeTrackingFormInputs = $('input[name="_webrCDFormVariableNames"]', this.formObject);

            var fqnString = changeTrackingFormInputs.val();
            var fqnArray = fqnString ? fqnString.split("\t") : [];

            var fullyQualifiedNameArray = [];

            _.each(fqnArray, function(fqn) {
                // if we already have an object representing the current property, ignore it.
                //if(this.getPropertyByInputName(fqn)) {
                //    return;
                //}

                var object = $('[name="' + fqn + '"]');


                if(object.length === 0) {
                    DEBUG.warn("Change Tracking object not found: " + fqn);
                    return;
                }

                var initialValue;
                if(object.length > 1) {
                    initialValue = object.map(function() {
                        return $(this).val();
                    }).toArray();
                }
                else {
                    initialValue = object.val();
                }

                fullyQualifiedNameArray.push({
                    name: fqn,
                    jQueryObject: object,
                    initialValue: initialValue
                });
            }, this);

            return fullyQualifiedNameArray;
        };

        /**
         * Adds error to be displayed with displayErrors
         * @param {string} str
         * @see View#displayErrors
         */
        View.prototype.addError = function(str) {
            if(typeof str !== "string") {
                throw new Error("Error value was: " + JSON.stringify(str));
            }
            this.errorArray.push(str);
        };

        /**
         * Clears errors displayed
         * TODO rename to clearErrors
         */
        View.prototype.clearError = function() {
            this.errorArray = [];
            this.errorField.html(null);
        };

        /**
         * Display stored errors
         */
        View.prototype.displayErrors = function() {
            var outputString = globals.get("_webrFormFeedbackStrings").submissionError + '<br/>';

            if(this.errorArray.length > 0) {
                outputString += '<ul>';
                while(this.errorArray.length) {
                    outputString += '<li>' + this.errorArray.shift() + '</li>';
                }
                outputString += '</ul>';
            }

            this.errorField.html(outputString);
        };

        /**
         * @returns {jQuery} jQuery object representing the View's error field
         */
        View.prototype.findErrorField = function() {
//FIXME - this works because it's using the id selector, which finds the first one and ignores the rest. this doesn't guarantee we have the right one though
            var errorField = $("#pageClientSideError", this.formObject);

            if( errorField.length === 1) {
                DEBUG.custom({indent:1, data:"Found Error Feedback Field"});
            } else if(errorField.length > 1) {
                DEBUG.warn("Found " + errorField.length + " Error Feedback Fields");
            } else {
                DEBUG.error("No Error Feedback field found");
            }
            return errorField;
        };

		/*
		* Resets the error field within the containing object
		*/
		View.prototype.resetErrorField = function(errorContainingObject){
			var errorField = errorContainingObject.find("#pageClientSideError");

            if( errorField.length === 1) {
                DEBUG.custom({indent:1, data:"ResetErrorField: Found Error Feedback Field"});
            } else if(errorField.length > 1) {
                DEBUG.warn("ResetErrorField: Found " + errorField.length + " Error Feedback Fields");
            } else {
                DEBUG.error("ResetErrorField: No Error Feedback field found");
            }
            this.errorField = errorField;
		}
		
        /**
         * @returns {jQuery} jQuery object representing the View's form metadata field
         */
        View.prototype.findFormMetadataField = function() {
            //FIXME does this guarantee this is the Root EntityView's mdetadata field?
            var formMetadataField = this.formObject.find("input[name=_webrFormMetadata]").first();

            if( formMetadataField.length === 1) {
                DEBUG.custom({indent:1, data:"Found Form Metadata Field"});
            } else if(formMetadataField.length > 1) {
                DEBUG.warn("Found " + errorField.length + " Form Metadata Fields");
            } else {
                DEBUG.error("No Form Metadata field found");
            }
            return formMetadataField;
        };

        /**
         * @returns {jQuery} jQuery object representing the View's form DOM object
         */
        View.prototype.findFormDomObject = function() {
            var view = this;

            var formDomObject = $('form.EntityViewForm')
                .filter(function() {
                    var poref = $('input[name="__EntityViewPOREF"]', this).val();
                    return poref === view.poref;
                });

            if (formDomObject.length === 0) {
                return null;
            }
            if (formDomObject.length > 1) {
                throw new Error("More than one form was found on the page.");
            }
            DEBUG.custom({indent: 1, data: "Form DOM object found"});

            return formDomObject;
        };

        /**
         * Puts focus on the first element on the view's form
         * This always re-finds the first element because it is not always static (because we can hide or show
         * the first element)
         */
        View.prototype.focus = function() {
            $("#ViewArea", this.formObject)
                .find(":input:visible:enabled,input[name^='_webrRichText_']")
                .filter(function() {
                    // Necessary because the :visible filter includes elements with visibility set to hidden.
                    return this.style.visibility !== "hidden";
                })
                .first()
                .focus();
        };

        /**
         * Determines form metadata to send to the server
         * @returns {{fullyQualifiedName: (string), displayType: (string), persistence: (string), validate: boolean}}
         */
        View.prototype.getFormMetadata = function() {
            var formMetadata = {};

            formMetadata.regions = _.compact(_.map(this.regions, function(region) {
                // currently we only need to pass to the form handler the hidden forms.
                // this function can be expanded to contain more data.
                //if(region.currentDisplayType.isVisible) {
                if(region.isVisible()) {
                    return null;
                }

                var returnValue = {
                    fullyQualifiedName : region.fullyQualifiedName,
                    displayType : region.currentDisplayType.name,
                    persistence : region.getCurrentPersistence().name,
                    validate 	: region.validateWhenNonEditable
                };

                return returnValue;
            }));

            formMetadata.internalProperties = _.compact(_.map(this.internalProperties, function(property) {
                // currently we only need to pass to the form handler the hidden forms.
                // this function can be expanded to contain more data.

                //if(property.currentDisplayType.isVisible) {
                if(property.isVisible()) {
                    return null;
                }

                var returnValue = {
                    fullyQualifiedName : property.fullyQualifiedName,
                    displayType : property.currentDisplayType.name, // currently always hidden
                    persistence : property.getCurrentPersistence().name,
                    validate 	: property.validateWhenNonEditable
                };

                return returnValue;
            }));

            return formMetadata;
        };

        /**
         * @returns {jQuery} jQuery object representing the form DOM element
         */
        View.prototype.getFormObject = function() {
            return this.formObject;
        };

        /**
         * @returns {Array.<Property>} Array of properties on this view
         */
        View.prototype.getProperties = function() {
            return _.clone(this.properties);
        };

        /**
         * @param {Property|jQuery|string} obj
         * @returns {?Property} Specified property from this view.
         */
        View.prototype.getProperty = cachedMemoize(function (obj) {
            if (obj === undefined || obj === null) {
                return undefined;
            }

            if (obj instanceof Property) {
                return obj;
            }

            //TODO[LOW] ie8 does not support HTMLElement
            //if (obj instanceof HTMLElement) {
            //    obj = $(obj);
            //}

            if (obj instanceof jQuery) {
                return _.find(this.properties, function (property) {
                    return (property.containingObject && property.containingObject.is($(obj))) ||
                        (property.formObjects      && property.formObjects.is($(obj)));
                });
            }
            else if ((typeof obj) === "string") {
                return _.findWhere(this.properties, {fullyQualifiedName: obj});
            }
            throw new Error("Unknown method argument type");
        }, function () {
            return this.cache.property;
        });

        /**
         * @param {string} str
         * @return {?Property} Specified property from this view.
         */
        View.prototype.getPropertyByInputName = cachedMemoize(function (str) {
            if((typeof str) !== "string") {
                throw new Error("Input must be string.");
            }

            //return _.findWhere(this.properties, {inputFieldName:str});

            return _.find(this.properties, function(property) {
                return property.inputFieldName === str;
            });

        }, function () {
            return this.cache.property;
        });

        /**
         * @param {Region|jQuery|HTMLElement|string} obj
         * @returns {?Region} Specified region from this view.
         */
        View.prototype.getRegion = cachedMemoize(function (obj) {
            if (obj === undefined || obj === null) {
                return undefined;
            }

            if (obj instanceof Region) {
                return obj;
            }

            if (obj instanceof HTMLElement) {
                obj = $(obj);
            }

            if (obj instanceof jQuery) {
                return _.find(this.regions, function (region) {
                    return region.containingObject.is($(obj));
                });
            }
            else if ((typeof obj) === "string") {
                return _.findWhere(this.regions, {fullyQualifiedName: obj});
            }
            throw new Error("Unknown method argument type.");
        }, function () {
            return this.cache.region;
        });

        /**
         * @returns {Array.<Region>} Array of regions on this view
         */
        View.prototype.getRegions = function() {
            return _.clone(this.regions);
        };

        /**
         * @param {Rule|string} obj
         * @returns {?Property} Specified property from this view.
         */
        View.prototype.getRule = cachedMemoize(function (obj) {
            if (obj === undefined || obj === null) {
                return undefined;
            }

            if (obj instanceof Rule) {
                return obj;
            }

            if ((typeof obj) === "string") {
                return _.findWhere(this.rules, {name: obj});
            }
            throw new Error("Unknown method argument type.");
        }, function () {
            return this.cache.rule;
        });

        /**
         * @returns {Array.<Rule>} Array of rules on this view
         */
        View.prototype.getRules = function () {
            return _.clone(this.rules);
        };

        /**
         * ViewLocation is gathered from EntityViewLocation
         * @returns {string} view location name
         */
        View.prototype.getViewLocation = function() {
            return this.viewLocation;
        };

        /**
         * @returns {Array.<ViewAsset>} Array of ViewAssets on this view
         */
        View.prototype.getViewAssets = function() {
            return _.union(this.regions, this.properties);
        };

        /**
         * @returns {string}
         */
        View.prototype.getViewId = function() {
            return this.viewId;
        };

        /**
         * TODO[LOW] is this still necessary?
         * SC00001204: If this is the Safari browser, copy the change detection hash
         * from the client-side script variable to the hidden form field.  This is
         * necessary to work around the field value caching bug in Safari which
         * causes a stale change detection hash to be submitted.  This happens
         * even though the page isn't cached.
         */
        View.prototype.initializeSafariCDCachingWorkaround = function() {
            //TODO[LOW] this will also execute in chrome. is this what we want?
            if (navigator.appVersion.indexOf("Safari") > 0) {
                DEBUG.warn("doSafariCDCachingWorkaround had to execute.");
                // for some reason, rm console does not contain _webrCDHash
                if(this._webrCDHash.length > 0) {
                    this._webrCDHash[0].value = globals.get("_webrCDHash0");
                } else {
                    DEBUG.warn("doSafariCDCachingWorkaround could not find '_webrCDHash'.");
                }
            }
        };

        /**
         * TODO move to ViewAsset
         * Convenience method to initialize a ViewAsset object
         * @param {Function} type
         * @param {Object} metadata
         * @param {View} view
         * @returns {ViewAsset}
         */
        View.prototype.initializeViewAsset = function(type, metadata, view) {
            return _.compact(_.map(metadata, function (data) {
                try {
                    if(data.type === "Entity/DataSnapShot") {
                        return null;
                    }
                    return new type(data, view);
                } catch (e) {
                    console.error("Could not instantiate object: " + e.message);
                    DEBUG.error(e);
                    return null;
                }
            }));
        };

        /**
         * @returns {boolean} If any of the View's assets are dirty, then returns true, else false.
         */
        View.prototype.isDirty = function() {
            //if(!this.cache.hasFormChanged) {
            //    return false;
            //}

            //compare server side change detection hashes
            if(this._webrCDHash && this._webrCDCurrentHash &&
                this._webrCDHash.val() !== this._webrCDCurrentHash.val()) {
                return true;
            }

            var isDirty = false;
            _.each(this.dirtyObjects, function(dirtyObject) {
                if(isDirty) {
                    return;
                }

                var obj = dirtyObject.jQueryObject;

                var currentValue;
                if(obj.length > 1) {
                    currentValue = obj.map(function() {
                        return $(this).val();
                    }).toArray();
                }
                else {
                    currentValue = obj.val();
                }

                //var currentValue = obj.length === 0 ? obj.val() : obj.map(function() {
                //    return $(this).val();
                //}).toArray();

                if(currentValue === []) {
                    currentValue = null;
                }

                var initialValue = dirtyObject.initialValue === [] ? null : dirtyObject.initialValue;

                isDirty = !_.isEqual(currentValue, initialValue);

            });
            if(isDirty) {
                return true;
            }

            var firstDirtyObject = _.find(this.properties, function(property) {
                return property.isDirty();
            });
            // firstDirtyObject is a property or is undefined
            return !!firstDirtyObject;
        };

        /**
         * @returns {boolean} readOnly value of the view
         */
        View.prototype.isReadOnly = function() {
            return this.readOnly;
        };

        /**
         * Convenience method to attach event handlers to the View's form
         */
        View.prototype.on = function() {
            this.formObject.on.apply(this, arguments);
        };

        /**
         * Currently does not allow adding submit handlers to the view itself. Only reason for this: ran out of time
         * to implement it.
         * Currently just executes the submit handlers on the view assets.
         * These functions are the very last thing before form.submit() is called.
         */
        View.prototype.onSubmit = function() {
            _.each(this.properties, function(property) {
                property.onSubmit();
            });
        };

        /**
         * Convenience method to attach event handlers to the View's form
         * @param callbackFunction
         */
        View.prototype.ready = function(callbackFunction) {
            //this.viewReadyDefer.done(callbackFunction);
            globalReadyFunction(callbackFunction, this.viewId);
        };

        /**
         * Convenience method to attach event handlers to the View's form
         */
        View.prototype.trigger = function() {
            this.formObject.trigger.apply({}, arguments);
        };

        /**
         * Validates all ViewAssets on this View.
         * @returns {Object} jQuery Promise. Promise resolves when all validation is complete.
         */
        View.prototype.validate = function() {
            // $.when.apply({}, validationPromiseArray) won't work because it stops on the first failure.
            // This means all the required field errors won't show, just the first one.
            // PortalTools.whenAllFinish is our own 'when' function which does not have this issue.
            return whenAllFinish.apply({},
                _.map(this.properties, function(property) {
                    return property.validate(); // returns promise
                })
            );
        };

        /**
         * Updates the form metadata field
         */
        View.prototype.updateFormMetadata = function() {
            if(!this.formMetadataField) {
                return;
            }
            //this.formMetadataField.val(this.getFormMetadata());
            this.formMetadataField.attr('value', JSON.stringify(this.getFormMetadata()));
        };


        /**
         * Parent type of all Properties, Regions, etc. on the view.
         * @param {Object} parameters - Parameters used to initialize
         * @param {View} view - EntityView this is associated to.
         * @constructor
         * @abstract
         * @memberof module:EntityView/View
         */
        function ViewAsset(parameters, view) {
            if (!parameters) {
                throw new Error("No parameters defined.");
            }

            if (!parameters.fullyQualifiedName) {
                throw new Error("'fullyQualifiedName' was not defined which is required.");
            }
            if(parameters.controlType === "dataSnapShot") {
                return;
            }

            DEBUG.custom({
                data: function () {
                    return "Setting up " + DebugUtil.getTypeName(this) + " named '" + parameters.fullyQualifiedName + "'";
                }, context: this
            });

            _.bindAll(this,
                'changeHandler',
                'checkFormObjectLength', // InternalProperty
                'displayUpdateHandler',
                'determineIsEditPossible',
                'findContainer',
                'findErrorFeedbackContainer', // InternalProperty
                'findConditionalDisplayContent',
                'findFormDomObjects', // InternalProperty
                'findRequiredField', // InternalProperty
                'findOptionValues', // InternalProperty
                'updateDisplayEvent',
                'generateInputFieldName', //Property
                'generateInputFieldTagType', //Property
                'getChildProperties',
                'getContainingRegions',
                'getCurrentPersistence',
                'isCurrentlyEditable',
                'isRequired', // InternalProperty
                'isValidationNecessary',
                'isVisible',
                //'jsonReplacer',
                'on',
                'onSubmit',
                'setupContainerEventHandler',
                //'setupUtilityActions',
                'trigger',
                'triggerRuleUpdate',
                'updateDisplay',
                'validate'
            );

            this.view = view;

            this.cache = {
                //hasViewAssetChanged: false //TODO I don't think this will work with legacy postbacks
                hasViewAssetChanged: true //TODO use true until we can fix the legacy problem
            };
            this.onSubmitCallbacks = [];

            this.poref = parameters.poref;

            this.fullyQualifiedName = parameters.fullyQualifiedName;
            this.targetTypeName = parameters.targetTypeName;
            this.qualifiedName  = parameters.qualifiedName;

            this.ruleName = parameters.ruleName;

            this.hidden   = parameters.hidden;
            this.readOnly = parameters.readOnly;
            this.required = parameters.required;

            this.controlType = parameters.controlType;
            this.layoutOrder = parameters.layoutOrder;
            
            this.multiline = parameters.multiline;

            if(parameters.type) {
                var typeSplit = parameters.type.split('/');
                this.type = typeSplit[0];
                if (typeSplit.length > 0) {
                    this.eType = typeSplit[1];
                }
            }

			//If parameters.persistedValue is undefined, then the metadata should contain persistedValuesArray object
            this.persistedValue = parameters.persistedValue || parameters.persistedValuesArray;

            this.format = parameters.format;
            this.currencyFormat = parameters.currencyFormat;
            //this.dateFormat = parameters.dateFormat;

            if(this.type === "Date") {
                require(["moment"]);
                if(!this.format) { this.format = "shortY2KDate"; }
                this.dateFormat = this.generateDateFormat(this.format);
            }

            this.currentDisplayType = _displayType.getDisplayType(parameters.currentDisplayType);

            if (parameters.displayTypes) {
                this.displayTypes = {
                    ruleTrue  : _displayType.getDisplayType(parameters.displayTypes.ruleTrue),
                    ruleFalse : _displayType.getDisplayType(parameters.displayTypes.ruleFalse)
                };
            }

            this.isEditPossible = this.determineIsEditPossible();

            /**
             * Debounced version of triggerRuleUpdate
             * @alias triggerRuleUpdate
             */
            this.triggerRuleUpdateDebounced = _.debounce(this.triggerRuleUpdate, defaultOptions.debounceWait);

            this.inputFieldName    = this.generateInputFieldName();
            this.inputFieldTagType = this.generateInputFieldTagType();

            this.persistanceWhenNonEditable = _persistenceType.getPersistenceType(parameters.persistanceWhenNonEditable || "clear");
            this.validateWhenNonEditable    = parameters.validateWhenNonEditable || false;

            this.requiredFormClassName = '_webrRequired_' + this.fullyQualifiedName;
            if(this.controlType === "simpledocument") {
                this.requiredFormClassName = this.requiredFormClassName + ".targetURL";
            }
            this.requiredFormDOMString = '<input name="' + this.requiredFormClassName + '" type="hidden">';

            faultTolerantDomReady(function () {
                DEBUG.log("domReady [BEGIN]: " + this.fullyQualifiedName, this);

                this.containingObject = this.findContainer();

                // Does not have a container if it's an ExternalProperty
                this.$event = this.containingObject ? this.containingObject : $({});
                this.$event.on("displayUpdated", this.displayUpdateHandler);

                this.errorField   = this.findErrorFeedbackContainer();
                this.formObjects  = this.findFormDomObjects();
                this.optionValues = this.findOptionValues();
                this.conditionalDisplayContent = this.findConditionalDisplayContent();

                this.setupContainerEventHandler();

                //this.setupUtilityActions();

                DEBUG.log("domReady [ END ]: " + this.fullyQualifiedName, this);
            }, this);

        }

        /**
         * Ignores Button click events.
         * @param {Event} event
         * @param {Object} data
         */
        ViewAsset.prototype.changeHandler = function (event, data) {
            this.cache.hasViewAssetChanged = true; //TODO probably doesn't work well with legacy postbacks

            DEBUG.custom({
                type: "event", data: function () {
                    var message;
                    //var property = View.getProperty(event.target);
                    if (event.hasOwnProperty("editor")) {
                        //This is a ckeditor event
                        message = "SAW EVENT: (changeHandler)\n" +
                            //"On: " + (property ? property.fullyQualifiedName : "not a property") + "\n" +
                        "Type: ckeditor\n" +
                        "Listener: " + this.fullyQualifiedName + ")";
                    }
                    else // otherwise should be a normal jquery event
                    {
                        message = "SAW EVENT: (changeHandler)\n" +
                            //"On: " + (property ? property.fullyQualifiedName : "not a property") + "\n" +
                        "Type: " + event.type + "\n" +
                        "Listener: " + this.fullyQualifiedName;
                    }
                    return message;
                }, context: this
            });

            // there is no change if the target type is a button.
            if ( event.target && ( event.target instanceof HTMLButtonElement ||
                (event.target instanceof HTMLInputElement && event.target.type === "button"))) {
                return;
            }

            // Stops change event propagation when change events are triggered on properties inside a region (this is
            // done in ViewAsset.prototype.displayUpdateHandler). This prevents the change event from unnecessarily
            // running on parent dom elements.
            if (data && data.stopPropagation) {
                DEBUG.custom({
                    indent: 1,
                    data: "Stopping propagation of '" + event.type + "' event: " + this.fullyQualifiedName
                });
                event.stopPropagation();
            }

            // Only needs to run for properties since only properties can trigger rules
            if (this instanceof Property) {
                DEBUG.custom({
                    type: "event", data: "    EXECUTING DEBOUNCE FUNCTION: (changeHandler)\n" +
                    "    After Debounce: triggerRuleUpdate\n" +
                    "    On: " + this.fullyQualifiedName
                });
                this.triggerRuleUpdateDebounced(data);
            }
        };

        /**
         * @returns {boolean} returns true if editing is a possibility, otherwise false.
         */
        ViewAsset.prototype.determineIsEditPossible = function () {
            if(this.view.isReadOnly() || this.readOnly) {
                return false;
            }
            else if(!this.ruleName) {
                // If there is no rule, by default it is editable.
                // Even if it's hidden, it's still possible to edit.
                return true;
            }
            else if(this.displayTypes) {
                // this asset is theoretically possible to edit at some point it it's view lifetime.
                return (this.displayTypes.ruleTrue  && this.displayTypes.ruleTrue.isEditable)
                    || (this.displayTypes.ruleFalse && this.displayTypes.ruleFalse.isEditable);
            }
            else {
                // should never get here though
                return false;
            }
        };

        /**
         * Handles displayUpdate event, which is run after one of the conditional displays changes.
         * If this is a region, it will trigger the change event on all child ViewAssets.
         * @param {Event} event
         * @param {Object} data
         */
        ViewAsset.prototype.displayUpdateHandler = function (event, data) {
            //FIXME
            //DEBUG.custom({
            //    type: "event", data: function () {
            //        var property = this.view.getProperty(event.target);
            //        return "SAW EVENT: (displayUpdateHandler)\n" +
            //            "On: " + (property ? property.fullyQualifiedName : "not a property") + "\n" +
            //            "Type: " + event.type + "\n" +
            //            "Listener: " + this.fullyQualifiedName;
            //    },
            //    context: this
            //});

            if (this instanceof Region) {
                // This needs to run even if not visible so that required fields can be set to not-required.
                _.each(this.getChildProperties(), function (property) {
                    DEBUG.custom({
                        type: "event", data: "    TRIGGERING EVENT: (displayUpdateHandler)\n" +
                        "    Type: change\n" +
                        "    On: " + property.fullyQualifiedName + " (sub-region of: " + this.fullyQualifiedName + ")"
                    });
                    property.$event.trigger("change", {stopPropagation: true});
                }, this);
            }

            this.$event.trigger("change", data);
        };

        /**
         * @returns {string} Returns the input field for specific type/controlType
         */
        ViewAsset.prototype.generateInputFieldName = function () {
            if(this.type === "Set") {
                return this.fullyQualifiedName + '_setItem';
            }
            if(this.controlType === "simpledocument") {
                return this.fullyQualifiedName + '.targetURL';
            }
            if(this.multiline) {
                return this.fullyQualifiedName + '_text';
            }

            return this.fullyQualifiedName;
        };

        /**
         * @returns {string} Returns HTML tag type for specific type/controlType
         */
        ViewAsset.prototype.generateInputFieldTagType = function () {
            if(this.hidden) {
                return 'input';
            }
            switch (this.controlType) {
                case "radio":
                case "singleline":
                case "documentContentFileUploadView":
                case "checkbox":
                case "calendar":
                case "singleDocumentUploadEditView":
                case "singleEntityDataEntry":
                //case "singleEntitySelection":
                case "chooserSelection":
                case "simpledocumenttable":
                //case "documentTableUploadEditview":
                case "checkboxSelection":
                case "dataEntry": //"entityTableDataEntry":
                case "dataentrygrid":
                case "simpledocument":
                case "inlinecdtview":
                    return 'input';
                case "richtext":
                case "textarea":
                    return 'textarea';
                case "select":
                    return 'select';
                case "dataSnapShot":
                    DEBUG.warn(this.controlType + " control type is not supported.");
                default:
                    return undefined;
            }
        };

        /**
         * This function is cached based on the fullyQualifiedName and the last time this property was updated.
         * @returns {Array.<ViewAsset>} Returns *all* child properties for this ViewAsset (Applicable only to Regions).
         */
        ViewAsset.prototype.getChildProperties = cachedMemoize(function () {
                return _.filter(this.view.properties, function (property) {
                    if (property instanceof ExternalProperty) {
                        return false;
                    }

                    var thisContainer = this.containingObject.get(0);
                    var propertyContainer = property.containingObject.get(0);

                    return $.contains(thisContainer, propertyContainer);
                }, this);
            },
            function () {
                return this.view.cache.property;
            },
            function () {
                return this.fullyQualifiedName;
            }
        );

        /**
         * This function is cached based on the fullyQualifiedName, constraining region, and the last time this
         * property was updated.
         * @param {Region} constrainingRegion - Specifies region to check within. Default is to check all regions.
         * @returns {Array.<Region>} Returns *all* all regions that this ViewAsset is contained within.
         */
        ViewAsset.prototype.getContainingRegions = cachedMemoize(function (constrainingRegion) {
                // The default is to get all regions in the EntityView and determine if the current
                // ViewAsset is inside that region.
                // If this method is passed a constraining region, we do the same thing, but instead
                // of looking at all regions in the entityView, we now only look at the
                var regionsToCheck = constrainingRegion ?
                    constrainingRegion.getChildRegions() :
                    this.view.regions;

                return _.filter(regionsToCheck, function (region) {
                    return region.containingObject &&
                            // $.contains requires dom elements and not jquery elements
                        $.contains(region.containingObject.get(0), this.containingObject.get(0));
                }, this);
            },
            function () {
                return this.view.cache.region;
            },
            function () {
                return this.fullyQualifiedName + ( arguments && arguments[0] ? "_Region_" + arguments[0] : "");
            }
        );

        /**
         * @returns {_persistenceType} The current Persistence type.
         */
        ViewAsset.prototype.getCurrentPersistence = function () {
            if(this.isCurrentlyEditable()) {
                return _persistenceType.SAVE;
            }

            return this.persistanceWhenNonEditable;
        };

        /**
         * @returns {boolean} Returns true if this ViewAsset is editable by the user.
         */
        ViewAsset.prototype.isCurrentlyEditable = function () {
            if(!this.isEditPossible) {
                return false;
            }

            var foundNonEditableRegion = _.find(this.getContainingRegions(), function (region) {
                return !region.isCurrentlyEditable();
            });

            // check to see if this property is within a non-editable region
            if (foundNonEditableRegion !== undefined) {
                return false;
            }

            return this.currentDisplayType.isEditable;
        };

        /**
         * @param constrainingRegion
         * @returns {boolean} Returns false if Hidden, otherwise true.
         */
        ViewAsset.prototype.isVisible = function (constrainingRegion) {
            var constrainingRegionArray = this.getContainingRegions(constrainingRegion);
            var nonVisibleRegions = _.find(constrainingRegionArray, function (region) {
                return !region.isVisible();
            });

            // check to see if this property is within a non-editable region
            if (nonVisibleRegions !== undefined) {
                return false;
            }

            if (constrainingRegion && !constrainingRegion.contains(this)) {
                throw new Error("constraining region does not constrain this property: " + this.fullyQualifiedName);
            }

            return this.currentDisplayType.isVisible;
        };

        //ViewAsset.prototype.jsonReplacer = function (key, value) {
        //    console.log(key, value, this);
        //    switch(key) {
        //        case "fullyQualifiedName":
        //            return value;
        //        case "displayTypes":
        //            return value.currentValue.name;
        //        default:
        //            return value;
        //    }
        //};

        /**
         * Convenience method to attach events to this ViewAsset.
         */
        ViewAsset.prototype.on = function () {
            this.$event.on.apply(this.$event, arguments);
        };

        /**
         * Enables callbacks to run right before navigation (eg: form submission).
         * These callbacks should be the last thing to execute before navigation.
         * @param callbackFunction
         */
        ViewAsset.prototype.onSubmit = function (callbackFunction) {
            if (callbackFunction) {
                this.onSubmitCallbacks.push(callbackFunction);
                return;
            }

            var i;
            for (i = 0; i < this.onSubmitCallbacks.length; i++) {
                this.onSubmitCallbacks[i].apply(this);
            }

        };


        //ViewAsset.prototype.setupUtilityActions = function() {
        //    switch (this.controlType) {
        //        case "checkbox":
        //        case "radio":
        //        case "checkboxSelection":
        //            $(".lnkClear", this.containingObject).click({viewAsset: this}, function(event) {
        //                event.preventDefault();
        //                event.data.viewAsset.clear();
        //            });
        //            break;
        //        //case "richtext":
        //        //case "singleline":
        //        //case "calendar":
        //        //case "textarea":
        //        //case "documentContentFileUploadView":
        //        //case "simpledocument":
        //        //case "singleDocumentUploadEditView":
        //        //case "singleEntityDataEntry":
        //        //case "chooserSelection":
        //        //case "simpledocumenttable":
        //        //case "dataEntry": //"entityTableDataEntry":
        //        //case "dataentrygrid":
        //        //case "select":
        //        //case "dataSnapShot":
        //    }
        //};

        /**
         * Convenience method to trigger events on this ViewAsset.
         */
        ViewAsset.prototype.trigger = function () {
            this.$event.trigger.apply(this.$event, arguments);
        };

        /**
         * This is used by triggerRuleUpdateDebounced, which calls this function.
         * This tells the rules associated with this ViewAsset to rerun.
         * @param {Object} data
         */
        ViewAsset.prototype.triggerRuleUpdate = function (data) {
            DEBUG.log("DEBOUNCED: triggerRuleUpdate");
            DEBUG.custom({
                type: "event", data: "    TRIGGERING EVENT: (changeHandler)\n" +
                "    Type: ruleUpdate\n" +
                "    On: " + this.fullyQualifiedName
            });
            this.$event.trigger("ruleUpdate", data);


            // Need to update the form metadata for CDTs that have AJAX callbacks. These CDTs
            // post to the form the current form values as they are, not as the form submit handler
            // modifies them
            // TODO Ideally, this shouldn't be here. this is extra work that should not need to be done at each form field update.
            this.view.updateFormMetadata();
        };

        /**
         * Updates the current display type to the requested display type.
         * Executes the displayUpdated event on this ViewAssent once completed.
         * @param {_displayType} requestedDisplayType - Display type to change to
         * @param {Object} data - event data object to pass on to the displayUpdate event handler.
         */
        ViewAsset.prototype.updateDisplay = function (requestedDisplayType, data) {
            var currentDisplayType = this.currentDisplayType;

            if (!requestedDisplayType) {
                DEBUG.custom({indent: 1, data: "No requested display type given, using current display type."});
                requestedDisplayType = currentDisplayType;
            }
            DEBUG.custom({
                indent: 1, data: function () {
                    return "Request to update conditional display: " + this.fullyQualifiedName + "\n" +
                        "    current   displayType: " + currentDisplayType.name + "\n" +
                        "    requested displayType: " + requestedDisplayType.name;
                }, context: this
            });

            if (this instanceof Region) { // if true, this is a region (regions do not have a formObjects)
                switch (requestedDisplayType) {
                    case _displayType.DISABLED:
                    case _displayType.READONLY:
                        throw new Error("Regions only support Enabled and Hidden. '" +
                        requestedDisplayType.name + "' was requested.");
                }
            }

            // sometimes the css animationend event doesn't run, so old css classnames can still
            // exist. This here ensures these classes go away.
            this.conditionalDisplayContent.removeClass("updateViewAsset showViewAsset hideViewAsset");

            switch (requestedDisplayType) {
                case _displayType.ENABLED:
                    if (this.formObjects) {
                        this.formObjects.prop("disabled", false);
                    }

                    if (currentDisplayType.isVisible) {
                        this.conditionalDisplayContent.applyCSSAnimation("updateViewAsset");
                    } else {
                        this.conditionalDisplayContent.applyCSSAnimation("showViewAsset");
                        //this.conditionalDisplayContent.slideDown(1000, function () {
                        this.conditionalDisplayContent.fadeIn(1000, function () {
                            $(this).removeProp("hidden");
                        });
                    }
                    break;

                case _displayType.HIDDEN:
                    if (currentDisplayType.isVisible) {
                        this.conditionalDisplayContent.applyCSSAnimation("hideViewAsset");
                        //this.conditionalDisplayContent.slideUp(1000, function () {
                        this.conditionalDisplayContent.fadeOut(1000, function () {
                            $(this).prop('hidden', 'hidden');
                        });
                    }

                    if (this.formObjects) {
                        this.formObjects.prop("disabled", false);
                    }

                    break;

                case _displayType.READONLY:
                case _displayType.DISABLED:
                    throw new Error("Not yet implemented for _displayType: " + requestedDisplayType.name);
            }

            this.currentDisplayType = requestedDisplayType;

            DEBUG.custom({
                type: "event", data: "    TRIGGERING EVENT: (ViewAsset.prototype.updateDisplayEvent)\n" +
                "    Type: displayUpdated\n" +
                "    On: " + this.fullyQualifiedName
            });
            this.$event.trigger("displayUpdated", data);
        };

        /**
         * Rule executes this when rule determines the display needs to update.
         * TODO[LOW] the name should be something like 'ruleUpdatedHandler'
         * @param {Event} event
         * @param {Object} data
         */
        ViewAsset.prototype.updateDisplayEvent = function (event, data) {
            DEBUG.custom({
                type: "event", data: function () {
                    return "SAW EVENT: (ViewAsset.prototype.updateDisplayEvent)\n" +
                        "On: " + event.type + "\n" +
                        "Type: " + event.type + "(" + event.namespace + ")\n" +
                        "Listener: " + this.fullyQualifiedName + "\n" +
                        (data ? "Rule Result: " + data.ruleResult + "\n" : "") +
                        "Get type: " + DebugUtil.getTypeName(this);
                },
                context: this
            });

            //This is triggered when a rule for this property triggers
            var displayTypeResult = data && data.ruleResult ? this.displayTypes.ruleTrue : this.displayTypes.ruleFalse;
            this.updateDisplay(displayTypeResult, data);
            //this.currentDisplayType = displayTypeResult;

            //DEBUG.custom("    TRIGGERING EVENT: (ViewAsset.prototype.updateDisplayEvent)\n" +
            //    "    Type: displayUpdated\n" +
            //    "    On: " + this.fullyQualifiedName);
            //this.$event.trigger("displayUpdated", data);
        };

        /**
         * @abstract
         */
        ViewAsset.prototype.checkFormObjectLength =
            ViewAsset.prototype.findConditionalDisplayContent =
                ViewAsset.prototype.findContainer =
                    ViewAsset.prototype.findErrorFeedbackContainer =
                        ViewAsset.prototype.findFormDomObjects =
                            ViewAsset.prototype.findRequiredField =
                                ViewAsset.prototype.findOptionValues =
                                    ViewAsset.prototype.isRequired =
                                        ViewAsset.prototype.isValidationNecessary =
                                            ViewAsset.prototype.setupContainerEventHandler =
                                                ViewAsset.prototype.validate =
                                                    function () {
                                                        throw new Error("must be overridden by subclass");
                                                    };

        /**
         * Represents a Region
         * @param {Object} parameters - Parameters to be passed to ViewAsset constructor.
         * @param {View} view - EntityView this is associated to.
         * @constructor
         * @extends ViewAsset
         * @memberof module:EntityView/View
         */
        function Region(parameters, view) {
            ViewAsset.call(this, parameters, view);

            _.bindAll(this,
                'contains',
                'getChildRegions'
            );

            this.view.cache.region = _.now();
        }
        Region.prototype = Object.create(ViewAsset.prototype);
        Region.prototype.constructor = Region;

        /**
         * @returns {jQuery} Returns jQuery object representing area to Conditionally display
         */
        Region.prototype.findConditionalDisplayContent = function() {
            return this.containingObject;
        };

        /**
         * @param obj - Object to compare containment against.
         * @returns {boolean} - Returns true if obj is contained within this region.
         */
        Region.prototype.contains = function (obj) {
            if (obj instanceof jQuery) {
                obj = obj.get(0);
            }
            if (!(obj instanceof HTMLElement)) {
                throw new Error("Object must be of type jQuery or HTMLElement");
            }
            return $.contains(this.containingObject.get(0), obj);
        };

        /**
         * @returns {boolean} returns true.
         */
        Region.prototype.determineIsEditPossible = function () {
            return true;
        };

        /**
         * @returns {jQuery} Container object for this region (a div)
         */
        Region.prototype.findContainer = function () {
            //TODO[LOW] this won't work when multiple views are on a page since the id might be used by the system (ex: two views contain the same property)
            var selector = '#_' + this.view.viewId + '_region_' + this.fullyQualifiedName + '_container';

            // containingObject => container
            var containingObject = $(selector);
            if (containingObject.length === 1) {
                DEBUG.custom({indent: 1, data: "Found DOM container object found using selector: " + selector});
            } else if (containingObject.length > 1) {
                DEBUG.warn("Found " + containingObject.length + " DOM container object found using selector: " + selector);
            } else {
                throw new Error("No DOM container object found using selector: " + selector);
            }

            return containingObject;
        };

        /**
         * @returns {Array.<Region>} - Returns all regions contained within this Region
         */
        Region.prototype.getChildRegions = cachedMemoize(function () {
                return _.filter(this.view.regions, function (region) {
                    return region.containingObject &&
                            // $.contains requires dom elements and not jquery elements
                        $.contains(this.containingObject.get(0), region.containingObject.get(0));
                }, this);
            },
            function () {
                return this.view.cache.region;
            },
            function () {
                return this.fullyQualifiedName;
            }
        );

        /**
         * @returns {boolean} Returns false. Validation is not necessary on Regions.
         */
        Region.prototype.isValidationNecessary = function () {
            return false;
        };

        /**
         * Associates this.changeHandler to any update event caused by child properties.
         */
        Region.prototype.setupContainerEventHandler = function () {
            this.$event.on("change input textchange", this.changeHandler);
        };

        /**
         * These functions are not used by Region, but still need to be defined.
         * @type {Function}
         */
        Region.prototype.checkFormObjectLength =
            Region.prototype.findErrorFeedbackContainer =
                Region.prototype.findFormDomObjects =
                    Region.prototype.findRequiredField =
                        Region.prototype.findOptionValues =
                            Region.prototype.generateInputFieldName =
                                Region.prototype.generateInputFieldTagType =
                                    Region.prototype.isRequired =
                                        Region.prototype.validate =
                                            function() {
                                                return undefined;
                                            };

        /**
         * This type is an abstract type intended to be implemented by it's subclasses
         * @param {Object} parameters - Parameters to be passed to ViewAsset constructor.
         * @param {View} view - EntityView this is associated to.
         * @constructor
         * @abstract
         * @extends ViewAsset
         * @memberof module:EntityView/View
         */
        function Property(parameters, view) {
            ViewAsset.call(this, parameters, view);

            _.bindAll(this,
                'addError',
                'clear',
                'clearError',
                'findRequireInputFields',
                'focus',
                'generateDateFormat',
                'getCurrentValue',
                'getFormValue',
                'getParsedPersistentValue',
                'getParsedFormValue',
                'getPersistentValue',
                'isCurrentlyRequired',
                'isDirty',
                'isEmpty',
                'setValue',
                'updateRequiredInputFields',
                'validateCKEditorWYSIWYG',
                'validateCustomMethod', // TODO[LOW] look in to seeing if we actually need this
                'validateRequiredField'
            );

            this.view.cache.property = _.now();

            return this;
        }
        Property.prototype = Object.create(ViewAsset.prototype);
        Property.prototype.constructor = Property;

        /**
         * @abstract
         * @param str
         */
        Property.prototype.addError = function (str) {
            console.error(str);
        };

        /**
         * @abstract
         */
        Property.prototype.clearError = function () {
        };

        /**
         * Executes the ViewAsset changeHandler and then executes updateRequiredInputFields().
         * @param {Event} event
         * @param {Object} data
         */
        Property.prototype.changeHandler = function (event, data) {
            ViewAsset.prototype.changeHandler.apply(this, arguments);
            this.updateRequiredInputFields();
        };

        /**
         * @returns {jQuery} - Returns the required input form field (the input that tells the form what is required).
         */
        Property.prototype.findRequireInputFields = function () {
          	//
          	switch(this.controlType) {
                case "dataentrygrid":
                	return $("input[name^='_webrRequired_']", this.containingObject);
                default:
            		return $('input[name="' + PortalTools.escapeJQuerySelector(this.requiredFormClassName) + '"]', this.containingObject);
            }		
        };

        /**
         * TODO[LOW] does not currently work for Entities or Sets.
         * @param {*} val - the value to set this property to.
         */
        Property.prototype.setValue = function (val) {
            switch (this.type) {
                case 'Entity':
                case 'Set':
                    throw new Error("setValue is currently unsupported for: " + this.type);
            }
            this.formObjects.val(val);
        };

        /**
         * @returns {undefined|string} Returns the moments.js date format based on the Portal property date type.
         */
        Property.prototype.generateDateFormat = function () {
            if(!this.format) {
                return undefined;
            }
            switch (this.format.toLowerCase()) {
                case 'verbose':
                    return "dddd, MMMM D, YYYY h:mm:ss A Z";
                case 'longdatetime':
                case 'general':
                    return "dddd, MMMM D, YYYY h:mm:ss A";
                case 'longdate':
                    return "dddd, MMMM D, YYYY";
                case 'mediumdatetime':
                    return "M/D/YYYY h:mm:ss A";
                case 'shorty2kdatetimewithtimezone':
                    return "M/D/YYYY h:mm A Z";
                case 'shorty2kdatetime':
                case 'shortdatetime':
                case 'shortadaptivedatetime':
                    return "M/D/YYYY h:mm A";
                case 'shorttime':
                case 'mediumtime':
                case 'longtime':
                    return "h:mm:ss A";
                case 'shorty2kdate':
                case 'shortdate':
                case 'shortadaptivedate':
                case 'mediumdate':
                default:
                    return "M/D/YYYY";
            }
            return undefined;
        };

        /**
         * @returns {*} Returns the JavaScript version of the Persistent value.
         */
        Property.prototype.getParsedPersistentValue = function () {
        	if(!Array.isArray(this.persistedValue)) {
        		return this.processPersistentValue(this.persistedValue.raw, this.persistedValue.string);
        	} else {
        		var _self = this;
        		return _.map(_self.persistedValue, function(x){
        			return _self.processPersistentValue(x.raw, x.string);
        		});
        	}
        }
        
        Property.prototype.processPersistentValue = function(raw, string) {
            switch (this.type) {
                case "Date":
                    if(!raw) {
                        return null;
                    }
                    return getMoment(string, this.format, this.dateFormat);
                case "DocumentContent":
                    return string === "" ? null : string;
                case "simpledocumenttable":
                case "chooserSelection":
                case "dataEntry":
                case "simpledocument":
                    if(raw === nullOIDString) {
                        return null;
                    }
                /* falls through */
                default:
                	//TFS 4681: Since the metadata json is encoded on the server before passing on to the client,
                	//some of the ascii characters are unicode encoded. JSON.parse() and unescape() are used to
                	//decode them back to ascii characters.
                	return (typeof raw === "string" &&
                			((raw != unescape(raw)) || (/(\\u|\\\")/g.test(raw)))
                		   )
                		? unescape(JSON.parse('"' + raw.replace(/(\\)?(\")/g, function($0, $1, $2){
                				return $1 ? $0 : '\\' + $2;
                		  }) + '"')).replace(/\\\"/g, '"')
                		: raw;
            }
        };

        /**
         * Adds or removes required field input field based on it's Conditional Display status.
         */
        Property.prototype.updateRequiredInputFields = function () {
            var isCurrentlyRequired = this.isCurrentlyRequired();
            var requiredInputFields = this.findRequireInputFields();
            var hasRequiredInputField = requiredInputFields && requiredInputFields.length > 0;

            DEBUG.custom({
                indent: 1, data: function () {
                    return "Updating Required Input Fields:\n" +
                        "    On: " + this.fullyQualifiedName + "\n" +
                        "    Currently Required: " + isCurrentlyRequired + "\n" +
                        "    hasRequiredInputField: " + hasRequiredInputField;
                }, context: this
            });

            if (!isCurrentlyRequired) {
                if (hasRequiredInputField) {
                    DEBUG.custom({indent: 1, data: "    Removing required field."});
                    requiredInputFields.remove();
                }
                return;
            }

            if (!hasRequiredInputField) {
                DEBUG.custom({indent: 1, data: "    Adding (Required) to " + this.fullyQualifiedName});
                this.containingObject.prepend(this.requiredFormDOMString);
            }
        };

        /**
         * Validates property. Checks if A) is currently required, and B) runs the custom validation method.
         * If CKEditor, it returns it to the WYSIWYG editor before validating.
         * @returns {jQuery.Deferred} Returns jQuery Promise of the validation.
         */
        Property.prototype.validate = function () {
            var property = this;
            var deferred = $.Deferred();

            property.clearError();

            if (!this.isValidationNecessary()) {
                return deferred.resolve().promise();
            }

            // Reset rich text editor
            this.validateCKEditorWYSIWYG()
                .fail(function () {
                    DEBUG.custom({
                        indent: 1,
                        data: "Failed  - CKEditor change view - Source -> WYSIWYG: " + property.fullyQualifiedName
                    });
                    deferred.reject("CKEditorSourceCheck");
                })
                .done(function () {
                    if (!property.validateRequiredField()) {
                        DEBUG.custom({
                            indent: 1,
                            data: "    Failed  - Required Field Check: " + property.fullyQualifiedName
                        });

                        var webrFormFeedbackStrings = globals.get("_webrFormFeedbackStrings");
                        var missingRequiredFieldString = property.controlType === "dataentrygrid" ?
                            webrFormFeedbackStrings.missingSubRequiredValue : webrFormFeedbackStrings.missingRequiredValue;

                        property.addError(missingRequiredFieldString);

                        if(!property.isVisible()) {
                        	var caption = property.containingObject.find("#_span_ALD_caption").text();
                            property.view.addError("<strong>(Currently Hidden)</strong> " + caption + " " + missingRequiredFieldString)
                        }

                        deferred.reject("RequiredFieldsCheck");
                    } else {
                        DEBUG.custom({
                            indent: 1,
                            data: "    Success - Required Field Check: " + property.fullyQualifiedName
                        });
                    }

                    var customMethodResult = property.validateCustomMethod();

                    // boolean variables are not instances of Object
                    // the jQuery Promise object is instance of Object
                    if (customMethodResult instanceof Object) {
                        customMethodResult
                            .done(function () {
                                deferred.resolve();
                            })
                            .fail(function (passedObject) {
                                DEBUG.custom({
                                    indent: 1,
                                    data: "    Failed  - Custom Check: " + property.fullyQualifiedName
                                });
                                property.addError(passedObject.errorMessage ? passedObject.errorMessage : "This property did not validate.");
                                deferred.reject("CustomCheck");
                            });
                    } else if (typeof customMethodResult === "boolean") {
                        if (customMethodResult) {
                            deferred.resolve();
                        } else {
                            property.addError("This property did not validate.");
                            deferred.reject("CustomCheck");
                        }
                    } else {
                        deferred.reject(Error("Unknown validateCustomMethod() Result: " + customMethodResult));
                    }

                });

            return deferred.promise();
        };

        /**
         * this should work until setValidateCustomMethod is used.
         * @abstract
         * @returns {boolean} Always returns true.
         */
        Property.prototype.validateCustomMethod = function () {
            DEBUG.custom({indent: 1, data: "    No validateCustomMethod available."});
            return true;
        };

        /**
         * @abstract
         * @type {Function}
         */
        Property.prototype.DEBUG_showValues =
            Property.prototype.clear =
                Property.prototype.focus =
                    Property.prototype.getCurrentValue =
                        Property.prototype.getPersistentValue =
                            Property.prototype.getFormValue =
                                Property.prototype.getParsedFormValue =
                                    Property.prototype.isCurrentlyRequired =
                                        Property.prototype.isDirty =
                                            Property.prototype.isEmpty =
                                                Property.prototype.validateCKEditorWYSIWYG =
                                                    Property.prototype.validateRequiredField =
                                                        function () {
                                                            throw new Error("This function must be overridden by subclass");
                                                        };


        /**
         * This property represent properties necessary on the page for Rule evaluation, but not on
         * the EntityView itself.
         * @param {Object} parameters - Parameters to be passed to ViewAsset constructor.
         * @param {View} view - EntityView this is associated to.
         * @constructor
         * @extends Property
         * @memberof module:EntityView/View
         */
        function ExternalProperty(parameters, view) {
            Property.call(this, parameters, view);
            this.currentDisplayType = _displayType.HIDDEN;
        }
        ExternalProperty.prototype = Object.create(Property.prototype);
        ExternalProperty.prototype.constructor = ExternalProperty;

        /**
         * Debugging function to display the current and persisted values of this object.
         * @returns {string} String output of current and persisted values.
         */
        ExternalProperty.prototype.DEBUG_showValues = function () {
            var parsedValue = this.getParsedPersistentValue();
             var str = "Values (" + this.type + "/" + this.controlType + "):";
            if(Array.isArray(parsedValue)){
            	str += "\nThis property is a part of an entity set with following values";
            	$(parsedValue).each(function(x){
            		str +=  "\n\npersisted string : " + JSON.stringify(x.string) + " (" + DebugUtil.getTypeName(x.string) + ")" +
                  			"\npersisted raw    : " + JSON.stringify(x.raw) + " (" + DebugUtil.getTypeName(x.raw) + ")";
                });
                return str;
            }
            var warnings = "";

            switch (this.type) {
                case "Currency": //FIXME was never updated to take in to account currency getting set properly
                case "Date":  //FIXME  need to take in to account Moment.js
                    if (!_.isEqual(this.persistedValue.string, parsedValue)) {
                        warnings += "!!! Persisted String and Parse Form values differ !!!\n";
                    }
                    break;
                default:
                    if (!_.isEqual(this.persistedValue.raw, parsedValue)) {
                        warnings += "!!! Persisted Raw and Parse Form values differ !!!\n";
                    }
            }

            // add white space to make it visible
            if (warnings.length > 0) {
                warnings = "\n" + warnings;
            }

            str += "\npersisted string : " + JSON.stringify(this.persistedValue.string) + " (" + DebugUtil.getTypeName(this.persistedValue.string) + ")" +
                  "\npersisted raw    : " + JSON.stringify(this.persistedValue.raw) + " (" + DebugUtil.getTypeName(this.persistedValue.raw) + ")";

            return str + warnings;

        };

        /**
         * Always returns the persisted value.
         * @type {Function}
         */
        ExternalProperty.prototype.getCurrentValue =
            ExternalProperty.prototype.getPersistentValue =
                function () {
                    return this.getParsedPersistentValue();
                };

        /**
         * These functions are not used by ExternalProperty, but still need to be defined.
         * @type {Function}
         */
        ExternalProperty.prototype.findContainer =
            ExternalProperty.prototype.findErrorFeedbackContainer =
                ExternalProperty.prototype.findFormDomObjects =
                    ExternalProperty.prototype.findRequiredField =
                        ExternalProperty.prototype.findOptionValues =
                            ExternalProperty.prototype.getFormValue =
                                ExternalProperty.prototype.getParsedFormValue =
                                    ExternalProperty.prototype.isRequired =
                                        function () {
                                            return undefined;
                                        };

        /**
         * These functions are not used by ExternalProperty, but still need to be defined.
         * TODO can these return undefined?
         * @type {Function}
         */
        ExternalProperty.prototype.findConditionalDisplayContent =
            ExternalProperty.prototype.getChildProperties =
                ExternalProperty.prototype.getContainingRegions =
                    ExternalProperty.prototype.findRequireInputFields =
                        ExternalProperty.prototype.generateInputFieldName =
                            ExternalProperty.prototype.setupContainerEventHandler =
                                function () {
                                    return null;
                                };

        /**
         * Always returns false for ExternalProperty
         * @type {Function}
         */
        ExternalProperty.prototype.isCurrentlyEditable =
            ExternalProperty.prototype.getCurrentPersistence =
                ExternalProperty.prototype.isCurrentlyRequired =
                    ExternalProperty.prototype.isDirty =
                        ExternalProperty.prototype.isEmpty =
                            ExternalProperty.prototype.isValidationNecessary =
                                ExternalProperty.prototype.isVisible =
                                    function () {
                                        return false;
                                    };

        /**
         * Various functionality ExternalProperties do not support.
         * @type {Function}
         */
        ExternalProperty.prototype.clear =
            ExternalProperty.prototype.changeHandler =
                ExternalProperty.prototype.displayUpdateHandler =
                    ExternalProperty.prototype.focus =
                        ExternalProperty.prototype.on =
                            ExternalProperty.prototype.setValue =
                                ExternalProperty.prototype.trigger =
                                    ExternalProperty.prototype.triggerRuleUpdate =
                                        ExternalProperty.prototype.UpdateDisplay =
                                            ExternalProperty.prototype.UpdateDisplayEvent =
                                                ExternalProperty.prototype.updateRequiredInputFields =
                                                    ExternalProperty.prototype.validateCKEditorWYSIWYG =
                                                        ExternalProperty.prototype.validateRequiredField =
                                                            function () {
                                                                throw new Error("Not Supported by ExternalProperty.");
                                                            };


        //
        //  TODO[LOW] requires testing and development
        //
        function CustomProperty(parameters, view) {
            Property.call(this, parameters, view);

            this.customFunctions = parameters.customFunctions;

            CustomProperty.setupPrototypeFunctions(
                "addError",
                "clearError",
                "focus",
                "findContainer",
                "getFormValue",
                "getParsedFormValue",
                "getParsedPersistentValue",
                "getCurrentPersistence",
                "isCurrentlyRequired",
                "isCurrentlyEditable",
                "isEmpty",
                "isDirty",
                "setupContainerEventHandler",
                "validateCKEditorWYSIWYG",
                "validateRequiredField"
            );
        }
        CustomProperty.prototype = Object.create(Property.prototype);
        CustomProperty.prototype.constructor = ExternalProperty;
        CustomProperty.abstractMethod = function (functionName) {
            if (!this.customFunctions) {
                return undefined;
            }
            if (this.customFunctions[functionName]) {
                return this.customFunctions[functionName];
            }
            return this.prototype[functionName];
        };
        CustomProperty.setupPrototypeFunctions = function () {
            _.each(arguments, function (functionName) {
                CustomProperty.prototype[functionName] = CustomProperty.abstractMethod(functionName);
            });
        };



        /**
         * //Javascript object to represent the Properties on the View
         * @param {Object} parameters - Parameters to be passed to ViewAsset constructor.
         * @param {View} view - EntityView this is associated to.
         * @constructor
         * @extends Property
         * @memberof module:EntityView/View
         */
        function InternalProperty(parameters, view) {
            Property.call(this, parameters, view);

            _.bindAll(this,
                'DEBUG_showValues'
            );
        }
        InternalProperty.prototype = Object.create(Property.prototype);
        InternalProperty.prototype.constructor = InternalProperty;

        /**
         * Debugging function to display the current and persisted values of this object.
         * @returns {string} String output of current and persisted values.
         */
            //TODO[HIGH] add getparsedpersistedvalue
        InternalProperty.prototype.DEBUG_showValues = function () {
            var parsedValue = this.getParsedFormValue();
            var formValue = this.getFormValue();
            var warnings = "";

            switch (this.type) {
                case "Currency": //FIXME was never updated to take in to account currency getting set properly
                case "Date":  //FIXME  need to take in to account Moment.js
                    if (!_.isEqual(this.persistedValue.string, parsedValue)) {
                        warnings += "!!! Persisted String and Parse Form values differ !!!\n";
                    }
                    break;
                default:
                    if (!_.isEqual(this.persistedValue.raw, parsedValue)) {
                        warnings += "!!! Persisted Raw and Parse Form values differ !!!\n";
                    }
            }

            if (this.isDirty()) {
                warnings += "!!! isDirty !!!\n";
            }

            // add white space to make it visible
            if (warnings.length > 0) {
                warnings = "\n" + warnings;
            }

            var str = "Values (" + this.type + "/" + this.controlType + "):" +
                "\npersisted string : " + JSON.stringify(this.persistedValue.string) + " (" + DebugUtil.getTypeName(this.persistedValue.string) + ")" +
                "\npersisted raw    : " + JSON.stringify(this.persistedValue.raw) + " (" + DebugUtil.getTypeName(this.persistedValue.raw) + ")" +
                "\nfound     parsed : " + JSON.stringify(parsedValue) + " (" + DebugUtil.getTypeName(parsedValue) + ")" +
                "\nfound     form   : " + JSON.stringify(formValue) + " (" + DebugUtil.getTypeName(formValue) + ")";

            return str + warnings;

        };

        /**
         * Adds error to be displayed on this property
         * @param {string} str
         */
        InternalProperty.prototype.addError = function (str) {
            if(!this.errorField || this.errorField === 0) {
                return;
            }
            this.errorField.append("<br/>" + str);
        };

        /**
         * Clears errors displayed on this property
         */
        InternalProperty.prototype.clearError = function () {
            if(!this.errorField || this.errorField === 0) {
                return;
            }
            this.errorField.html(null);
        };

        /**
         * Utility function which throws an error if form object does not have the correct number of elements.
         * @param formObject
         */
        InternalProperty.prototype.checkFormObjectLength = function (formObject) {
            if (this.readOnly || this.view.isReadOnly()) {
                // do nothing, read only has no form values
                return;
            }
            switch (this.type + "/" + this.controlType) {
                case "Set/dataEntry":
                case "Set/dataentrygrid":
                case "Set/inlinecdtview":
                case "Set/simpledocumenttable":
                case "Set/chooserSelection":
                case "Set/checkboxSelection":
                case "Entity/radio":
                case "Entity/dataEntry":
                case "Entity/dataSnapShot":
                    // Do nothing: it's ok to have no form objects for these control types
                    break;
                default:
                    if (!formObject || formObject.length === 0) {
                        throw new Error("There are no form objects for this property.");
                    }
                    break;
            }
        };

        /**
         * Clears property value. Only works for some types of properties/controlTypes
         */
        InternalProperty.prototype.clear = function () {
            if (this.view.isReadOnly()) {
                DEBUG.custom({indent: 1, data: "View is readonly, unable to clear."});
                return;
            }
            switch (this.controlType) {
                case "checkbox":
                case "radio":
                case "checkboxSelection":
                    this.formObjects.prop("checked", false);
                    break;
                case "richtext":
                    this.ckEditor.setData('').change();
                /* falls through */
                case "singleline":
                case "calendar":
                case "textarea":
                    this.formObjects.val('');
                    break;
                case "documentContentFileUploadView":
                case "simpledocument":
                case "singleDocumentUploadEditView":
                case "singleEntityDataEntry":
                case "chooserSelection":
                case "simpledocumenttable":
                case "dataEntry": //"entityTableDataEntry":
                case "inlinecdtview":
                case "dataentrygrid":
                case "select":
                case "dataSnapShot":
                    DEBUG.warn(controlType + " control type is not supported.");
                    break;
                default:
                    throw new Error("find form object control type '" + this.controlType + "' is not implemented.");
            }
            this.formObjects.change();
        };

        /**
         * @returns {jQuery} Returns a jQuery object that represents the area to conditionally display.
         */
        InternalProperty.prototype.findConditionalDisplayContent = function() {
            return this.containingObject;
        };

        /**
         * @returns {jQuery} Returns a jQuery object that represents the containing DOM region of the property.
         */
        InternalProperty.prototype.findContainer = function () {
            var containerId = '_' + this.fullyQualifiedName.replace(/\s/g, "-") + '_container';
            // this won't work since the id can contain space.
            // Yes, I know this is not what is valid, but internal attributes can have spaces,
            // so this has spaces. Sigh.
            //var selector = PortalTools.escapeJQuerySelector('#' + containerId);
            var selector = '[id="' + PortalTools.escapeJQuerySelector(containerId) + '"]';
            //this.containingObject = $(selector, viewArea); // doesn't work due to layout / HTML5 issue
            var containingObject = $(selector);

            if (containingObject.length === 1) {
                DEBUG.custom({indent: 1, data: "Found DOM container object found using selector: " + selector});
            } else if (containingObject.length > 1) {
                DEBUG.warn("Found " + containingObject.length + " DOM container object found using selector: " + selector);
            } else {
                throw new Error("No DOM container object found using selector: " + selector);
            }
            return containingObject;
        };

        /**
         * @returns {jQuery} Returns a jQuery object that represents the Error Feedback Container for this property.
         */
            // ensure all feedback containers are found
        InternalProperty.prototype.findErrorFeedbackContainer = function () {
            if(this.readOnly || this.hidden || this.controlType === "dataSnapShot") {
                return undefined;
            }
            var containerId = PortalTools.escapeJQuerySelector(this.fullyQualifiedName + '_clientSideError');
            //DEBUG.log(1, "DOM error feedback container ID is: " + containerId);
            var selector = 'span[id="' + containerId + '"]';
            //var containingObject = $(selector, viewArea); // doesn't work due to layout / HTML5 issue
            var containingObject = $(selector);

            if (containingObject.length === 1) {
                DEBUG.custom({
                    indent: 1,
                    data: "Found DOM error feedback container object found using selector: " + selector
                });
            } else if (containingObject.length > 1) {
                DEBUG.warn("Found " + containingObject.length + " DOM error feedback container object found using selector: " + selector);
            } else {
                throw new Error("No DOM error feedback container object found using selector: " + selector);
            }
            return containingObject;
        };

        /**
         * @returns {jQuery} Returns a jQuery object that represents the form object(s) for this property.
         */
        InternalProperty.prototype.findFormDomObjects = function () {
            if (this.view.isReadOnly()) {
                return null;
            }

            var selector = this.inputFieldTagType + '[name="' + this.inputFieldName + '"]';

            var obj = $(selector, this.view.formObject);

            this.checkFormObjectLength(obj);

            DEBUG.custom({
                indent: 1, data: function () {
                    return "Found form objects: " + (obj ? obj.length : 0);
                }, context: this
            });

            return obj;
        };

        /**
         * @returns {jQuery} Returns a jQuery object that represents the Required Field for this property.
         */
        InternalProperty.prototype.findRequiredField = function () {
            var requiredObject = $('span.RequiredSpan', this.containingObject);

            if (requiredObject.length === 1) {
                DEBUG.custom({indent: 1, data: "Found DOM required object found using selector."});
            } else if (requiredObject.length > 1) {
                DEBUG.warn("Found " + requiredObject.length + " DOM required object found using selector.");
            } else {
                throw new Error("No DOM required object found using selector.");
            }
            return requiredObject;
        };

        /**
         * If this property is a controlType of select, radio, or checkbox, it gets the selectable values.
         * @returns {jQuery} Returns a jQuery object representing the values of the selectable options.
         */
        InternalProperty.prototype.findOptionValues = function () {
            if (this.view.isReadOnly()) {
                return null;
            }

            if (this.type !== "Set" && this.type !== "Entity") {
                return null;
            }

            switch (this.controlType) {
                case 'select':
                    return _.compact(_.map($("option", this.formObjects[0]), function (object) {
                        var value = $(object).val();
                        return value === "" ? null : value;
                    }));

                case "radio":
                case "checkbox":
                    return _.map(this.formObjects, function (formObject) {
                        return $(formObject).val();
                    });

                default:
                    return null;
            }
        };

        /**
         * Focuses on this property
         */
        InternalProperty.prototype.focus = function () {
            switch (this.controlType) {
                case "richtext":
                    if (!CKEDITOR) {
                        //CKEDITOR isn't even a thing. If you have a richtext editor, this is BAAAAAAAAD.
                        throw new Error("CKEDITOR object does not exist.");
                    }

                    // Workaround for maintaining the correct tab order when exiting the editor via the tab key.
                    // Problem: When programmatically placing the initial focus on the editor (using its special
                    // method), the browser doesn't seem to fully recognize it and consequently takes you to the
                    // first item in the tab order (the browser's location bar) when tabbing out -- as though
                    // nothing had the focus originally.
                    // Solution: First place focus on an element that can visibly show it, then set focus on the
                    // editor. We'll use an "a" tag since we should always have one available.
                    $("a").first().focus();

                    // Now attempt to set focus on the editor straight away and check if it worked. It will if the
                    // editor is fully initialized. Unfortunately, there's no (documented) way to simply interrogate
                    // it for status.
                    var ckEditorInstance = CKEDITOR.instances[this.inputFieldName];
                    if (ckEditorInstance) {
                        ckEditorInstance.focus();
                    } else {
                        // Editor does not have focus, presumably because it's not yet initialized.
                        // Attempt to set focus via an "instanceReady" handler. Technically a race condition,
                        // but always seems to come up as a win in testing. Not crucial that focus gets set anyway.
                        CKEDITOR.on("instanceReady", function (event) {
                            ckEditorInstance = CKEDITOR.instances[this.inputFieldName];
                            if (event.editor === instance) {
                                ckEditorInstance.focus();
                            }
                        }, this);
                    }

                    break;
                default:
                    this.formObjects.first().focus();
            }
        };

        /**
         * @returns {*} Returns the raw form value for this property
         */
            // determine if this property is required. Need to check it's visibility
        InternalProperty.prototype.getFormValue = function () {
            if (this.readOnly) {
                DEBUG.warn("InternalProperty is readonly and does not have a form value.");
                return undefined;
            }

            var value;

            this.checkFormObjectLength(this.formObjects);

            switch (this.controlType) {
                case "checkbox":
                case "radio":
                    if (this.hidden) {
                        value = this.formObjects.val();
                    } else {
                        value = this.formObjects.filter(":checked").val();
                    }

                    if (value === undefined || value === "") {
                        value = null;
                    }
                    break;
                case "select":
                    value = this.formObjects.val();
                    if (value === "") {
                        value = null;
                    }
                    break;
                case "checkboxSelection":
                    value = [];
                    this.formObjects.filter(":checked").each(function () {
                        value.push($(this).val());
                    });
                    if (value.length === 0) {
                        value = [];
                    }
                    break;
                case "simpledocumenttable":
                    value = [];
                    this.formObjects.each(function () {
                        value.push($(this).val());
                    });
                    if (value.length === 0) {
                        value = [];
                    }

                    //
                    // TODO[MEDIUM] this is hackish. This code does not belong here. Need to find out where to better place this.
                    //
                    //Let's get the files to be uploaded
                    //var uploadItemAreas = $('#' + this.fullyQualifiedName + "_newUploadDiv tr"); // no idea why this isn't working
                    var uploadItemAreas = $('div[id="' + this.fullyQualifiedName + '_newUploadDiv"] tr', this.containingObject);

                    //var uploadItems = [];

                    uploadItemAreas.each(function () {
                        var item = $(this);
                        var filename = $('input[name$=".targetURL"]', item).val();
                        var description = $('input[name$="_docDesc"]', item).val();

                        if(description === "type file description") {
                            description = null;
                        }

                        if (filename || (description && description !== "type file description")) {
                            //TODO[HIGH], use a better object than POJSO (plain old javascript object)
                            //TODO[HIGH] the names we use should better reflect the document entity object
                            //uploadItems.push({
                            value.push({
                                filename: filename,
                                description: description
                            });
                        }
                    });



                    break;
                case 'dataentrygrid':
                    value = [];
                    this.formObjects.each(function () {
                        value.push($(this).val());
                    });
                    if (value.length === 0) {
                        value = [];
                    }
                    var aldOID = this.poref;
                    var encodedald = aldOID.replace(/(\.|\[|\])/g, "\\$1");
                    var newRowAreas =$("table[id^='"+encodedald+"_DataTable'] tr.ALDNewSetItemRow");
                    newRowAreas.each(function () {
                        var item = $(this);
                        var inputItem = $('input:first', item);
                        var prefixName = inputItem.attr("name");
                        value.push(prefixName);
                    });
                    break;
                case "dataEntry":
                case "inlinecdtview":
                case "chooserSelection":
                    switch (this.type) {
                        case "Set":
                            value = [];
                            this.formObjects.each(function () {
                                value.push($(this).val());
                            });
                            if (value.length === 0) {
                                value = []; //TODO change to null?
                            }
                            break;
                        case "Entity":
                            value = this.formObjects.val();
                            if(value === "") {
                                value = null;
                            }
                            break;
                        default:
                            throw new Error("getFormValue from control type '" + this.controlType + "' and type '" + this.type + "' is not implemented.");
                    }
                    break;
                case "documentContentFileUploadView":
                case "simpledocument":
                case "calendar":
                case "singleline":
                case "singleEntityDataEntry":
                case "singleEntitySelection":
                case "singleDocumentUploadEditView":
                case "documentTableUploadEditview":
                case "textarea":
                    value = this.formObjects.val();
                    break;
                case "richtext":
                    value = CKEDITOR.instances[this.inputFieldName].getData();
                    break;
                case "dataSnapShot":
                    DEBUG.warn(this.controlType + " control type is not supported.");
                    break;
                default:
                    throw new Error("getFormValue from control type '" + this.controlType + "' is not implemented.");
            }

            return value;
        };

        /**
         * @returns {*} JavaScript value the form Object
         */
        InternalProperty.prototype.getParsedFormValue = function () {
            if (this.readOnly) {
                return undefined;
            }

            //TODO[LOW] this is not the ideal way of doing things
            return parseFormValue(this.getFormValue(), {
                controlType        : this.controlType,
                hidden             : this.hidden,
                type               : this.type,
                format             : this.format,
                dateFormat         : this.dateFormat,
                currencyFormat     : this.currencyFormat,
                fullyQualifiedName : this.fullyQualifiedName,
                containingObject   : this.containingObject
            });
        };

        /**
         * @returns {*} Returns the current value. Takes in to account all the things. If you were to save the
         * form, this is the value you save.
         */
        InternalProperty.prototype.getCurrentValue = function () {
            if(this.readOnly) { //TODO isCurrentlyReadOnly
                return this.getParsedPersistentValue();
            }

            if(!this.isEditPossible) {
                return this.getParsedFormValue();
            }

            // Editable but not currently editable
            if (!this.isCurrentlyEditable() ) {
                switch(this.persistanceWhenNonEditable) {
                    // When hidden, on post the data will ________, so we assume the same here.
                    case _persistenceType.CLEAR:
                        return null;
                    case _persistenceType.IGNORE:
                        return this.getParsedPersistentValue();
                    case _persistenceType.SAVE:
                        return this.getParsedFormValue();
                }
            }

            return this.getParsedFormValue();
        };

        /**
         * @returns {_persistenceType} The current Persistence type.
         */
        InternalProperty.prototype.getCurrentPersistence = function () {
            if(this.readOnly) { //TODO isCurrentlyReadOnly
                return _persistenceType.IGNORE;
            }

            return ViewAsset.prototype.getCurrentPersistence.call(this);
        };

        /**
         * @returns {*} Returns the JavaScript value of the persistence value.
         */
        InternalProperty.prototype.getPersistentValue = function () {
            return this.getParsedPersistentValue();
        };

        /**
         * @returns {boolean} Returns true if the property is required.
         */
        InternalProperty.prototype.isRequired = function () {
            if(this.controlType === "dataentrygrid") {
                var isRequiredObjects = $("input[name^='_webrRequired_']", this.containingObject);
                var isAnySubFieldRequired = isRequiredObjects.length > 0;
                if(isAnySubFieldRequired){
                    DEBUG.custom({indent: 2, data: "Property has required sub fields."});
                    return true;
                }
                else{
                    DEBUG.custom({indent: 2, data: "Property has no required sub fields."});
                }
            }

            DEBUG.custom({indent: 2, data: function (){
                return this.required ? "Property has required flag.":"Property has no required flag.";

            }});
            return this.required;
        };

        /**
         * This function takes in to account the Conditional Display settings.
         * @returns {boolean} Returns true if the property is currently required.
         */
        InternalProperty.prototype.isCurrentlyRequired = function () {
            if(!this.isRequired()) {
                DEBUG.custom({indent: 2, data: "Property is not required."});
                return false;
            }
            DEBUG.custom({indent: 2, data: "Property is required."});

            if(this.isCurrentlyEditable()) {
                DEBUG.custom({indent: 2, data: "Property is editable."});
                return true;
            }
            DEBUG.custom({indent: 2, data: "Property is not editable."});

            // so if it is not editable and...
            if(this.validateWhenNonEditable) {
                DEBUG.custom({indent: 2, data: "Property set to validate when non-editable"});
                return true;
            }
            DEBUG.custom({indent: 2, data: "Property set to validate only when editable"});

            return false;
        };

        /**
         * @returns {boolean} Returns true if the property value has changed.
         */
        InternalProperty.prototype.isDirty = function () {
            if (this.readOnly || !this.cache.hasViewAssetChanged) {
                return false;
            }

            var currentValue = this.getCurrentValue();
            var currentValueIsArray = _.isArray(currentValue);

            var persistedValue = this.getParsedPersistentValue();
            var persistedValueIsArray = _.isArray(persistedValue);

            if(currentValueIsArray && !currentValue.length) {
                currentValue = null;
            }
            if(persistedValueIsArray && !persistedValue.length) {
                persistedValue = null;
            }

            switch (this.controlType) {
                case "calendar":
                    return !isSameMoment(persistedValue, currentValue);
                case "checkbox":
                    // force boolean. false and null are both acceptable here as the same value.
                    persistedValue = !!persistedValue;
                    currentValue = !!currentValue;
                /* falls-through */
                default:
                    if(currentValueIsArray && persistedValueIsArray) {
                        // lets get the cheap comparisons out of the way...
                        // * check if they are the same array. (Doubt this will ever happen though)
                        // * check if both are null or empty (empty arrays were made null above)
                        if(currentValue === persistedValue) {
                            return false;
                        }

                        // this checks to see if the array values are the same regardless of order.
                        return !(_.isEmpty(_.difference(currentValue, persistedValue))
                        && _.isEmpty(_.difference(persistedValue, currentValue)));
                    }

                    return !_.isEqual(persistedValue, currentValue);
            }
        };

        /**
         * @returns {boolean} Returns true if property has not been filled out.
         */
        InternalProperty.prototype.isEmpty = function () {
            return isValueEmptyForControlType(this.getCurrentValue(), this.controlType);
        };

        /**
         * @returns {boolean} Returns true if validation is necessary.
         */
        InternalProperty.prototype.isValidationNecessary = function () {
            DEBUG.custom({indent: 1, data: "testing if validation is necessary for: " + this.fullyQualifiedName});

            // It's not possible to edit this property, so no reason to validate it.
            if (!this.isEditPossible) {
                DEBUG.custom({indent: 2, data: "NO VALIDATION: Editing not possible."});
                return false;
            }
            DEBUG.custom({indent: 2, data: "Editing is possible."});

            if (this.isCurrentlyEditable()) {
                DEBUG.custom({indent: 2, data: "VALIDATE: Editing is enabled"});
                return true;
            }
            DEBUG.custom({indent: 2, data: "Editing is not enabled."});

            if(this.validateWhenNonEditable) {
                DEBUG.custom({indent: 2, data: "VALIDATE: Property set to validate when not editable."});
                return true;
            }

            switch(this.persistanceWhenNonEditable) {
                // When hidden, on post the data will ________, so we assume the same here.
                case _persistenceType.IGNORE:
                case _persistenceType.CLEAR:
                    DEBUG.custom({indent: 2, data: "NO VALIDATION: On post, property will: " + this.persistanceWhenNonEditable});
                    return false;
                case _persistenceType.SAVE:
                    DEBUG.custom({indent: 2, data: "VALIDATE: On post, property will: " + this.persistanceWhenNonEditable});
                    return true;
            }

            DEBUG.custom({indent: 2, data: "VALIDATE"});
            return true;
        };

        /**
         * Attaches changeHandler to formObjects on this Property.
         * TODO[LOW] should probably abstract out the type of event
         */
        InternalProperty.prototype.setupContainerEventHandler = function () {
            DEBUG.custom({
                indent: 1,
                data: "Adding Event Handler: [InternalProperty:" + this.fullyQualifiedName + ", Type:Input Update]"
            });
            if (this.readOnly || !this.formObjects) {
                return;
            }
            switch (this.controlType) {
                case "simpledocumenttable":
                    //var newUploadDiv = $('div[id="' + this.fullyQualifiedName + '_newUploadDiv"]');
                    //newUploadDiv.on("change input", this.changeHandler);
                    this.containingObject.on("change input", this.changeHandler);
                /* falls through */
                case "singleEntityDataEntry":
                case "singleDocumentUploadEditView":
                case "dataEntry": //"entityTableDataEntry":
                case "inlinecdtview":
                case "singleEntitySelection":
                case "chooserSelection":
                case "dataentrygrid":
                    // Sadly we need to set this up elsewhere since these input types send events to <body> and not itself.
                    this.view.messageCenter.addBodyTrigger(this);
                    break;
                case "richtext":
                    // http://docs.ckeditor.com/#!/api/CKEDITOR.editor-event-change
                    // http://outsidemybox.org/development/towards-an-exhaustive-list-of-events-for-ckeditor/

                    CKEDITOR.on('instanceReady', function (evt) {
                    	// Bind CKEditor handlers for the current field only.
                    	var editor = evt.editor;
                        var formObjectsName = this.formObjects.attr("name");
                        if (evt.editor.name !== formObjectsName)
                        	// Not the editor corresponding to this field.
                        	return;
                      
                        editor.on("change", this.changeHandler);
                        // Setup change detection when changing to mode 'source'
                        // must setup after event 'mode' since the textarea does not exist until that happens.
                        editor.on('mode', function (event) {
                            var property = event.listenerData;
                            if (this.mode === 'source') {
                                var editable = editor.editable();
                                editable.attachListener(editable, 'input', property.changeHandler);
                            }
                        }, editor, this);
                    }, this);

                    break;
                case "dataSnapShot":
                    DEBUG.custom({indent: 1, data: "dataSnapShot is read only. No event setup necessary."});
                    break;
                case "checkbox":
                case "radio":
                case "checkboxSelection":
                case "calendar":
                case "select":
                    this.$event.on("change input", this.changeHandler);
                    break;
                //case "textarea":
                //case "singleline":
                //case "documentContentFileUploadView":
                //case "simpledocument":
                default:
                    // As of summer 2015...
                    // Event types:
                    //     onchange occurs only when you blur the textbox
                    //     onkeyup & onkeypress doesn't always occur on text change
                    //     onkeydown occurs on text change (but cannot track cut & paste with mouse click)
                    //     onpaste & oncut occurs with keypress and even with the mouse right click.
                    //   So, to track the change in textbox, we need onkeydown, oncut and onpaste
                    //   oninput and textInput are the events which is the actual solution for this but,
                    //   its not supported by all the browsers
                    //     oninput - starting ie9
                    //     textInput - starting ie9, not available in ff
                    //     ie9 bug: does not detect backspaces
                    //   Since there is no best built in solution, we had to use a jQuery plugin: jquery.textchange

					// This is a hack. Other parts of the code rely on triggering $event.change,
					// However, textchange apparently won't bubble up the DOM. :(
                    var eventObject = this.$event;
                    this.formObjects.on("textchange", function() {
                        eventObject.change();
                    });
                    
                    this.$event.on("change", this.changeHandler);
                    break;
            }
        };

        /**
         * Ensure that all rich text fields are ready for submission.
         * A CKEditor that is still in source mode will return a "raw" value
         * for the form submission. The subsequent rendering in wysiwyg mode
         * can then trigger a "false change" by merely reformatting the HTML
         * or converting certain characters to HTML entities. Therefore, any
         * editors still in source mode first need to be switched to wysiwyg
         * mode (an asynchronous operation) in order to prevent false positives
         * in the diffs.
         * @returns {jQuery.Promise} Returns a Promise of the CKEditor validation
         */
        InternalProperty.prototype.validateCKEditorWYSIWYG = function () {
            var deferred = $.Deferred();
            if (this.controlType === "richtext") {
                var ckEditorInstance = CKEDITOR.instances[this.inputFieldName];
                if (ckEditorInstance.mode === "source") {
                    ckEditorInstance.setMode("wysiwyg", function () {
                        deferred.resolve();
                    });
                } else {
                    deferred.resolve();
                }
            } else {
                deferred.resolve();
            }
            return deferred.promise();
        };

        /**
         * @returns {boolean} Returns false if property is currently required and value is empty. Otherwise returns true.
         */
            //TODO make function name more clear
        InternalProperty.prototype.validateRequiredField = function () {
            switch(this.controlType) {
                case "dataentrygrid":
                    var isRequiredObjects = $("input[name^='_webrRequired_']", this.containingObject);
                    var isAnySubFieldRequired = isRequiredObjects.length > 0;

                    var isAnyEmpty;
                    if(isAnySubFieldRequired){
                        $(isRequiredObjects).each(function(){
                            if(isAnyEmpty === undefined){
                                var fieldName = $(this).attr('name');
                                if(!fieldName){
                                    return;
                                }
                                fieldName = fieldName.substring("_webrRequired_".length);
                                var val = $("[name='"+fieldName+"']",this.containingObject).val();
                                if(!val || (val && (val === nullOIDString || val === ""))) isAnyEmpty = true;
                            }
                        });
                    }
                    if(isAnySubFieldRequired && isAnyEmpty){
                        return false;
                    }
                /* falls-through */
                default:
                    return !(this.isCurrentlyRequired() && this.isEmpty());
            }
        };


        /**
         * Object to represent an EntityView Rule.
         * @param {Object} parameters - Parameters used to initialize
         * @param {View} view - EntityView this is associated to.
         * @constructor
         * @memberof module:EntityView/View
         */
        function Rule(parameters, view) {
            if (!parameters) {
                throw new Error("'parameters' was not defined. Rules require parameters.");
            }
            if (!parameters.name) {
                throw new Error("'name' was not defined. Rules require name.");
            }
            if (!parameters.internalName) {
                throw new Error("'name' was not defined. Rules require name.");
            }
            if (parameters.currentRuleResult === null || parameters.currentRuleResult === undefined) {
                throw new Error("'currentResult' was not defined. Rules require currentResult.");
            }
            if (!parameters.logic) {
                throw new Error("'logic' was not defined. Rules require logic.");
            }
            if (!parameters.triggerNames) {
                throw new Error("'triggerNames' was not defined. Rules require triggerNames.");
            }

            _.bindAll(this,
                'getProperties',
                'executeLogic',
                'setUpdateDisplayTrigger',
                'setUpdateLogicTriggerFromProperty'
            );

            DEBUG.log("Create Rule: " + parameters.name + "(" + parameters.internalName + ")" );

            this.view = view;

            // Whenever the rule updates, this value is also updated.
            // This enables the ability to cache function return values.
            this.view.cache.rule = _.now();

            this.name              = parameters.name;              // String
            this.internalName      = parameters.internalName;      // String
            this.eventName         = encodeURI(this.internalName + defaultOptions.ruleNamespace);
            this.currentRuleResult = parameters.currentRuleResult; // Boolean
            this.triggerNames      = parameters.triggerNames;      // Array of Strings - contains fully qualified names
            this.triggerProperties = [];                           // Array of Properties

            this.ruleEvaluator = new RuleEvaluator(parameters.logic, this.view);

            //
            //  Find all regions and properties that rely on this rule and add event listeners to those
            //  for this event.
            //
            var viewAssetsRelyingOnThisRule = this.attachUpdateDisplayTrigger();

            DEBUG.custom({indent: 1, data: "Properties/Regions relying on this rule:"});
            _.each(viewAssetsRelyingOnThisRule, function (viewAsset) {
                DEBUG.custom({indent: 2, data: viewAsset.fullyQualifiedName});
            });

            //
            //  Add event listener to all properties listed as a trigger.
            //  listeners are setup to execute rule logic.
            //
            this.triggerProperties = _.compact(_.map(parameters.triggerNames, function (name) {
                return view.getProperty(name.fullyQualifiedName);
            }));

            faultTolerantDomReady(function () {
                DEBUG.log("domReady(Rule - " + this.name + ":" + this.internalName + "):");
                _.each(this.triggerProperties, this.setUpdateLogicTriggerFromProperty);
            }, this);

            DEBUG.custom({indent: 1, data: "Triggers for this rule:"});
            _.each(this.triggerProperties, function (viewAsset) {
                DEBUG.custom({indent: 1, data: "    " + viewAsset.fullyQualifiedName});
            });

            if (viewAssetsRelyingOnThisRule.length === 0 &&
                this.triggerProperties === null) {
                throw new Error("No trigger properties, listening properties, or listening regions found.");
            }

        }
		
		/*
		* Attaches UpdateDisplay event handler to all the properties and regions relying on this rule
		*/
		Rule.prototype.attachUpdateDisplayTrigger = function(){
			var ruleControlledAssets = _.where(this.view.getViewAssets(), {ruleName: this.internalName});
            _.each(ruleControlledAssets, this.setUpdateDisplayTrigger);
            return ruleControlledAssets;
		}
		
        /**
         * //FIXME split between event handler and executeLogic function
         * Executes the rule logic. If result has changed, then the rule update event is triggered
         * @param {Event} event
         * @param {Object} data
         */
        Rule.prototype.executeLogic = function (event, data) {
            if (event) {
                DEBUG.custom({
                    type: "event", context: this, data: function () {
                        // only properties can cause events.
                        // these events are run from the container DOM object

                        //var property = this.view.getProperty(event.target); // ie8 problem
                        var fullyQualifiedName = convertContainerIdToFQN(event.target.id);
                        var property = this.view.getProperty(fullyQualifiedName);

                        return "SAW EVENT: (Rule.prototype.executeLogic)\n" +
                            "On: " + (property ? property.fullyQualifiedName : "not a property") + "\n" +
                            "Type: " + event.type + "\n" +
                            "Listener: " + this.name + ":" + this.internalName;
                    }
                });
            }

            DEBUG.custom({
                indent: 1, context: this, data: function () {
                    return "executeLogic on Rule: " + this.name + ":" + this.internalName;
                }
            });

            var ruleResult = this.ruleEvaluator.evaluate(2);
            var hasRuleChanged = ruleResult !== this.currentRuleResult;

            DEBUG.custom({
                indent: 2, context: this, data: function () {
                    return "Result (old->new): " + this.currentRuleResult + " -> " + ruleResult +
                        "\n" + (hasRuleChanged ? "!!! Rule change detected !!!" : "No rule change detected.");
                }
            });


            // We don't want to be making any changes if there has been no changes in the ruleName.
            if (!hasRuleChanged) {
                return;
            }

            this.currentRuleResult = ruleResult;

            // Rule result changed, so let the listeners know they need to update.
            if (typeof data === "object") {
                data.ruleResult = ruleResult;
            } else {
                data = {ruleResult: ruleResult};
            }

            DEBUG.custom({
                type: "event", data: function () {
                    return "    TRIGGERING EVENT (Rule.prototype.executeLogic):\n" +
                        "    Type: " + this.eventName + "\n" +
                        "    On: messageCenter";
                }, context: this
            });
            this.view.messageCenter.messageObject.trigger(this.eventName, data);
        };

        /**
         * TODO[LOW] rename getTriggerProperties
         * @returns {Array.<ViewAsset>} properties that trigger this rule.
         */
        Rule.prototype.getProperties = function () {
            return this.triggerProperties;
        };

        /**
         * Registers updateDisplayEvent handler on the ViewAsset
         * @param viewAsset
         */
        Rule.prototype.setUpdateDisplayTrigger = function (viewAsset) {
        	DEBUG.custom({
                indent: 1, context: this, data: function () {
                    return "Adding Event Handler:" +
                        "\nEvent    [Rule:" + this.name + ":" + this.internalName + ", Type:Logic Update] " + this.eventName +
                        "\nListener [" + DebugUtil.getTypeName(viewAsset) + ":" + viewAsset.fullyQualifiedName + ", Executes:Display Update]";
                }
            });

            this.view.messageCenter.messageObject.on(this.eventName, viewAsset.updateDisplayEvent);
        };

        /**
         * Sets up event listener on Property. When ruleUpdate event is triggered, execute the logic.
         * @param {ViewAsset} property - Property to setup event listener on.
         */
        Rule.prototype.setUpdateLogicTriggerFromProperty = function (property) {
        	// Filter out properties that don't need triggers
            if (property instanceof ExternalProperty) {
                DEBUG.custom({
                    indent: 1,
                    data: "ExternalProperty (" + property.fullyQualifiedName + ") was specified as a trigger, (no trigger was setup)."
                });
                return;
            }
         
            DEBUG.custom({
                indent: 1, data: function () {
                    return "Adding Event Handler:" +
                        "\nEvent    [" + DebugUtil.getTypeName(property) + ":" + property.fullyQualifiedName + ", Type:Input Update]," +
                        "\nListener [Rule:" + this.name + ":" + this.internalName + ", Executes:Logic Update]";
                }, context: this
            });

            // When input changes on property, execute rule logic
            property.$event.on("ruleUpdate", this.executeLogic);
        };

        /**
         * This object is used to evaluate rules. You pass it the logic and the view at the beginning and
         * it will evaluate the logic whenever 'evaluate' is called.
         * TODO[MEDIUM] there is some optimizations that can be done. Pre-compile some parts instead of building on the fly, etc.
         * @param {Object} logic
         * @param view
         * @constructor
         * @memberof module:EntityView/View
         */
        function RuleEvaluator(logic, view) {
            _.bindAll(this,
                'parseComparisonValue',
                'parsePropertyValue',
                'evaluateConditional',
                'evaluateGroup',
                'isShortCircuitable',
                'evaluate'
            );

            this.view = view;

            this.logic = logic;
            this.shortCircuit = true; // true means we short circuit

            // The following two are used for debugging messages only.
            this.wasShortCircuited = false;
            this.currentIndent = 0;
        }

        /**
         * Executes logic.
         * @param {number} indent - Number of indents to place before log
         * @returns {boolean} result of rule
         */
        RuleEvaluator.prototype.evaluate = function (indent) {
            this.currentIndent = indent || 0;
            this.wasShortCircuited = false; // reset shortCircuit
            var returnValue = this.evaluateGroup(this.logic);
            this.currentIndent = 0; // reset just in case
            return returnValue;
        };

        /**
         * Determines and returns the value associated with the logic to compare against.
         * @param property
         * @param value
         * @param comparator
         * @returns {*}
         */
        RuleEvaluator.prototype.parseComparisonValue = function (property, value, comparator) {
            //SPECIAL CASE
            if (property.eType === 'Person' && value === '$currentuser$') {
                return PortalTools.currentUser;
            }

            if (value === null || value === undefined) {
                return value;
            }

            switch (property.type) {
                case 'Entity':
                case 'Set':
                    //SPECIAL CASE
                    if (comparator === 'in') {
                        return property.findOptionValues();
                    }
                /* falls through */
                case 'String':
                    if (comparator === "like") {
                        var newValue = value;
                        // 'someString' -> someString
                        if (newValue.length > 0) {
                            if (newValue.charAt(0) == "'" && newValue.charAt(newValue.length - 1) == "'") {
                                newValue = newValue.substr(1, newValue.length - 2);
                            }
                        }

                        newValue = newValue
                            .replace("_", ".")   // replace _ with .
                            .replace("%", ".*"); // replace % with .*

                        return new RegExp("^" + newValue + "$", "i");
                    }
                    return value;
                case 'Integer':
                    return parseInt(value, 10); // ensure base 10
                case 'Float':
                case 'Double':
                case 'Currency':
                    return parseFloat(value);
                case 'Boolean':
                    switch (value.toLowerCase()) {
                        case 'true' :
                            return true;
                        case 'false':
                            return false;
                        default:
                            return null;
                    }
                case 'Date':
                    // we compare the date numerical representation, not the date object
                    return getMoment(value, property.format, property.dateFormat).valueOf();
                case 'Enumeration':
                    return value;
                case 'DocumentContent': // Custom Search does not allow comparisons to DocumentContent
                default:
                    throw new Error("Unknown type: " + property.type);
            }
        };

        /**
         * Gathers the current value of the property and returns value to use in comparison.
         * @param property
         * @returns {*}
         */
        RuleEvaluator.prototype.parsePropertyValue = function (property) {
            var result = property.getCurrentValue();

            // this is when multiple things are selected (checkbox)
            // for this function, when there is no option selected, it is considered null
            if (result instanceof Array && result.length === 0) {
                return null;
            }

            if (property.type === "Date" && result) {
                return (result.length == 1) ? result.valueOf() : _.map(result, function(x){ return x ? x.valueOf() : x });
            }

            return result;
        };

        /**
         * Determines if the logic was Short Circuited (eg: no more rule evaluation is necessary)
         * @param previousResult
         * @param connector
         * @returns {boolean}
         */
        RuleEvaluator.prototype.isShortCircuitable = function (previousResult, connector) {
            if (DEBUG.isLoggingOn()
                && !this.wasShortCircuited
                && previousResult === false
                && connector.toLowerCase() === "and"
            ) {
                this.wasShortCircuited = true;
                DEBUG.custom({
                    indent: this.currentIndent + 1,
                    divider: '|',
                    context: this,
                    data: function () {
                        return "SHORT CIRCUITED" + (this.shortCircuit ? "" : " (disabled)");
                    }
                });
            }

            return this.shortCircuit && previousResult === false && connector.toLowerCase() === "and";
        };

        /**
         * Evaluates logic conditional
         * @param item
         * @returns {boolean}
         */
        RuleEvaluator.prototype.evaluateConditional = function (item) {
            switch (item.type) {
                case 'DocumentContent': // not supported by branch criteria
                    throw new Error("Unsupported type: " + item.type);
            }

            switch (item.comparator) {
                case '<' :
                case '<=':
                case '>=':
                case '>' :
                case 'in':
                    if (item.currentValue == null || item.comparisonValue == null) {
                        return false;
                    }
                    break;
                case '=':
                    if (
                        (item.type === 'Set' || item.type === 'Entity') &&
                        (item.currentValue == null || item.comparisonValue == null)
                    ) {
                        return false;
                    }
            }
            
			var hasMultipleCurrentValues = Array.isArray(item.currentValue);//True if this is an external property and dereferenced from an entityset 
			/*
			* When dealing with savedsearches on entity set deferenced attributes, the server side logic looks
			* for ANY of the values satisfying the condition. The same logic has to be adapted over here as well.
			*  _.some() in underscorejs is equivalent to C# Linq method Any()
			*/
            switch (item.comparator) {
                case '<' : 			
                	return (!hasMultipleCurrentValues)
                		? item.currentValue < item.comparisonValue
                		: _.some(item.currentValue, function(x) { return x < item.comparisonValue; });
                case '<=':          
                	return (!hasMultipleCurrentValues)
                		? item.currentValue <=  item.comparisonValue 
                		: _.some(item.currentValue, function(x) { return x <= item.comparisonValue; });
                case '!=':    
                	return (!hasMultipleCurrentValues) 
                		? item.currentValue !== item.comparisonValue
                		: (!_.contains(item.currentValue, item.comparisonValue));
                case '>=':          
                	return (!hasMultipleCurrentValues) 
                		? item.currentValue >=  item.comparisonValue 
                		: _.some(item.currentValue, function(x) { return x >= item.comparisonValue; });
                case '>' :
                	return (!hasMultipleCurrentValues) 
                		? item.currentValue >  item.comparisonValue 
                		: _.some(item.currentValue, function(x) { return x >= item.comparisonValue; });
                case 'is null':     return item.currentValue === null;
                case 'is not null': return item.currentValue !== null;

                case '=' :
                    if (item.type === 'Set' || hasMultipleCurrentValues) {
                        // in this case propertyValue is an array of selected values
                        return _.contains(item.currentValue, item.comparisonValue)
                    }
                    return item.currentValue === item.comparisonValue;

                // in this case, parsedValue is list of POREFs in set
                case 'in':
                    return _.contains(item.comparisonValue, item.currentValue);

                // always string. in this case item.comparisonValue is a new RegExp()
                case 'like':
                    return (!hasMultipleCurrentValues)
                    	? item.comparisonValue.test(item.currentValue)
                    	: _.some(item.currentValue, function(x) { return item.comparisonValue.test(x); });

                default:
                    throw new Error("Unknown comparator: " + comparator);
            }
        };

        /**
         * Evaluates Logic Grouping
         * @param groupArray
         * @returns {boolean}
         */
        RuleEvaluator.prototype.evaluateGroup = function (groupArray) {
            if (!(groupArray instanceof Array)) {
                throw new Error("groupArray must be an Array.");
            }

            DEBUG.custom({
                indent: this.currentIndent,
                divider: '|',
                data: "Evaluating Group (" + groupArray.length + " items)"
            });


            var logicItem,
                currentResult,
                previousResult,
                evaluationResult,
                connector,
                property;

            for (var i = 0; i < groupArray.length; i++) {
                if (DEBUG.isLoggingOn()) {
                    previousResult = currentResult;
                }

                logicItem = groupArray[i];
                connector = logicItem.Connector;

                if (this.isShortCircuitable(previousResult, logicItem.Connector)) {
                    return;
                }

                if (logicItem.Group) {
                    this.currentIndent++;
                    evaluationResult = this.evaluateGroup(logicItem.Group);
                    this.currentIndent--;
                }
                else {
                    property = this.view.getProperty(logicItem.eType + "." + logicItem.QualifiedName);
                    evaluationResult = this.evaluateConditional({
                        comparator: logicItem.Comparator,
                        type: property.type,
                        currentValue: this.parsePropertyValue(property),
                        comparisonValue: this.parseComparisonValue(property, logicItem.Value, logicItem.Comparator)
                    });
                }

                if (currentResult === undefined) {
                    currentResult = evaluationResult;
                }

                if (connector) {
                    switch (connector.toLowerCase()) {
                        case 'and':
                            currentResult = currentResult && evaluationResult;
                            break;
                        case 'or' :
                            currentResult = currentResult || evaluationResult;
                            break;
                        default:
                            throw new Error("Unknown comparator: " + connector);
                    }
                }

                //TODO[MEDIUM] this needs to be cleaned up.
                DEBUG.custom({
                    indent: this.currentIndent + 1,
                    context: this,
                    divider: '|',
                    data: function () {
                        var previousResultString = previousResult ? "true " : "false";
                        var evaluationResultString = evaluationResult ? "true " : "false";
                        var currentResultString = currentResult ? "true " : "false";

                        var connectorString;
                        if (logicItem.Connector) {
                            switch (logicItem.Connector) {
                                case 'and':
                                    connectorString = "AND";
                                    break;
                                case 'or' :
                                    connectorString = "OR ";
                                    break;
                            }
                        }
                        else {
                            connectorString = "";
                        }


                        if (logicItem.Group) {
                            if (previousResult === undefined) {
                                return "(initial) [" + evaluationResultString + "] -> " + currentResultString +
                                    " | Group Result";
                            }

                            return previousResultString + " " + connectorString + " [" + evaluationResultString + "] -> " + currentResultString +
                                " | Group Result";

                        }


                        var fullyQualifiedName = logicItem.eType + "." + logicItem.QualifiedName;

                        var comparisonValue = logicItem.Value;
                        var comparator = logicItem.Comparator;

                        var parsedPropertyValue = this.parsePropertyValue(property);
                        var parsedComparisonValue = this.parseComparisonValue(property, comparisonValue, comparator);

                        if (parsedComparisonValue instanceof RegExp) {
                            parsedComparisonValue = parsedComparisonValue.source;
                        }


                        var comparisonString;

                        switch (comparator) {
                            case 'is null':
                            case 'is not null':
                                comparisonString = JSON.stringify(parsedPropertyValue) + " " + comparator;
                                break;
                            default:
                                comparisonString = JSON.stringify(parsedPropertyValue) + " " + comparator + " " + JSON.stringify(parsedComparisonValue) + " (" + comparisonValue + ")";
                        }

                        if (previousResult === undefined) {
                            // rule.currentRuleResult
                            return "(initial) [" + evaluationResultString + "] -> " + currentResultString
                                + " | " + comparisonString + " (" + property.type + ":" + fullyQualifiedName + ")";
                        }


                        return previousResultString + " " + connectorString + " [" + evaluationResultString + "] -> " + currentResultString +
                            " | " + comparisonString + " (" + property.type + ":" + fullyQualifiedName + ") ";

                    }
                });

            }

            return currentResult;

        };

        /**
         * This enables things access to the Types in here
         * @type {{View: View, Region: Region, Property: Property, InternalProperty: InternalProperty, ExternalProperty: ExternalProperty, Rule: Rule}}
         */
        //View.types = {
        //    View : View,
        //    Region : Region,
        //    Property : Property,
        //    InternalProperty : InternalProperty,
        //    ExternalProperty : ExternalProperty,
        //    Rule : Rule
        //};

        var metadataBucket = (function() {
            var bucketArray = [];
            return {
                add: function (bucket, metadata) {
                    if (!bucketArray[bucket]) {
                        bucketArray[bucket] = [];
                    }
                    bucketArray[bucket].push(metadata);
                },
                get: function (bucket) {
                    return bucketArray[bucket];
                }
            };
        })();

        View.addChildViewMetadata = function(RootViewPOREF, metadata) {
            metadataBucket.add(RootViewPOREF, metadata)
        };

        /**
         * @param id
         * @returns {View} Returns the View instance based on the viewId.
         */
        View.get = function(id) {
            if(viewInstances.length === 0) {
                return undefined;
            }

            if(id) {
                return _.find(viewInstances, function(view) {
                    return id === view.getViewId();
                });
            }

            if(viewInstances.length > 1) {
                console.log("There is more than one view instance, grabbing instance: " + viewInstances[0].getViewId());
            }
            return viewInstances[0];
        };
        
        View.removeById = function(id){
        	if(!id){
        		return;
        	}
        	var view = _.find(viewInstances, function(v){
        		return id === v.getViewId();
        	});
        	
        	View.remove(view);
        }
        
        View.remove = function(view) {
        	if(!view){
        		return;
        	}
        	viewInstances = _.without(viewInstances, view);
        }

        /**
         * Adds a callback handler that executes once the view is ready.
         * @param viewId
         * @param callbackFunction
         */
        View.ready = function(viewId, callbackFunction) {
            globalReadyFunction(callbackFunction, viewId);
        };

//alert("View B");
        //root.PCH = View; //for debugging in ie8
        return View;

    });

